#ifndef __AUBE_H__
#define __AUBE_H__


typedef unsigned char  bit8;
typedef short bit16;
typedef int   bit32;
typedef long  bit64;


struct xm_channel_note {
   bit8 note;
   bit8 instrument;
   bit8 volume;
   bit8 effect_type;
   bit8 effect_parameter;
};

struct xm_channel_note_list {
   struct xm_channel_note *ch_note;
   struct xm_channel_note_list *next;
};

struct xm_row_list {
   struct xm_channel_note_list *row;
   struct xm_row_list *next;
};

struct xm_pattern {
   int header_length;
   char packing_type;
   short num_rows;
   short patterndata_size;
   struct xm_row_list *rows;
};


struct xm_pattern_list {
   struct xm_pattern *pattern;
   struct xm_pattern_list *next;
};


struct xm_sample {
   int sample_length;
   int sample_loop_start;
   int sample_loop_length;
   bit8 volume;
   char finetune;
   bit8 type;
   bit8 panning;
   char rel_note_number;
   bit8 reserved;
   char sample_name[23];
   bit16 *data; /* pre-extended with loop or zeroes */
   bit16 *postloop; /* extra data after loop end */
};


struct xm_instrument {
   int instrument_size;
   char instrument_name[29];
   char instrument_type;
   short num_samples;
   int header_size;
   char sample_number[96];
   char vol_envlp[48];
   char pan_envlp[48];
   char num_vol_pts;
   char num_pan_pts;
   char vol_sustain_pt;
   char vol_loop_start_pt;
   char vol_loop_end_pt;
   char pan_sustain_pt;
   char pan_loop_start_pt;
   char pan_loop_end_pt;
   char vol_type;
   char pan_type;
   char vib_type;
   char vib_sweep;
   char vib_depth;
   char vib_rate;
   short vol_fadeout;
   char reserved[22];
   struct xm_sample **samples;
};

struct xm {
   char ID_text[18];
   char mod_name[29];
   char one_a;
   char tracker_name[21];
   char version_low;
   char version_high;
   int header_size;
   short song_length;
   short restart_position;
   short num_channels;
   short num_patterns;
   short num_instruments;
   short flags;
   short tempo;
   short bpm;
   char pattern_order_table[256];
   struct xm_pattern_list *patterns;
   struct xm_instrument *instruments[128];
};

struct xm_channel_note *empty_xm_channel_note(void);
struct xm_channel_note_list *empty_xm_channel_note_list(int num_channels);
struct xm_pattern *empty_xm_pattern(int num_rows, int num_channels);
struct xm_pattern_list *empty_xm_patterns(int num_patterns, int num_channels);
struct xm *empty_xm();

int test_mod(char *filename);
struct xm *load_mod(char *filename);

struct xm_sample *load_raw(char *filename);

int test_s3m(char *filename);
struct xm *load_s3m(char *filename);

struct xm_instrument *dummy_xm_instrument(void);
int test_xm(char *filename);
struct xm *load_xm(char *filename);


#endif  /* __AUBE_H__ */
