/*
 * xm.c
 * 
 * XM file reader
 * 
 * Copyright (C) 1996 Conrad Parker
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>

#include "aube.h"
#include "aube_internal.h"

extern int semitone_tab[60];

struct xm_channel_note *load_mod_channel_note(int is_15, FILE *f)
{
   int i;
   struct xm_channel_note *ch_note;   
   unsigned char byte1, byte2, byte3, byte4;
   unsigned int period;
   
   ch_note = (struct xm_channel_note *)malloc
     (sizeof(struct xm_channel_note));

   ch_note->note = 0;
   ch_note->instrument = 0;
   ch_note->volume = 0;
   ch_note->effect_type = 0;
   ch_note->effect_parameter = 0;
   
   fread(&byte1, sizeof(char), (size_t)1, f);
   fread(&byte2, sizeof(char), (size_t)1, f);
   fread(&byte3, sizeof(char), (size_t)1, f);
   fread(&byte4, sizeof(char), (size_t)1, f);

/*   period = (unsigned int)byte1;*/
   period = (byte1<<8) + byte2;
   ch_note->instrument = (byte1 & 0xf0) + ((byte3 & 0xf0) >> 4);
   ch_note->volume = (short)00;
   ch_note->effect_type = (short)(byte3 & 0x0f);
   ch_note->effect_parameter = (short)byte4;

   if(is_15) {
      ch_note->instrument >>= 4;
   }
   
   if(period == 0) {
      ch_note->note = 0;
   }
   else {
      for(i=0; i<84; i++) {
	 if(period >= semitone_tab[i]) {
	    break;
	 }
      }
      if(i<12) i+=60;
      if(i<24) i+=48;
      ch_note->note = i;
   }
   
   return ch_note;
}


struct xm_channel_note_list *load_mod_channel_note_list(int num_channels, int is_15, FILE *f)
{
   struct xm_channel_note_list *ret_channel_note_list, *channel_note_list,
     *prev_channel_note_list;
   int i;

   ret_channel_note_list = (struct xm_channel_note_list *)malloc(sizeof(struct xm_channel_note_list));
   
   prev_channel_note_list = NULL;
   
   for(i=0; i<num_channels; i++) {
      channel_note_list = (struct xm_channel_note_list *)malloc(sizeof(struct xm_channel_note_list));
      
      channel_note_list->ch_note = load_mod_channel_note(is_15, f);

      if(prev_channel_note_list) {
	 prev_channel_note_list->next = channel_note_list;
      }
      else {
	 ret_channel_note_list = channel_note_list;
      }
      
      prev_channel_note_list = channel_note_list;
   }
   prev_channel_note_list->next = NULL;
   
   return ret_channel_note_list;
}

struct xm_pattern *load_mod_pattern(int num_channels, int is_15, FILE *f)
{
   struct xm_pattern *ret_pattern;
   struct xm_row_list *row_list, *prev_row_list;
   int i;

   ret_pattern = (struct xm_pattern *)malloc(sizeof(struct xm_pattern));

   ret_pattern->header_length = 0;
   ret_pattern->packing_type = 0;
   ret_pattern->num_rows = 64;
   ret_pattern->patterndata_size = 0;
	 
   row_list = NULL;
   prev_row_list = NULL;
   
   for(i=0; i<64; i++) {
      row_list = (struct xm_row_list *)malloc(sizeof(struct xm_row_list));
      row_list->row = load_mod_channel_note_list(num_channels, is_15, f);
      
      if(prev_row_list) {
	 prev_row_list->next = row_list;
      }
      else {
	 ret_pattern->rows = row_list;
      }
      
      prev_row_list = row_list;
   }   
   prev_row_list->next = NULL;
   
   return ret_pattern;
}

struct xm_pattern_list *load_mod_patterns(int num_patterns, int num_channels,
					  int is_15, FILE *f)
{
   struct xm_pattern_list *ret_pattern_list, *pattern_list, *prev_pattern_list;
   struct xm_pattern pattern;
   
   int i;

   int ftell_start, ftell_end;

   pattern_list = NULL;
   prev_pattern_list = NULL;
   
   for(i=0; i<num_patterns; i++) {
/*      fread(&header_length, sizeof(int), (size_t)1, f);
      
      fread(&packing_type, sizeof(char), (size_t)1, f);
      
      fread(&num_rows, sizeof(short), (size_t)1, f);
      
      fread(&patterndata_size, sizeof(short), (size_t)1, f);
*/
      
/*      if(patterndata_size > 0) {*/
	 
      ftell_start = ftell(f);

/*      fprintf(stdout, "PATTERN %d:\tftell (should be %d): %d\n", i, 1084 + i*1024, ftell_start);*/
      
	 pattern_list = (struct xm_pattern_list *)malloc(sizeof(struct xm_pattern_list));
	 
	 pattern_list->pattern = load_mod_pattern(num_channels, is_15, f);

/*	 
	 set_current_pattern(pattern_list->pattern);
*/	 
	 ftell_end = ftell(f);
/*      }*/
      
      if(prev_pattern_list) {
	 prev_pattern_list->next = pattern_list;
      }
      else {
	 ret_pattern_list = pattern_list;
      }
      
      prev_pattern_list = pattern_list;
   }
   if(prev_pattern_list != NULL) {   
      prev_pattern_list->next = NULL;
   }
   
   return ret_pattern_list;
}

void load_mod_sample_data(struct xm_sample *sample, FILE *f)
{
   int i;
   unsigned short p;
   unsigned char d;
   bit16 *s;
   
   sample->sample_length <<= 1;
   sample->sample_loop_start <<= 1;
   sample->sample_loop_length <<= 1;
   
   sample->data = (bit16 *)malloc((size_t)(sample->sample_length) + SAMPLE_EXTEND);
   sample->postloop = NULL;
   
   s = sample->data;
   
   p= 0x00;
   for(i=0; i<sample->sample_length>>1; i++) {
      fread(&d, sizeof(unsigned char), (size_t)1, f);
      p = (unsigned short)d<<8;
      *(s++) = p;
   }
   
   preextend(sample);
}


struct xm_sample *load_mod_sample(FILE *f)
{
   struct xm_sample *sample;
   unsigned short tmp_short;
   unsigned char byte1, byte2;
   sample = (struct xm_sample *)malloc(sizeof(struct xm_sample));

   /* MOD sample lengths are stored in words */
   fread(&byte1, sizeof(char), (size_t)1, f);
   fread(&byte2, sizeof(char), (size_t)1, f);
   sample->sample_length = (unsigned int)(((byte1<<8) + byte2)<<1);

   fread(&sample->finetune, sizeof(char), (size_t)1, f);
   /* ensure upper four bits zero */
   sample->finetune &= 0x0f;
   
   fread(&sample->volume, sizeof(char), (size_t)1, f);

   fread(&byte1, sizeof(char), (size_t)1, f);
   fread(&byte2, sizeof(char), (size_t)1, f);
   sample->sample_loop_start = (unsigned int)(((byte1<<8) + byte2)<<1);
   
   fread(&byte1, sizeof(char), (size_t)1, f);
   fread(&byte2, sizeof(char), (size_t)1, f);
   sample->sample_loop_length = (unsigned int)(((byte1<<8) + byte2)<<1);
   if(sample->sample_loop_length == 2) {
      sample->sample_loop_length = 0;
   }

   sample->panning = 0;
  
/*   fread(sample->sample_name, sizeof(char), (size_t)22, f);*/
   sample->sample_name[0] = '\0';

   return sample;
}  

struct xm_sample **load_mod_samples(FILE *f)
{
   struct xm_sample **samples;
   int i;
   int ftell_start, ftell_end;
   
   samples = (struct xm_sample **)malloc(sizeof(struct xm_sample *));
   
   samples[0] = load_mod_sample(f);

   return samples;
}

void load_mod_sample_data_all(struct xm *m, FILE *f)
{
   int i;

   for(i=0; i<m->num_instruments; i++) {
      if(m->instruments[i]->samples[0]->sample_length == 0) {
	 m->instruments[i]->samples[0]->data = NULL;
      }
      else {
	 load_mod_sample_data(m->instruments[i]->samples[0], f);
      }
   }

}


struct xm_instrument *load_mod_instrument(FILE *f)
{
   struct xm_instrument *ret_instrument;
   size_t rest;
   int i;
   int dummy;
   int ftell_start, ftell_end;
   
   ret_instrument = (struct xm_instrument *)malloc(sizeof(struct xm_instrument));

   ftell_start = ftell(f);
   
/*   fread(&ret_instrument->instrument_size, sizeof(int), (size_t)1, f);*/

   fread(ret_instrument->instrument_name, sizeof(char), (size_t)22, f);
   ret_instrument->instrument_name[22] = '\0';

   ret_instrument->num_samples = 1;
   
/*   fread(&ret_instrument->instrument_type, sizeof(char),
	 (size_t)1, f);
   fread(&ret_instrument->num_samples, sizeof(short), (size_t)1, f);

   if(ret_instrument->num_samples == 0) {
      ret_instrument->header_size = 0;
      ret_instrument->samples = NULL;
      fread(&dummy, (size_t)4, (size_t)1, f);
   }
   else {
      fread(&ret_instrument->header_size, sizeof(int), (size_t)1, f);
*/      /*
       fprintf(stdout, "\tHEADER SIZE: %02X\n", ret_instrument->header_size);
       */ 
/*      fread(ret_instrument->sample_number, sizeof(char), (size_t)96, f);
      fread(ret_instrument->vol_envlp, sizeof(char), (size_t)48, f);
      fread(ret_instrument->pan_envlp, sizeof(char), (size_t)48, f);
      fread(&ret_instrument->num_vol_pts, sizeof(char), (size_t)38, f);
*/      
      ret_instrument->samples = load_mod_samples(f);
/*   }

   ftell_end = ftell(f);
*/
   /*
   if(ret_instrument->instrument_size != ftell_end - ftell_start) {
      fprintf(stdout, "\tread %u/%u ... resyncing\n", ftell_end - ftell_start,
	      ret_instrument->instrument_size);
      fseek(f, ftell_start + ret_instrument->instrument_size, SEEK_SET);
   }
    */
   
   return ret_instrument;
      
}

void load_mod_instruments_all(struct xm *m, FILE *f)
{
   int i;
   long ftell_15;
   int i_size;
   struct xm_sample_list *sl;
   
   for(i=0; i < 31; i++) {
      m->instruments[i] = load_mod_instrument(f);
   }

   for(;i<=128; i++) {
      m->instruments[i] = dummy_xm_instrument();
   }

/*   update_instrlist();*/
}
      
struct xm *load_mod(char *filename)
{
   FILE *f;
   struct xm *m;
   char song_len;
   int i;
   char MK[4];
   
   f = fopen(filename, "r");
   
   m = (struct xm *)malloc(sizeof(struct xm));
   
/*   fread(m->ID_text, sizeof(char), (size_t)17, f);*/
   m->ID_text[0] = '\0';
   
   fread(m->mod_name, sizeof(char), (size_t)20, f);
   m->mod_name[20] = '\0';

/*   fread(&m->one_a, sizeof(char), (size_t)1, f);
   fread(m->tracker_name, sizeof(char), (size_t)20, f);
*/
   m->tracker_name[0] = '\0';
   
/*   fread(&m->version_low, sizeof(char), (size_t)2, f);

   if(m->version_low != 4 && m->version_high != 1) {
      fprintf(stderr, "Unknown version %d.%d -- aborting\n", m->version_high, m->version_low);
      return(NULL);
   }
   
   fread(&m->header_size, sizeof(int), (size_t)1, f);

*/
   /*
   update_modinfo();
   */

   load_mod_instruments_all(m, f);
   
   /* ftell should be 950 */
/*   fprintf(stderr, "ftell (should be 950): %d\n", ftell(f));*/

   fread(&song_len, sizeof(char), (size_t)1, f);
   m->song_length = (short)song_len;

   /* dummy */
   fread(&song_len, sizeof(char), (size_t)1, f);
   
   m->restart_position = 0;
   m->num_channels = 4;
   m->num_instruments = 31;
   m->flags = 0x0; /* use amiga frequency table ... */
   m->tempo = 6; /* tempo & bpm until set in mod ... */
   m->bpm = 125; /* unfortunately the mod spec sets default bpm to 125 :( */
   
   fread(&m->pattern_order_table, sizeof(char), (size_t)128, f);
   for(i=128; i< 256; i++) {
      m->pattern_order_table[i] = 0;
   }

   /* find num_patterns */
   m->num_patterns = 0;
   for(i=0; i< 128; i++) {
      if(m->pattern_order_table[i] > m->num_patterns) {
	 m->num_patterns = m->pattern_order_table[i];
      }
   }
/*   fprintf(stdout, "num_patterns: %d\n", m->num_patterns);*/
   
   
   /* M.K. - signature */
   fread(&MK, sizeof(char), (size_t)4, f);
/*   fprintf(stdout, "%4s\n", MK);*/

   if(!strncmp(MK, (char *)"6CHN", 4)) {
      m->num_channels = 6;
   }
   else if(!strncmp(MK, (char *)"8CHN", 4) || !strncmp(MK, (char *)"FLT8", 4)) {
      m->num_channels = 8;
   }
   else if(!isgraph(MK[0])) {
      /*&& !isgraph(MK[1]) &&
      !isgraph(MK[2]) && !isgraph(MK[3])) {*/
      fprintf(stdout, "15 instrument mod\n");
      m->num_instruments = 15;
   }
   
   /*
   update_patt_list();
   */
   
   m->patterns = load_mod_patterns(m->num_patterns, m->num_channels,
				   (m->num_instruments == 15), f);

   load_mod_sample_data_all(m, f);
   
   fclose(f);

/*
   update_modinfo();
  */
   
   return(m);
}

