#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <math.h>

#include "aube.h"
#include "aube_internal.h"

extern struct xm *mod;

extern int song_pos;

extern int period_tab[96];
extern double note_rates[96];

int player_pid = -1;
int playing = 0;

int dev_dsp;
int row_time;

AUBEchannel *parameters;

bit16 *playbuffer16 = NULL;
bit8 *playbuffer8 = NULL;

void open_dev_dsp(void);
void close_dev_dsp(void);   


void play_sample(struct xm_sample *sample)
{
   write(dev_dsp, sample->data, (size_t)sample->sample_length);
}

void play_sample_scale(struct xm_sample *sample, float rate_f)
{
   bit32 offset = 0, playlength = 0;
   float offset_f=0.0, playlength_f = 0.0;
   int t;
   bit16 pb;

   playlength_f = ((float)sample->sample_length) / rate_f;
   playlength = (unsigned int)playlength_f;

   if(PLAYBACK_BITRATE == 16) {
      if(!playbuffer16) playbuffer16 = (bit16 *)calloc((size_t)playlength, sizeof(bit16));
   }
   else {
      if(!playbuffer8) playbuffer8 = (char *)calloc((size_t)playlength, sizeof(bit8));
   }
   
   for(t = 0; t < playlength>>1; t++) {
     offset_f = (float)t * rate_f;
     offset = (int)offset_f;
      if(PLAYBACK_BITRATE == 16) {
	 playbuffer16[2 * t] = (bit16)(*(sample->data + offset))*(sample->volume) >>8;
	 playbuffer16[2 * t + 1] = playbuffer16[2 * t];
      }
      else {
	 pb =  (bit16)(((*(sample->data + offset))*(sample->volume))>>16);
	 playbuffer8[2 * t] = (bit8)(pb+128);
	 playbuffer8[2 * t + 1] = playbuffer8[2 * t];
      }
   }

   if(PLAYBACK_BITRATE == 16) {      
      write(dev_dsp, playbuffer16, (size_t)(playlength<<1));
   }
   else {
      write(dev_dsp, playbuffer8, (size_t)(playlength));
   }
}


void play_instrument(struct xm_instrument *instrument)
{
   if(instrument == NULL) {
      return;
   }
   else if(instrument->samples == NULL) {
      return;
   }
   

   open_dev_dsp();
   
   play_sample(instrument->samples[instrument->sample_number[47]]);
   
   close_dev_dsp();
}

void play_instrument_scale(struct xm_instrument *instrument, int note_num)
{
   struct xm_sample *sample;
   float period, frequency;
   float rate_f;
   
   if(!instrument->samples) {
      return;
   }
   
   open_dev_dsp();

   sample = instrument->samples[instrument->sample_number[note_num]];
   
   note_num += 12 + sample->rel_note_number;
   
   if(mod->flags & 0x1) {
      /* Linear frequency */
      period = (7680) - ((note_num-1) << 6) - (sample->finetune>>1) ;
      frequency = 8363.0 * pow(2, (4608.0 - period) / 768.0);
   }
   else {
      /* Amiga frequency */
      period =
	period_tab[((note_num % 12) * 8)/* + pl->finetune/16*/] * 16 / pow(2, (int)(note_num/12));
      frequency = 8363 * 1712 / period;
   }
   rate_f = frequency / (2.0 * (float)PLAYBACK_FREQ);
   
   play_sample_scale(sample, rate_f);
   
   close_dev_dsp();
}

void allocate_parameters(int num_channels)
{
   int i;

   parameters = (AUBEchannel *)malloc(num_channels * sizeof(AUBEchannel));

   for(i=0; i<num_channels; i++) {
      parameters[i] = (AUBEchannel)malloc(sizeof(_AUBEchannel));
      parameters[i]->active = (char)0x1;
      parameters[i]->data = NULL;
      parameters[i]->limit = 0;
      parameters[i]->loop_start = 0.0;
      parameters[i]->loop_end = 0.0;
      parameters[i]->offset = 0;
      parameters[i]->offset_f = 0.0;
      parameters[i]->rate_f = 0.0;
      parameters[i]->note = 0;
      parameters[i]->finetune = 0;
      parameters[i]->panning = 128;
      parameters[i]->portamento = 0;
      parameters[i]->porta_limit = 0;
      parameters[i]->volume = 0;
   }
}


int read_row_effects(struct xm_channel_note_list *row)
{
   struct xm_channel_note_list *chnl;
   AUBEchannel pl;
   struct xm_instrument *instr;
   int vol;
   float period, frequency;
   struct xm_sample *sample;
   int x, y, z;
   int p;
   int keepgoing=1;
   
   if(parameters == NULL) {
      allocate_parameters(mod->num_channels);
   }
   
   /* reset data pointers for new notes ... */
   p = 0;
   for(chnl = row; chnl != NULL; chnl = chnl->next) {

      pl = parameters[p];
      
      /* NOTE / INSTRUMENT COLUMNS */
      if(chnl->ch_note->note != 0 && chnl->ch_note->instrument != 0) {
	 instr = mod->instruments[chnl->ch_note->instrument - 1];	 
	 sample = instr->samples[instr->sample_number[chnl->ch_note->note]];

	 pl->note = chnl->ch_note->note + sample->rel_note_number;
	 pl->finetune = sample->finetune;
	 pl->data = sample->data;
	 pl->limit = sample->sample_length>>1;
	 pl->loop_start = sample->sample_loop_start>>1;
	 pl->loop_end = (sample->sample_loop_start + sample->sample_loop_length)>>1;
	 pl->offset = 0;
	 pl->offset_f = 0.0;

	 /*	 pl->rate_f = note_rates[pl->note];*/
	 /* FFWD : " * 2.0 <==> note += 12 ... */

	 pl->panning = sample->panning;
/*	 if(!pl->panning) {
	    pl->panning = (p&1)?0:255;
	 }
 */
	 pl->portamento = 0;
	 pl->volume = sample->volume;
      }
      
      /* VOLUME COLUMN */
      if((vol = chnl->ch_note->volume) >= 0x10) {
	 if(vol <= 0x5f) {
	    pl->volume = vol - 0x10;
	 }
	 else if(vol <= 0x6f) {
	    pl->volume -= vol & 0x0f;
	 }
	 else if(vol <= 0x7f) {
	    pl->volume += vol & 0x0f;
	 }
	 else if(vol <= 0xbf) {
	 }
	 else if(vol <= 0xcf) {
	    pl->panning = (vol & 0x0f) << 4;
	 }
	 else if(vol <= 0xdf) {
	    pl->panning += vol & 0x0f;
	 }
	 else if(vol <= 0xef) {
	    pl->panning -= vol & 0x0f;
	 }
	 
      }

      x = (chnl->ch_note->effect_parameter & 0xf0) >> 4;
      y = chnl->ch_note->effect_parameter & 0x0f;
      z = x * 16 + y;

      /* EFFECT COLUMN */
      switch(chnl->ch_note->effect_type) {
       case 0x01:
	 pl->portamento = z;
	 break;
       case 0x02:
	 pl->portamento = -z;
	 break;
       case 0x08:
	 pl->panning = z;
	 break;
       case 0x09:
	 pl->offset = z<<8;
	 pl->offset_f = (float)pl->offset;
	 break;
       case 0x0A:
	 if(x) pl->volume += x * (mod->tempo - 1);
	 else pl->volume -= y * (mod->tempo - 1);
	 if(pl->volume > 64) pl->volume = 64;
	 if(pl->volume < 0) pl->volume = 0;
	 break;
       case 0x0C:
	 pl->volume = z;
	 break;
       case 0x0D:
	 keepgoing = 0;
	 break;
       case 0xE5:
	 pl->finetune = z;
	 break;
       case 0xE9:
	 pl->offset_f = 0.0;
	 break;
       case 0x0F:
	 if(z==0) z=1;
	 if(z <= 32) {
	    mod->tempo = z;
	 }
	 else {
	    mod->bpm = z;
	 }
	 break;
       case 'P' - 55:
	 pl->panning += chnl->ch_note->effect_parameter;
	 break;
       default:
	 break;
      }
      
      if(mod->flags & 0x1) {
	 /* Linear frequency */
	 period = (7680) - ((pl->note-1)<<6) - (pl->finetune>>1);
	 frequency = 8363.0 * pow(2, (4608.0 - period) / 768.0);
      }
      else {
	 /* Amiga frequency */
	 period =
	   period_tab[((pl->note % 12) * 8) + pl->finetune/16] * 16 / pow(2, (int)(pl->note/12));
	 frequency = 8363 * 1712 / (2 * period);
      }
      pl->rate_f = frequency  / ((float)PLAYBACK_FREQ);

      /*      
      if(pl->panning < 0) {
	 pl->panning = 0;
      }
      else if (pl->panning > 255) {
	 pl->panning = 255;
      }
       */
         
      p++;
   }
   
   return(keepgoing);
}

void mix_row(void)
{
   int t, p, s, buft=0;
   AUBEchannel pl;
   bit32 di[2];
   bit32 d;
   bit32 di_n;
   float di_f;
   
   row_time = (int)((float)(60 * PLAYBACK_FREQ) / (float)(24 * mod->bpm));

   if(PLAYBACK_BITRATE == 16) {   
      if(!playbuffer16) playbuffer16 = (bit16 *)calloc(2 * row_time * mod->tempo, sizeof(bit16));
   }
   else {
      if(!playbuffer8) playbuffer8 = (char *)calloc(2 * row_time * mod->tempo, sizeof(char));
   }
   
   
   di_n = mod->num_channels<<4;
   di_f = 1.0 / (float)di_n;
   for(s=0; s<mod->tempo; s++) {
      /* per tick processing */
      for(p=0; p < mod->num_channels; p++) {
	 pl = parameters[p];
	 pl->note += pl->portamento;
	 /* if(pl->note < 0) pl->note = 0;*/
	 if(pl->note > 96) pl->note = 96;
	 if(pl->data && pl->offset >= pl->limit) {
	    if(pl->loop_end) {
	       while(pl->offset >= pl->limit) {  
		  pl->offset -= pl->loop_end - pl->loop_start;
	       }
	       pl->offset_f = (float)pl->offset;
	    }
	    else {
	       pl->data = NULL;
	       pl->limit = 0;
	    }
	 }
      }

      for(t=0; t < row_time; t++ /* FFWD: t += 2 */ ) {
	 di[0] = 0; di[1] = 0;
	 for(p=0; p < mod->num_channels; p++) {
	    pl = parameters[p];
	    if(pl->data) {
	       d = (*(pl->data + pl->offset)) * pl->volume;
	       di[0] += (d * pl->panning)>>8;
	       di[1] += (d * (255 - pl->panning))>>8;
	       
	       pl->offset_f += pl->rate_f;	       
	       pl->offset = (int)pl->offset_f;
	    }
	 }
	 if(PLAYBACK_BITRATE == 16) {
	    playbuffer16[buft++] = (bit16)(di[0]>>8);
	    playbuffer16[buft++] = (bit16)(di[1]>>8);
	 }
	 else {
	    playbuffer8[buft++] = (bit8)((di[0]>>16)+128);
	    playbuffer8[buft++] = (bit8)((di[1]>>16)+128);
	 }
	 
      }
   }
}

void write_playbuffer(int f)
{	
   if(PLAYBACK_BITRATE == 16) {
      write(f, playbuffer16, (size_t)row_time * mod->tempo * 2 * sizeof(bit16));
   }
   else{
      write(f, playbuffer8, (size_t)row_time * mod->tempo * 2 * sizeof(bit8));      
   }
}

int play_row(struct xm_channel_note_list *row)
{
   int keepgoing = 1;
   
   keepgoing = read_row_effects(row);
   mix_row();
   write_playbuffer(dev_dsp);
   
   return(keepgoing);
}

void play_pattern(struct xm_pattern *pattern)
{
   struct xm_row_list *row_list;

/*   
   scroll_to(0);
*/   
   for(row_list = pattern->rows;
       row_list != NULL;
       row_list = row_list->next) {   
      if(!play_row(row_list->row)) break;
   }
}

void play_single_pattern(struct xm_pattern *pattern)
{
   open_dev_dsp();
   for(;;) {
      play_pattern(pattern);
   }
   
   close_dev_dsp();
}

struct xm_pattern *get_pattern(struct xm *mod, int patt_num)
{
   int i=0;
   struct xm_pattern_list *pattern_list;
   
   pattern_list = mod->patterns;
   for(i=0; i<patt_num; i++) {
      pattern_list = pattern_list->next;
   }
   return(pattern_list->pattern);
}

void play_mod(struct xm *mod)
{
   short i;
   struct xm_pattern *pattern;

   open_dev_dsp();
   
   /*fprintf(stdout, "mod length: %d\n", mod->song_length);*/
   
   for(i=song_pos; i < mod->song_length; i++) {
      /*fprintf(stdout, "Getting pattern [%d] => %d / %d :\n", i, mod->pattern_order_table[i], mod->song_length);*/
      pattern = get_pattern(mod, mod->pattern_order_table[i]);
/*      set_current_pattern(pattern);*/
      play_pattern(pattern);
   }

   for(;;) {   
      for(i=mod->restart_position; i < (int)mod->song_length; i++) {
	 fprintf(stdout, "Playing pattern [%d] => %d\n", i, mod->pattern_order_table[i]);
	 pattern = get_pattern(mod, mod->pattern_order_table[i]);
	 /*      set_current_pattern(pattern);*/
	 play_pattern(pattern);
      }
   }

   
   close_dev_dsp();
}


void open_dev_dsp(void)
{
   int frequency, stereo, samplesize;

   int numfrags, fragsize, fragmentsize;
   
   if(!(dev_dsp = open("/dev/dsp", O_WRONLY | O_NDELAY, 0))) {
      fprintf(stdout, "Unable to open /dev/dsp\n");
      exit(1);
   }
   
   
   samplesize = PLAYBACK_BITRATE;
   if( ioctl( dev_dsp, SNDCTL_DSP_SAMPLESIZE, &samplesize ) == -1 ) {
      fprintf( stderr, "Unable to set sample size!\n");
      exit(1);
   }

   stereo = 1;
   if( ioctl( dev_dsp, SNDCTL_DSP_STEREO, &stereo ) == -1 ) {
      fprintf( stderr, "Unable to set stereo playback!\n");
      exit(1);
   }

   frequency = PLAYBACK_FREQ;
   if( ioctl( dev_dsp, SNDCTL_DSP_SPEED, &frequency ) == -1 ) {
      fprintf( stderr, "Unable to set playback frequency!\n");
      exit(1);
   }

   numfrags = 4;
   fragsize = 17;
   
   fragmentsize = (numfrags << 16) | fragsize;
   if( ioctl( dev_dsp, SNDCTL_DSP_SETFRAGMENT, &fragmentsize) == -1) {
      fprintf( stderr, "Unable to set buffer fragment!\n");
      exit(1);
   }
   
   
   /*
   if( ioctl( dev_dsp, SNDCTL_DSP_SYNC, 0 ) == -1 ) {
      fprintf( stderr, "Unable to sync dma!\n");
      exit(1);
   }
   */


}

void close_dev_dsp(void)
{
   close(dev_dsp);
}

