/*
 * xm.c
 * 
 * XM file reader
 * 
 * Copyright (C) 1996 Conrad Parker
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>

#include "aube.h"

struct xm_channel_note *load_s3m_channel_note(unsigned char what, FILE *f)
{
   int i;
   struct xm_channel_note *ch_note;   
   
   ch_note = empty_xm_channel_note();
   
   /* hehehe ... code ch_note loader!! then sample data loader!! */
      
   if(what&32) {
      fread(&ch_note->note, sizeof(char), (size_t)1, f);
      fread(&ch_note->instrument, sizeof(char), (size_t)1, f);
   }
   if(what&64) {
      fread(&ch_note->volume, sizeof(char), (size_t)1, f);
      if(ch_note->volume == 255) {
	 ch_note->volume = 0;
      }
      
   }
   if(what&128) {
      fread(&ch_note->effect_type, sizeof(char), (size_t)1, f);
      if(ch_note->effect_type == 255) {
	 ch_note->effect_type = 0;
      }
      fread(&ch_note->effect_parameter, sizeof(char), (size_t)1, f);
   }
   
   return ch_note;
}


struct xm_channel_note_list *load_s3m_channel_note_list(int num_channels, FILE *f)
{
   struct xm_channel_note_list *ret_channel_note_list, *channel_note_list,
     *prev_channel_note_list;
   int i;
   unsigned char what;
   
   ret_channel_note_list = (struct xm_channel_note_list *)malloc(sizeof(struct xm_channel_note_list));
   
   prev_channel_note_list = NULL;
   
   for(i=0; i<num_channels; i++) {

      fread(&what, sizeof(char), (size_t)1, f);
      if(what == 0) break;
      
      for(;i<(what&31) && (i<num_channels);i++) {
	 channel_note_list =
	   (struct xm_channel_note_list *)malloc(sizeof(struct xm_channel_note_list));
	 
	 channel_note_list->ch_note = empty_xm_channel_note();
	 
	 if(prev_channel_note_list) {
	    prev_channel_note_list->next = channel_note_list;
	 }
	 else {
	    ret_channel_note_list = channel_note_list;
	 }
	 
	 prev_channel_note_list = channel_note_list;
      }
         
      channel_note_list =
	(struct xm_channel_note_list *)malloc(sizeof(struct xm_channel_note_list));
      
      channel_note_list->ch_note = load_s3m_channel_note(what, f);

      if(prev_channel_note_list) {
	 prev_channel_note_list->next = channel_note_list;
      }
      else {
	 ret_channel_note_list = channel_note_list;
      }
      
      prev_channel_note_list = channel_note_list;
   }
   /* second for loop will be encountered after what==0 ==> end row */
   for(;i<num_channels; i++) {
      	 channel_note_list =
	   (struct xm_channel_note_list *)malloc(sizeof(struct xm_channel_note_list));
	 
	 channel_note_list->ch_note = empty_xm_channel_note();
	 
	 if(prev_channel_note_list) {
	    prev_channel_note_list->next = channel_note_list;
	 }
	 else {
	    ret_channel_note_list = channel_note_list;
	 }
	 
	 prev_channel_note_list = channel_note_list;
   }
      
   prev_channel_note_list->next = NULL;
   
   return ret_channel_note_list;
}

struct xm_pattern *load_s3m_pattern(int num_channels, long para, FILE *f)
{
   struct xm_pattern *ret_pattern;
   struct xm_row_list *row_list, *prev_row_list;
   int i;

   fseek(f, para, SEEK_SET);
   
   ret_pattern = (struct xm_pattern *)malloc(sizeof(struct xm_pattern));

   ret_pattern->header_length = 0;
   ret_pattern->packing_type = 0;
   ret_pattern->num_rows = 64;
   ret_pattern->patterndata_size = 0;
	 
   row_list = NULL;
   prev_row_list = NULL;
   
   for(i=0; i<64; i++) {
      row_list = (struct xm_row_list *)malloc(sizeof(struct xm_row_list));
      row_list->row = load_s3m_channel_note_list(num_channels, f);
      
      if(prev_row_list) {
	 prev_row_list->next = row_list;
      }
      else {
	 ret_pattern->rows = row_list;
      }
      
      prev_row_list = row_list;
   }   
   prev_row_list->next = NULL;
   
   return ret_pattern;
}

struct xm_pattern_list *load_s3m_patterns(int num_patterns, int num_channels,
					  long *pat_para, FILE *f)
{
   struct xm_pattern_list *ret_pattern_list, *pattern_list, *prev_pattern_list;
   struct xm_pattern pattern;
   
   int i;

   int ftell_start, ftell_end;

   pattern_list = NULL;
   prev_pattern_list = NULL;
   
   for(i=0; i<num_patterns; i++) {
      
      pattern_list = (struct xm_pattern_list *)malloc(sizeof(struct xm_pattern_list));
      
      pattern_list->pattern = load_s3m_pattern(num_channels, pat_para[i], f);

      if(prev_pattern_list) {
	 prev_pattern_list->next = pattern_list;
      }
      else {
	 ret_pattern_list = pattern_list;
      }
      
      prev_pattern_list = pattern_list;
   }
   if(prev_pattern_list != NULL) {   
      prev_pattern_list->next = NULL;
   }
   
   return ret_pattern_list;
}

void load_s3m_sample_data_8(struct xm_sample *sample, FILE *f)
{
   int i;
   unsigned short p;
   unsigned char d;
   unsigned short *s;
   
   sample->data = (unsigned short *)malloc((size_t)sample->sample_length * 2);
   
   s = sample->data;
   
   p= 0x00;
   for(i=0; i<sample->sample_length; i++) {
      fread(&d, sizeof(unsigned char), (size_t)1, f);
      p = (unsigned short)d<<8;
      *(s++) = p;
   }
}

void load_s3m_sample_data_16(struct xm_sample *sample, FILE *f)
{
   int i;
   unsigned short p;
   unsigned short ds;
   unsigned short *s;
   
   sample->data = (unsigned short *)malloc((size_t)sample->sample_length);
   
   s = sample->data;
   
   p = 0x00;
   sample->sample_loop_start >>= 1;
   sample->sample_loop_length >>= 1;
   
   for(i=0; i<sample->sample_length/2; i++) {
      fread(&ds, sizeof(unsigned short), (size_t)1, f);
      p += ds;
      *(s++) = p;
   }  
}

struct xm_instrument *load_s3m_instrument(long para, FILE *f)
{
   struct xm_instrument *ret_instrument;
   int i;
   size_t rest;
   int dummy;
   int ftell_start, ftell_end;
   char T;
   char dos_fname[13];
   char F;
   
   ret_instrument = (struct xm_instrument *)malloc(sizeof(struct xm_instrument));

   fseek(f, para, SEEK_SET);
   
   fread(&T, sizeof(char), (size_t)1, f);
   fprintf(stdout, "T: %d\n", T);
   
   fread(dos_fname, sizeof(char), (size_t)12, f);
   dos_fname[12] = '\0';
   fprintf(stdout, "dos filename: %s\n", dos_fname);
   
   fseek(f, para + (long)0x1f, SEEK_SET);
   fread(&F, sizeof(char), (size_t)1, f);
   
   fseek(f, para + (long)0x30, SEEK_SET);
   /* s3m ins names can be up to 28 characters long */
   fread(ret_instrument->instrument_name, sizeof(char), (size_t)28, f);
   ret_instrument->instrument_name[28] = '\0';
   fprintf(stdout, "ins name: %s\n", ret_instrument->instrument_name);
   
   ret_instrument->num_samples = 1;

   ret_instrument->samples = (struct xm_sample **)malloc(sizeof(struct xm_sample *));

   
   ret_instrument->samples[0] =
     (struct xm_sample *)malloc(sizeof(struct xm_sample));
   

   fseek(f, para + (long)0x10, SEEK_SET);
   fread(&ret_instrument->samples[0]->sample_length, sizeof(int), (size_t)1,f);
   if(ret_instrument->samples[0]->sample_length > 64000) {
      ret_instrument->samples[0]->sample_length = 64000;
   }
   fread(&ret_instrument->samples[0]->sample_loop_start, sizeof(int), (size_t)1,f);
   if(ret_instrument->samples[0]->sample_loop_start > 64000) {
      ret_instrument->samples[0]->sample_loop_start = 64000;
   }
   fread(&ret_instrument->samples[0]->sample_loop_length, sizeof(int), (size_t)1,f);
   ret_instrument->samples[0]->sample_loop_length -= ret_instrument->samples[0]->sample_length +1;
   if(ret_instrument->samples[0]->sample_loop_length > 64000) {
      ret_instrument->samples[0]->sample_loop_length = 64000;
   }

   if(!F&0x1) {
      /* it would probably be better to load existing loop parameters
       * and have a loop flag in the internal format, so loops could be
       * created but not used during song creation. Fuck it.
       */
      ret_instrument->samples[0]->sample_loop_start = 0;
      ret_instrument->samples[0]->sample_loop_length = 0;
   }
   
   fread(&ret_instrument->samples[0]->volume, sizeof(char), (size_t)1, f);
   
   fseek(f, para + (long)0x50, SEEK_SET);
   
   /* maube can't do stereo samples YET (because it's currently based on
    * FT2) so I'll just load the left channel. S3M can't do 16bit stereo
    * anyway... */

   /* F&0x2 == stereo */

   /* F&0x4 == 16bit */
   if(F&0x4) {
      load_s3m_sample_data_16(ret_instrument->samples[0], f);
   }
   else {
      load_s3m_sample_data_8(ret_instrument->samples[0], f);
   }
   
   return ret_instrument;      
}

void load_s3m_instruments_all(struct xm *m, long *ins_para, FILE *f)
{
   int i;
   long ftell_15;
   int i_size;
   struct xm_sample_list *sl;
   
   for(i=0; i < m->num_instruments; i++) {
      fprintf(stdout, "Loading instrument %d\n", i);
      m->instruments[i] = load_s3m_instrument(ins_para[i], f);
   }

   for(;i<=128; i++) {
      m->instruments[i] = dummy_xm_instrument();
   }

/*   update_instrlist();*/
}



struct xm *load_s3m(char *filename)
{
   FILE *f;
   struct xm *m;
   char song_len;
   int i;
   char type;
   short ordnum, insnum, patnum, cwtv, ffv, flags;
   char SCRM[4];
   short gv, is, it, mv;
   char chan_settings[32];
   long *ins_para, *pat_para, fheader_offset;
   short sdummy;

   f = fopen(filename, "r");
   
   m = (struct xm *)malloc(sizeof(struct xm));
   
/*   fread(m->ID_text, sizeof(char), (size_t)17, f);*/
   m->ID_text[0] = '\0';
   
   /**** S3M mod names can be up to 28 chars, not 20 ***/
   fread(m->mod_name, sizeof(char), (size_t)28, f);
   m->mod_name[28] = '\0';

   fprintf(stdout, "mod name: %s\n", m->mod_name);
   
   m->tracker_name[0] = '\0';

   fread(&m->one_a, sizeof(char), (size_t)1, f);
   fprintf(stdout, "1a: %x\n", m->one_a);
   
   fread(&type, sizeof(char), (size_t)1, f);
   fprintf(stdout, "type: %d\n", type);
   if(type != 16) {
      fprintf(stdout, "Not an s3m module.\n");
      free(m);
      return(NULL);
   }
   
   fseek(f, 0x20, SEEK_SET);
   
   fread(&ordnum, sizeof(short), (size_t)1, f);
   fread(&insnum, sizeof(short), (size_t)1, f);
   fread(&patnum, sizeof(short), (size_t)1, f);
   fread(&flags, sizeof(short), (size_t)1, f);
   fread(&cwtv, sizeof(short), (size_t)1, f);
   fread(&ffv, sizeof(short), (size_t)1, f);
   
   fprintf(stdout, "ordnum: %d\ninsnum: %d\npatnum: %d\n", ordnum, insnum, patnum);
   fprintf(stdout, "flags: %x\ncwtv: %x\nffv: %x\n", flags, cwtv, ffv);
   
   fseek(f, 0x30, SEEK_SET);
   
   fread(&gv, sizeof(char), (size_t)1, f);
   fread(&is, sizeof(char), (size_t)1, f);
   fread(&it, sizeof(char), (size_t)1, f);
   fread(&mv, sizeof(char), (size_t)1, f);

   fprintf(stdout, "gv: %x\nis: %x\nit: %x\nmv: %x\n", gv, is, it, mv);
   
   fseek(f, 0x40, SEEK_SET);
   fread(chan_settings, sizeof(char), (size_t)32, f);

   for(i=0; i<32; i++) {
      fprintf(stdout, "Channel %d: %x\n", i, chan_settings[i]);
   }
   
   if(ordnum <= 128) {
      m->song_length = (short)ordnum;
   }
   else {
      /* hack for xm compat */
      m->song_length = 128;
   }
    
   fprintf(stdout, "song_length: %d\n", m->song_length);
   
   m->num_instruments = insnum;
   m->num_patterns = patnum;
   
   fread(&m->pattern_order_table, sizeof(char), (size_t)m->song_length, f);

   for(i=0; i<m->song_length; i++) {
      fprintf(stdout, "Pattern %d: %d\n", i, m->pattern_order_table[i]);
   }
   
   
   m->restart_position = 0;
#ifdef CRAP
   /* calculate num channels based on channel settings*/
   for(i=32; i>0; i--) {
      if(chan_settings[i] & (1<<8)) {	 
	 m->num_channels = i;
	 break;
      }
   }
#endif
   m->num_channels = 32;
   
   ins_para = (long *)calloc(insnum, sizeof(long));
   pat_para = (long *)calloc(insnum, sizeof(long));
   fheader_offset = 0x60 + ordnum + (2*insnum) + (2*patnum);
   fprintf(stdout, "fheader_offset: %ld\n", fheader_offset);
   
   for(i=0; i<insnum; i++) {
      fread(&sdummy, sizeof(unsigned short), (size_t)1, f);
      ins_para[i] = (long)(sdummy) * 16;
      fprintf(stdout, "ins_para[%d]: %ld\n", i, ins_para[i]);
   }
   for(i=0; i<patnum; i++) {
      fread(&sdummy, sizeof(unsigned short), (size_t)1, f);
      pat_para[i] = (long)(sdummy) * 16 + fheader_offset;
      fprintf(stdout, "pat_para[%d]: %ld\n", i, pat_para[i]);
   }
   
   
   load_s3m_instruments_all(m, ins_para, f);

   m->patterns = load_s3m_patterns(m->num_patterns, m->num_channels, pat_para, f);

   
   fclose(f);

/*
   update_modinfo();
  */

   
   return(m);
}

int test_s3m(char *filename) {
   FILE *f;
   char one_a, type;
   
   f = fopen(filename, "r");
   
   fseek(f, 0x1c, SEEK_SET);
   
   fread(&one_a, sizeof(char), (size_t)1, f);
   fread(&type, sizeof(char), (size_t)1, f);
   
   fclose(f);
   
   return((one_a==0x1a)&&(type==16));
}
