/*
 * xm.c
 * 
 * XM file reader
 * 
 * Copyright (C) 1996 Conrad Parker
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "aube.h"
#include "aube_internal.h"

struct xm_channel_note *load_xm_channel_note(FILE *f)
{
   char compression;
   struct xm_channel_note *ch_note;
   
   ch_note = (struct xm_channel_note *)malloc
     (sizeof(struct xm_channel_note));

   ch_note->note = 0;
   ch_note->instrument = 0;
   ch_note->volume = 0;
   ch_note->effect_type = 0;
   ch_note->effect_parameter = 0;
   
   fread(&compression, sizeof(char), (size_t)1, f);
   if(compression & 0x80) {
      if(compression & 0x01) {
	 fread(&ch_note->note, sizeof(char), (size_t)1, f);
      }
      if(compression & 0x02) {
	 fread(&ch_note->instrument, sizeof(char), (size_t)1, f);
      }
      if(compression & 0x04) {
	 fread(&ch_note->volume, sizeof(char), (size_t)1, f);
      }
      if(compression & 0x08) {
	 fread(&ch_note->effect_type, sizeof(char), (size_t)1, f);
      }
      if(compression & 0x10) {
	 fread(&ch_note->effect_parameter, sizeof(char), (size_t)1, f);
      }
   }
   else {
      ch_note->note = compression & 0x7F;
      fread(&ch_note->instrument, sizeof(char), (size_t)1, f);
      fread(&ch_note->volume, sizeof(char), (size_t)1, f);
      fread(&ch_note->effect_type, sizeof(char), (size_t)1, f);
      fread(&ch_note->effect_parameter, sizeof(char), (size_t)1, f);
   }

   return ch_note;
}


struct xm_channel_note_list *load_xm_channel_note_list(int num_channels, FILE *f)
{
   struct xm_channel_note_list *ret_channel_note_list, *channel_note_list,
     *prev_channel_note_list;
   int i;

   ret_channel_note_list = (struct xm_channel_note_list *)malloc(sizeof(struct xm_channel_note_list));
   
   prev_channel_note_list = NULL;
   
   for(i=0; i<num_channels; i++) {
      channel_note_list = (struct xm_channel_note_list *)malloc(sizeof(struct xm_channel_note_list));
      
      channel_note_list->ch_note = load_xm_channel_note(f);

      if(prev_channel_note_list) {
	 prev_channel_note_list->next = channel_note_list;
      }
      else {
	 ret_channel_note_list = channel_note_list;
      }
      
      prev_channel_note_list = channel_note_list;
   }
   prev_channel_note_list->next = NULL;
   
   return ret_channel_note_list;
}

struct xm_pattern *load_xm_pattern(int header_length, char packing_type,
				   int num_rows, int num_channels,
				   int patterndata_size, FILE *f)
{
   struct xm_pattern *ret_pattern;
   struct xm_row_list *row_list, *prev_row_list;
   int i;

   ret_pattern = (struct xm_pattern *)malloc(sizeof(struct xm_pattern));

   ret_pattern->header_length = header_length;
   ret_pattern->packing_type = packing_type;
   ret_pattern->num_rows = num_rows;
   ret_pattern->patterndata_size = patterndata_size;
	 
   row_list = NULL;
   prev_row_list = NULL;
   
   for(i=0; i<num_rows; i++) {
      row_list = (struct xm_row_list *)malloc(sizeof(struct xm_row_list));
      row_list->row = load_xm_channel_note_list(num_channels, f);
      
      if(prev_row_list) {
	 prev_row_list->next = row_list;
      }
      else {
	 ret_pattern->rows = row_list;
      }
      
      prev_row_list = row_list;
   }   
   prev_row_list->next = NULL;
   
   return ret_pattern;
}

struct xm_pattern_list *load_xm_patterns(int num_patterns, int num_channels,
					 FILE *f)
{
   struct xm_pattern_list *ret_pattern_list=NULL, *pattern_list, *prev_pattern_list;
   int i;
   int header_length;
   char packing_type;
   short num_rows;
   short patterndata_size;   

   int ftell_start, ftell_end;

   pattern_list = NULL;
   prev_pattern_list = NULL;
   
   for(i=0; i<num_patterns; i++) {
      fread(&header_length, sizeof(int), (size_t)1, f);
      
      fread(&packing_type, sizeof(char), (size_t)1, f);
      
      fread(&num_rows, sizeof(short), (size_t)1, f);
      
      fread(&patterndata_size, sizeof(short), (size_t)1, f);

      if(patterndata_size > 0) {
	 
	 ftell_start = ftell(f);
	 
	 pattern_list = (struct xm_pattern_list *)malloc(sizeof(struct xm_pattern_list));
	 
	 pattern_list->pattern = load_xm_pattern(header_length, packing_type,
						 num_rows, num_channels,
						 patterndata_size, f);

/*	 
	 set_current_pattern(pattern_list->pattern);
*/	 
	 ftell_end = ftell(f);
      }
      
      if(prev_pattern_list) {
	 prev_pattern_list->next = pattern_list;
      }
      else {
	 ret_pattern_list = pattern_list;
      }
      
      prev_pattern_list = pattern_list;
   }
   if(prev_pattern_list != NULL) {   
      prev_pattern_list->next = NULL;
   }
   
   return ret_pattern_list;
}

void load_xm_sample_data(struct xm_sample *sample, FILE *f)
{
   int i;
   unsigned short p;
   unsigned char d;
   bit16 ds;
   bit16 *s;

   
   if(sample->type & 0x10) {
      /* 16 bit sample */
      sample->data = (bit16 *)malloc((size_t)sample->sample_length + SAMPLE_EXTEND);
      sample->postloop = NULL;

      s = sample->data;
      p = 0x00;

      for(i=0; i<(sample->sample_length>>1); i++) {
	 fread(&ds, sizeof(bit16), (size_t)1, f);
	 p += ds;
	 *(s++) = p;
      }
   }
   else {
      /* 8 bit sample */
      sample->sample_length <<= 1;
      sample->sample_loop_start <<= 1;
      sample->sample_loop_length <<= 1;
      
      sample->data = (bit16 *)malloc((size_t)(sample->sample_length) + SAMPLE_EXTEND);
      sample->postloop = NULL;
	 
      s = sample->data;
      
      p= 0x00;
	 
      for(i=0; i<sample->sample_length>>1; i++) {
	 fread(&d, sizeof(unsigned char), (size_t)1, f);
	 p += (unsigned short)d<<8;
	 *(s++) = p;
      }
   }

   preextend(sample);
}


struct xm_sample *load_xm_sample(FILE *f)
{
   struct xm_sample *sample;
   
   sample = (struct xm_sample *)malloc(sizeof(struct xm_sample));

   fread(&sample->sample_length, sizeof(int), (size_t)1, f);
   
   fread(&sample->sample_loop_start, sizeof(int), (size_t)1, f);
   fread(&sample->sample_loop_length, sizeof(int), (size_t)1, f);
   
   fread(&sample->volume, sizeof(char), (size_t)6, f);

   fread(sample->sample_name, sizeof(char), (size_t)22, f);
   sample->sample_name[22] = '\0';

   return sample;
}  

struct xm_sample **load_xm_samples(int num_samples, FILE *f)
{
   struct xm_sample **samples;
   int i;
   
   samples = (struct xm_sample **)calloc(num_samples, sizeof(struct xm_sample *));
   
   for(i=0; i<num_samples; i++) {
      samples[i] = load_xm_sample(f);
   }


   /* load sample data */
   for(i=0; i<num_samples; i++) {
      if(samples[i]->sample_length == 0) {
	 samples[i]->data = NULL;
      }
      else {
	 load_xm_sample_data(samples[i], f);
      }
   }
   
   
   return samples;
}


struct xm_instrument *load_xm_instrument(FILE *f)
{
   struct xm_instrument *ret_instrument;
   int dummy;
   int ftell_start, ftell_end;
   
   ret_instrument = (struct xm_instrument *)malloc(sizeof(struct xm_instrument));

   ftell_start = ftell(f);
   
   fread(&ret_instrument->instrument_size, sizeof(int), (size_t)1, f);

   fread(ret_instrument->instrument_name, sizeof(char), (size_t)22, f);
   ret_instrument->instrument_name[22] = '\0';

   fread(&ret_instrument->instrument_type, sizeof(char),
	 (size_t)1, f);
   fread(&ret_instrument->num_samples, sizeof(short), (size_t)1, f);

   if(ret_instrument->num_samples == 0) {
      ret_instrument->header_size = 0;
      ret_instrument->samples = NULL;
      fread(&dummy, (size_t)4, (size_t)1, f);
   }
   else {
      fread(&ret_instrument->header_size, sizeof(int), (size_t)1, f);
      /*
       fprintf(stdout, "\tHEADER SIZE: %02X\n", ret_instrument->header_size);
       */ 
      fread(ret_instrument->sample_number, sizeof(char), (size_t)96, f);
      fread(ret_instrument->vol_envlp, sizeof(char), (size_t)48, f);
      fread(ret_instrument->pan_envlp, sizeof(char), (size_t)48, f);
      fread(&ret_instrument->num_vol_pts, sizeof(char), (size_t)38, f);
      
      ret_instrument->samples = load_xm_samples(ret_instrument->num_samples, f);
   }

   ftell_end = ftell(f);

   /*
   if(ret_instrument->instrument_size != ftell_end - ftell_start) {
      fprintf(stdout, "\tread %u/%u ... resyncing\n", ftell_end - ftell_start,
	      ret_instrument->instrument_size);
      fseek(f, ftell_start + ret_instrument->instrument_size, SEEK_SET);
   }
    */
   
   return ret_instrument;
      
}

struct xm_instrument *dummy_xm_instrument(void)
{
   struct xm_instrument *ret_instrument;
   
   ret_instrument = (struct xm_instrument *)malloc(sizeof(struct xm_instrument));

   ret_instrument->instrument_name[0] = '\0';
   ret_instrument->instrument_type = 0;
   ret_instrument->num_samples = 0;
   
   return(ret_instrument);
}


/*
struct xm_instrument_list *load_xm_instruments(int num_instruments, FILE *f)
{
   struct xm_instrument_list *ret_instrument_list, 
     *instrument_list, *prev_instrument_list=NULL;
   int i;
   
   ret_instrument_list = NULL;
   prev_instrument_list = NULL;
   instrument_list = NULL;


   for(i=0; i<num_instruments; i++) {
      instrument_list = (struct xm_instrument_list *)
	malloc(sizeof(struct xm_instrument_list));
      
      instrument_list->instrument = load_xm_instrument(f);

      if(prev_instrument_list) {
	 prev_instrument_list->next = instrument_list;
      }
      else {
	 ret_instrument_list = instrument_list;
      }

      prev_instrument_list = instrument_list;
   }
   
   for(;i<128; i++) {
      instrument_list = (struct xm_instrument_list *)
	malloc(sizeof(struct xm_instrument_list));
      
      instrument_list->instrument = dummy_xm_instrument();
      
      if(prev_instrument_list) {
	 prev_instrument_list->next = instrument_list;
      }
      else {
	 ret_instrument_list = instrument_list;
      }
      
      prev_instrument_list = instrument_list;
   }
   
   prev_instrument_list->next = NULL;      
   
   return ret_instrument_list;
}
*/

void load_xm_instruments_all(struct xm *m, FILE *f)
{
   int i;
   
   for(i=0; i < m->num_instruments; i++) {      
      m->instruments[i] = load_xm_instrument(f);
   }
   
   for(;i<=128; i++) {
      m->instruments[i] = dummy_xm_instrument();
   }
   
/*   update_instrlist();*/
}
      
struct xm *load_xm(char *filename)
{
   FILE *f;
   struct xm *m;
   
   f = fopen(filename, "r");
   
   m = (struct xm *)malloc(sizeof(struct xm));
   
   fread(m->ID_text, sizeof(char), (size_t)17, f);
   m->ID_text[17] = '\0';
   
   fread(m->mod_name, sizeof(char), (size_t)20, f);
   m->mod_name[20] = '\0';

   fread(&m->one_a, sizeof(char), (size_t)1, f);
   fread(m->tracker_name, sizeof(char), (size_t)20, f);
   m->tracker_name[20] = '\0';
   
   fread(&m->version_low, sizeof(char), (size_t)2, f);

   if(m->version_low != 4 && m->version_high != 1) {
      fprintf(stderr, "Unknown version %d.%d -- aborting\n", m->version_high, m->version_low);
      return(NULL);
   }
   
   fread(&m->header_size, sizeof(int), (size_t)1, f);
   fread(&m->song_length, sizeof(short), (size_t)8, f);

   /*
   update_modinfo();
   */
   
   fread(&m->pattern_order_table, sizeof(char), (size_t)256, f);

   /*
   update_patt_list();
   */
   
   m->patterns = load_xm_patterns(m->num_patterns, m->num_channels, f);

   load_xm_instruments_all(m, f);
   
   fclose(f);

/*
   update_modinfo();
  */
   
   return(m);
}

int test_xm(char *filename) {
   FILE *f;
   char id_text[17];
   
   f = fopen(filename, "r");
   
   fread(id_text, sizeof(char), (size_t)17, f);
   
   fclose(f);
   
   return(memcmp(id_text, (char *)"Extended Module: ", 17) == 0);
}

