/*
 * xm.c
 * 
 * XM file reader
 * 
 * Copyright (C) 1996 Conrad Parker
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "aube.h"


extern struct xm *mod;
extern struct xm_pattern *current_pattern;
extern struct xm_instrument *current_instrument;
extern int current_track;
extern int current_row;
extern int current_pos;

struct xm_channel_note *empty_xm_channel_note(void)
{
   struct xm_channel_note *ch_note;
   
   ch_note = (struct xm_channel_note *)malloc
     (sizeof(struct xm_channel_note));

   ch_note->note = 0;
   ch_note->instrument = 0;
   ch_note->volume = 0;
   ch_note->effect_type = 0;
   ch_note->effect_parameter = 0;
   
   return ch_note;
}


struct xm_channel_note_list *empty_xm_channel_note_list(int num_channels)
{
   struct xm_channel_note_list *ret_channel_note_list, *channel_note_list,
     *prev_channel_note_list;
   int i;

   ret_channel_note_list = (struct xm_channel_note_list *)malloc(sizeof(struct xm_channel_note_list));
   
   prev_channel_note_list = NULL;
   
   for(i=0; i<num_channels; i++) {
      channel_note_list = (struct xm_channel_note_list *)malloc(sizeof(struct xm_channel_note_list));
      
      channel_note_list->ch_note = empty_xm_channel_note();

      if(prev_channel_note_list) {
	 prev_channel_note_list->next = channel_note_list;
      }
      else {
	 ret_channel_note_list = channel_note_list;
      }
      
      prev_channel_note_list = channel_note_list;
   }
   prev_channel_note_list->next = NULL;
   
   return ret_channel_note_list;
}

struct xm_pattern *empty_xm_pattern(int num_rows, int num_channels)
{
   struct xm_pattern *ret_pattern;
   struct xm_row_list *row_list, *prev_row_list;
   int i;

   ret_pattern = (struct xm_pattern *)malloc(sizeof(struct xm_pattern));
   ret_pattern->num_rows = num_rows;
   
   row_list = NULL;
   prev_row_list = NULL;
   
   for(i=0; i<num_rows; i++) {
      row_list = (struct xm_row_list *)malloc(sizeof(struct xm_row_list));
      row_list->row = empty_xm_channel_note_list(num_channels);
      
      if(prev_row_list) {
	 prev_row_list->next = row_list;
      }
      else {
	 ret_pattern->rows = row_list;
      }
      
      prev_row_list = row_list;
   }   
   prev_row_list->next = NULL;
   
   return ret_pattern;
}

struct xm_pattern_list *empty_xm_patterns(int num_patterns, int num_channels)
{
   struct xm_pattern_list *ret_pattern_list, *pattern_list, *prev_pattern_list;
   struct xm_pattern pattern;
   
   int i;
   int header_length;
   char packing_type;
   short num_rows = 64;

   int ftell_start, ftell_end;

   pattern_list = NULL;
   prev_pattern_list = NULL;
   
   for(i=0; i<num_patterns; i++) {

      pattern_list = (struct xm_pattern_list *)malloc(sizeof(struct xm_pattern_list));
      pattern_list->pattern = empty_xm_pattern(num_rows, num_channels);
      
      if(prev_pattern_list) {
	 prev_pattern_list->next = pattern_list;
      }
      else {
	 ret_pattern_list = pattern_list;
      }
      
      prev_pattern_list = pattern_list;
   }
   prev_pattern_list->next = NULL;
   
   return ret_pattern_list;
}


struct xm_sample *empty_xm_sample(void)
{
   struct xm_sample *sample;
   int i;
   unsigned short p;
   unsigned char d;
   unsigned short ds;
   unsigned short *s;
   int dummy;
   
   sample = (struct xm_sample *)malloc(sizeof(struct xm_sample));

   sample->sample_length = 0;
   sample->data = NULL;
   
   return sample;
}

struct xm_sample **empty_xm_samples(int num_samples)
{
   struct xm_sample **samples;
   int i;
   int ftell_start, ftell_end;
   
   samples = (struct xm_sample **)calloc(num_samples, sizeof(struct xm_sample *));
   
   for(i=0; i<num_samples; i++) {
      samples[i] = empty_xm_sample();
   }
   
   return samples;
}


struct xm_instrument *empty_xm_instrument(void)
{
   struct xm_instrument *ret_instrument;
   size_t rest;
   int i;
   int dummy;
   
   ret_instrument = (struct xm_instrument *)malloc(sizeof(struct xm_instrument));

   ret_instrument->instrument_size = 0;
   ret_instrument->instrument_name[0] = '\0';
   ret_instrument->instrument_type = 0;
   ret_instrument->num_samples = 1;
   ret_instrument->header_size = 0;
   ret_instrument->samples = empty_xm_samples(1);
      
   return ret_instrument;
      
}

/* dummy_xm_instrument() is still of course defined in xm.h
struct xm_instrument *dummy_xm_instrument(void)
{
   struct xm_instrument *ret_instrument;
   
   ret_instrument = (struct xm_instrument *)malloc(sizeof(struct xm_instrument));

   ret_instrument->instrument_name[0] = '\0';
   ret_instrument->instrument_type = 0;
   ret_instrument->num_samples = 0;
   
   return(ret_instrument);
}
*/


void empty_xm_instruments(struct xm *m, int num_instruments)
{
   int i;
   int ftell_start, ftell_end;
   int i_size;
   struct xm_sample_list *sl;
   
   for(i=0; i<128; i++) {
      m->instruments[i] = empty_xm_instrument();
   }
}

      
struct xm *empty_xm()
{
   int i;
   
   mod = (struct xm *)malloc(sizeof(struct xm));
   
   mod->ID_text[0] = '\0';
   
   strcpy(mod->mod_name, (char *)"Untitled");

   mod->one_a = 0x1a;

   strcpy(mod->tracker_name, (char *)"maube");
   
   mod->version_low = 4;
   mod->version_high = 1;

   mod->header_size = 0;
   mod->song_length = 0;
   mod->restart_position = 0;
   mod->num_channels = 8;
   mod->num_patterns = 1;
   mod->num_instruments = 0;
   mod->bpm = 162;
   mod->tempo = 6;
   
   for(i=0; i<256; i++) {   
      mod->pattern_order_table[i] = 0;
   }
   
   mod->patterns = empty_xm_patterns(mod->num_patterns, mod->num_channels);

   empty_xm_instruments(mod, mod->num_instruments);

/*   current_pattern = mod->patterns->pattern;
   current_instrument = mod->instruments[0];
*/
/*
   current_track = 0;
   current_row = 0;
   current_pos = 0;
*/      
   return mod;
}

