/*
 * xm_free.c
 * 
 * Copyright (C) 1996-1997 Conrad Parker
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "aube.h"

extern struct xm *mod;
extern struct xm_pattern *current_pattern;
extern struct xm_instrument *current_instrument;
extern int current_instrument_number;
extern int current_track;
extern int current_row;
extern int current_pos;

void free_xm_channel_note(struct xm_channel_note *ch_note)
{
   free(ch_note);
}


void free_xm_channel_note_list(struct xm_channel_note_list *channel_note_list,
			       int num_channels)
{
   struct xm_channel_note_list *temp_channel_note_list;
   int i;
   
   temp_channel_note_list = NULL;
   
   for(i=0; i<num_channels && channel_note_list; i++) {
      temp_channel_note_list = channel_note_list->next;
      
      free_xm_channel_note(channel_note_list->ch_note);

      free(channel_note_list);
      
      channel_note_list = temp_channel_note_list;
   }
}

void free_xm_pattern(struct xm_pattern *pattern, int num_channels)
{
   struct xm_row_list *row_list, *temp_row_list;
   int i;
   int num_rows;
   
   num_rows = pattern->num_rows;
   
   row_list = pattern->rows;
   
   for(i=0; i<num_rows; i++) {
      temp_row_list = row_list->next;
      
      if(row_list == NULL) {
	 fprintf(stderr, "NULL row_list\n");
      }
      else {	 
	 free_xm_channel_note_list(row_list->row, num_channels);
	 free(row_list);
      }
      
      row_list = temp_row_list;
   }   
   
   free(pattern);
}

void free_xm_patterns(struct xm_pattern_list *pattern_list, int num_patterns,
		      int num_channels)
{
   struct xm_pattern_list *temp_pattern_list;
   
   int i;

   
   for(i=0; i<num_patterns; i++) {
      if(!pattern_list) break;
      temp_pattern_list = pattern_list->next;

      free_xm_pattern(pattern_list->pattern, num_channels);
      free(pattern_list);
      
      pattern_list = temp_pattern_list;
   }

}


void free_xm_sample(struct xm_sample *sample)
{
   free(sample->data);
   free(sample);
}

void free_xm_samples(struct xm_sample **samples, int num_samples)
{
   int i;
   
   for(i=0; i<num_samples; i++) {
      free_xm_sample(samples[i]);
   }
   free(samples);
}


void free_xm_instrument(struct xm_instrument *instrument)
{
   if(instrument) {  
      free_xm_samples(instrument->samples, instrument->num_samples);
      free(instrument);
   }   
}


void free_xm_instruments(struct xm_instrument **instruments,
			 int num_instruments)
{
   int i;
   
   for(i=0; i<=128; i++) {
      free_xm_instrument(instruments[i]);
   }
}

void free_xm_patterns_all(struct xm *m)
{
   free_xm_patterns(m->patterns, m->num_patterns, m->num_channels);
   mod->num_patterns = 0;

   mod->patterns = empty_xm_patterns(mod->num_patterns, mod->num_channels);
/*   current_pattern = mod->patterns->pattern;*/
   
   /*
   update_modinfo();
    */
   
   
/*   update_tracker();*/
}

void free_xm_instruments_all(struct xm *m)
{
   free_xm_instruments(m->instruments, m->num_instruments);
   mod->num_instruments = 0;

   empty_xm_instruments(mod->num_instruments);
/*   current_instrument = mod->instruments[0];
   current_instrument_number = 0;
*/   
   /*
   update_modinfo();
    */
   
/*   update_instrlist();*/
}

void free_xm(struct xm *m)
{
   free_xm_patterns(m->patterns, m->num_patterns, m->num_channels);
   free_xm_instruments(m->instruments, m->num_instruments);

   free(m);
}

void clear_xm(struct xm *m)
{
   free_xm(m);
   
   m = empty_xm();
   
   /*
   update_modinfo();
   */
    
/*   update_instrlist();*/
/*   update_tracker();*/
}

