/*
 * xm_save.c
 * 
 * XM file saver
 * 
 * Copyright (C) 1996 Conrad Parker
 */

#include <stdio.h>
#include <string.h>

#include "aube.h"

int get_xm_channel_note_size(struct xm_channel_note *ch_note)
{
   char entries = 0;
   
   entries = ((ch_note->note==0)?0:1) + ((ch_note->instrument==0)?0:1) +
     ((ch_note->volume==0)?0:1) + ((ch_note->effect_type==0)?0:1) +
     ((ch_note->effect_parameter==0)?0:1);
   
   if(entries < 4) {
      return (entries + 1);
   }
   else {
      return 5;
   }
}


void save_xm_channel_note(struct xm_channel_note *ch_note, FILE *f)
{
   char compression;
   char entries = 0; /* number of nonzero entries */
   
   entries = ((ch_note->note==0)?0:1) + ((ch_note->instrument==0)?0:1) +
     ((ch_note->volume==0)?0:1) + ((ch_note->effect_type==0)?0:1) +
     ((ch_note->effect_parameter==0)?0:1);
   
   if(entries < 4) {
      compression = 0x80;
      if(ch_note->note != 0)
	compression |= 0x01;
      if(ch_note->instrument != 0)
	compression |= 0x02;
      if(ch_note->volume != 0)
	compression |= 0x04;
      if(ch_note->effect_type != 0)
	compression |= 0x08;
      if(ch_note->effect_parameter != 0)
	compression |= 0x10;
      
      fwrite(&compression, sizeof(char), (size_t)1, f);

      if(compression & 0x01) {
	 fwrite(&ch_note->note, sizeof(char), (size_t)1, f);
      }
      if(compression & 0x02) {
	 fwrite(&ch_note->instrument, sizeof(char), (size_t)1, f);
      }
      if(compression & 0x04) {
	 fwrite(&ch_note->volume, sizeof(char), (size_t)1, f);
      }
      if(compression & 0x08) {
	 fwrite(&ch_note->effect_type, sizeof(char), (size_t)1, f);
      }
      if(compression & 0x10) {
	 fwrite(&ch_note->effect_parameter, sizeof(char), (size_t)1, f);
      }
   }
   else {   
      fwrite(&ch_note->note, sizeof(char), (size_t)1, f);
      fwrite(&ch_note->instrument, sizeof(char), (size_t)1, f);
      fwrite(&ch_note->volume, sizeof(char), (size_t)1, f);
      fwrite(&ch_note->effect_type, sizeof(char), (size_t)1, f);
      fwrite(&ch_note->effect_parameter, sizeof(char), (size_t)1, f);
   }

}


void save_xm_channel_note_list(struct xm_channel_note_list *channel_note_list,
			       int num_channels, FILE *f)
{
   int i;
   
   for(i=0; i<num_channels; i++) {
      
      save_xm_channel_note(channel_note_list->ch_note, f);

      channel_note_list = channel_note_list->next;
   }
}

int get_xm_channel_note_list_size(struct xm_channel_note_list *row,
				  int num_channels)
{
   int i;
   int ch_note_list_size = 0;
   
   for(i=0; i<num_channels; i++) {
      ch_note_list_size += get_xm_channel_note_size(row->ch_note);
      
      row = row->next;
   }
   
   return ch_note_list_size;
}


int get_xm_patterndata_size(struct xm_pattern *pattern, int num_channels)
{
   int i;
   int patterndata_size = 0;
   struct xm_row_list *row_list;
   
   row_list = pattern->rows;
   for(i=0; i<pattern->num_rows; i++) {
      patterndata_size +=
	get_xm_channel_note_list_size(row_list->row, num_channels);
      
      row_list = row_list->next;
   }
   
   return patterndata_size;
}

void save_xm_pattern(struct xm_pattern *pattern,
		     int num_channels, FILE *f)
{
   struct xm_row_list *row_list;
   int i;

   fwrite(&pattern->header_length, sizeof(int), (size_t)1, f);
   
   fwrite(&pattern->packing_type, sizeof(char), (size_t)1, f);
   
   fwrite(&pattern->num_rows, sizeof(short), (size_t)1, f);
      
   pattern->patterndata_size = get_xm_patterndata_size(pattern, num_channels);
   fwrite(&pattern->patterndata_size, sizeof(short), (size_t)1, f);
   
   
   row_list = pattern->rows;
   for(i=0; i<pattern->num_rows; i++) {
      save_xm_channel_note_list(row_list->row, num_channels, f);
      
      row_list = row_list->next;
   }   
}

void save_xm_patterns(struct xm_pattern_list *pattern_list, int num_patterns,
		      int num_channels, FILE *f)
{
   int i;

   for(i=0; i<num_patterns; i++) {
      save_xm_pattern(pattern_list->pattern, num_channels, f);
      
      pattern_list = pattern_list->next;
   }
}


void save_xm_sample_header(struct xm_sample *sample, FILE *f)
{
   fwrite(&sample->sample_length, sizeof(int), (size_t)1, f);
   
   fwrite(&sample->sample_loop_start, sizeof(int), (size_t)1, f);
   fwrite(&sample->sample_loop_length, sizeof(int), (size_t)1, f);
   
   fwrite(&sample->volume, sizeof(char), (size_t)6, f);

   fwrite(sample->sample_name, sizeof(char), (size_t)22, f);
}

void save_xm_sample_data(struct xm_sample *sample, FILE *f)
{
   int i;
   unsigned short p, op;
   unsigned char d;
   unsigned short ds;
   unsigned short *s;
   
   
   if(!sample->data) return;
   
   if(sample->type & 0x10) {
      s = sample->data;

      p = 0x00;
      op = 0x00;
      for(i=0; i<sample->sample_length/2; i++) {
	 p = *(s++);
	 ds = p - op;
	 op = p;
	 fwrite(&ds, sizeof(unsigned short), (size_t)1, f);
      }  
   }
   else {
      s = sample->data;

      p = 0x00;
      op = 0x00;
      for(i=0; i<sample->sample_length; i++) {
	 p = *(s++);
	 d = (unsigned char)((p - op)>>8);
	 op = p;
	 fwrite(&d, sizeof(unsigned char), (size_t)1, f);
      }
   }

}  

void save_xm_samples(struct xm_sample **samples, int num_samples, FILE *f)
{
   int i;

   for(i=0; i<num_samples; i++) {
      save_xm_sample_header(samples[i], f);
   }
   for(i=0; i<num_samples; i++) {
      save_xm_sample_data(samples[i], f);
   }
}


void save_xm_instrument(struct xm_instrument *instrument, FILE *f)
{
   int dummy=0;

   fwrite(&instrument->instrument_size, sizeof(int), (size_t)1, f);

   fwrite(instrument->instrument_name, sizeof(char), (size_t)22, f);
   
   fwrite(&instrument->instrument_type, sizeof(char),
	 (size_t)1, f);
   fwrite(&instrument->num_samples, sizeof(short), (size_t)1, f);
   
   if(instrument->num_samples == 0) {
      fwrite(&dummy, (size_t)4, (size_t)1, f);
   }
   else {
      fwrite(&instrument->header_size, sizeof(int), (size_t)1, f);
      fwrite(instrument->sample_number, sizeof(char), (size_t)96, f);
      fwrite(instrument->vol_envlp, sizeof(char), (size_t)48, f);
      fwrite(instrument->pan_envlp, sizeof(char), (size_t)48, f);
      fwrite(&instrument->num_vol_pts, sizeof(char), (size_t)38, f);
      
      save_xm_samples(instrument->samples, instrument->num_samples, f);
   }

}

void save_xm_instruments_all(struct xm *m, FILE *f)
{
   int i;

   for(i=0; i < m->num_instruments; i++) {
      save_xm_instrument(m->instruments[i], f);
   }

}
      
void save_xm(struct xm *m, char *filename)
{
   FILE *f;
   char tracker_name[21];
   
   strncpy(tracker_name, (char *)"maube v0.01a", 20);

   printf("saving XM file: %s\n", filename);

   f = fopen(filename, "w");
   
   fwrite(m->ID_text, sizeof(char), (size_t)17, f);
   
   fwrite(m->mod_name, sizeof(char), (size_t)20, f);

   fwrite(&m->one_a, sizeof(char), (size_t)1, f);

   fwrite(tracker_name, sizeof(char), (size_t)20, f);
   
   fwrite(&m->version_low, sizeof(char), (size_t)2, f);

   fwrite(&m->header_size, sizeof(int), (size_t)1, f);
   fwrite(&m->song_length, sizeof(short), (size_t)8, f);

   fwrite(&m->pattern_order_table, sizeof(char), (size_t)256, f);
   
   save_xm_patterns(m->patterns, m->num_patterns, m->num_channels, f);

   save_xm_instruments_all(m, f);
   
   fclose(f);

}

