#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include <aube.h>

#include "control.h"

#include "button.h"
#include "toggle.h"
#include "panel.h"
#include "scrollbar.h"
#include "numrange.h"
#include "textfield.h"
#include "label.h"
#include "textlist.h"

extern char *progname;

extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern struct xm *mod;
extern struct xm_pattern *current_pattern;
extern int current_instrument_number;

extern GC top_gc;

extern Window instr_window;
extern Window tracker_top;
extern Window saube_top;

extern void prev_instrument(void);
extern void next_instrument(void);

extern struct toggle_data *current_play_toggle;
extern struct toggle_data *play_pattern_toggle;

extern int edit;

extern int player_pid;
extern int playing;


Window control_top;
struct toggle_data *current_play_toggle; /* "play" toggle button which
					  * started current play */

int song_pos=0;

static struct textfield *modname_textfield;

static struct panel *song_panel;
static struct panel *data_panel;
static struct panel *file_panel;
static struct panel *play_panel;
static struct panel *wins_panel;
static struct panel *instr_panel;

static struct button_data *inc_patt_button;
static struct button_data *dec_patt_button;
static struct button_data *ins_patt_button;
static struct button_data *del_patt_button;
static struct button_data *quit_button;
static struct button_data *load_button;
static struct button_data *save_button;
static struct button_data *zap_button;
static struct button_data *song_rew_button;
static struct button_data *song_fwd_button;
static struct button_data *stop_button;
static struct button_data *load_raw_button;

static struct toggle_data *tracker_toggle;
static struct toggle_data *saube_toggle;
static struct toggle_data *play_song_toggle;

static struct numrange *song_pos_numrange;
static struct numrange *restart_pos_numrange;
static struct numrange *bpm_numrange;
static struct numrange *spd_numrange;
static struct numrange *pattern_numrange;
static struct numrange *instrument_numrange;

static struct label *song_pos_label;
static struct label *restart_pos_label;
static struct label *status_label;
static struct label *bpm_label;
static struct label *spd_label;
static struct label *pattern_label;
static struct label *instrument_label;

static struct textlist *patt_list;
static struct textlist *instr_list;

void inc_patt_button_pressed(void)
{
   mod->pattern_order_table[song_pos]++;
   update_patt_list();
}

void dec_patt_button_pressed(void)
{
   mod->pattern_order_table[song_pos]--;
   update_patt_list();
}

void ins_patt_button_pressed(void)
{
   int i;
   
   for(i = mod->song_length++; i > song_pos; i--) {
      mod->pattern_order_table[i] = mod->pattern_order_table[i-1];
   }
   mod->pattern_order_table[song_pos] = get_current_pattern_number();

   update_patt_list();
}

void del_patt_button_pressed(void)
{
   int i;
   
   for(i = song_pos; i < mod->song_length; i++) {
      mod->pattern_order_table[i] = mod->pattern_order_table[i+1];
   }
   mod->song_length--;
   
   update_patt_list();
}

void load_raw_file_selected(char *filename)
{
   load_raw_sample(filename);
}

void load_raw_button_pressed(voi)
{
   create_filerequest(load_raw_file_selected);
}

void quit_button_pressed(void)
{
   clean_exit();
}

void load_file_selected(char *filename)
{
   load_over(filename);
}

void load_button_pressed(void)
{
   create_filerequest(load_file_selected);
}

void save_file_selected(char *filename)
{
   save_xm(mod, filename);
}

void save_button_pressed(void)
{
   create_filerequest(save_file_selected);
}

void zap_button_pressed(void)
{
   create_zap_window();
}

void tracker_toggle_pressed(void)
{
   if(tracker_top == 0L) {   
      create_tracker_window();
   }
}

void tracker_toggle_released(void)
{
   edit=0;
   if(tracker_top != 0L) {   
      close_tracker_window();
   }
}

void saube_toggle_pressed(void)
{
   if(saube_top == 0L) {
      create_saube_window();
   }
}

void saube_toggle_released(void)
{
   if(saube_top != 0L) {
      close_saube_window();
   }
}

void song_rew_button_pressed(void)
{
   song_pos--;
   if(song_pos < 0) song_pos = 0;
   change_selection(patt_list, song_pos);
}

void song_fwd_button_pressed(void)
{
   song_pos++;
   if(song_pos > mod->song_length) song_pos = mod->song_length;
   change_selection(patt_list, song_pos);
}



void play_song_button_pressed(void)
{
   int pid_status;
   
   if(player_pid > 0) {
      kill((pid_t)player_pid, SIGKILL);
      waitpid((pid_t)player_pid, &pid_status, 0);
      player_pid = -1;
      if(current_play_toggle && current_play_toggle != play_song_toggle) {
	 set_toggle_state(current_play_toggle, 0);
      }
   }

   playing=1;
   current_play_toggle = play_song_toggle;

   player_pid = fork();
   if(!player_pid) {
      play_mod(mod);
      exit(0);
   }
   else if (player_pid <= 0) {
      fprintf(stderr, "Unable to fork...\n");
   }

/*   start_sync();*/

}

void stop_button_pressed(void)
{
   int pid_status;

   if(playing) {
      
      kill((pid_t)player_pid, SIGKILL);
      waitpid((pid_t)player_pid, &pid_status, 0);
      player_pid = -1;
      playing=0;

      set_toggle_state(current_play_toggle, 0);
   }
   
/*   stop_sync();*/
}

void song_pos_changed(int new_pos)
{
   song_pos = new_pos;
   change_selection(patt_list, song_pos);
}

void restart_pos_changed(int new_pos)
{
   mod->restart_position = new_pos;
}

void bpm_changed(int new_bpm)
{
   mod->bpm = new_bpm;
}

void spd_changed(int new_spd)
{
   mod->tempo = new_spd;
}

void pattern_changed(int new_pattern)
{
   change_current_pattern_number(new_pattern);
}

void instrument_changed(int new_instrument)
{
   change_current_instrument_number(new_instrument);
}

void instrlist_changed(int instr_index)
{
   change_current_instrument_number(instr_index + 1);
}

void patt_list_changed(int patt_index)
{
   song_pos = patt_index;
   change_current_pattern_number(mod->pattern_order_table[patt_index]);
   update_tracker();
}

void update_modinfo(void)
{
   char control_title[256];
   
   sprintf(control_title, "maube: %s", mod->mod_name);
   change_window_title(control_top, control_title);
   
   change_textfield_text(modname_textfield, mod->mod_name);

   set_numrange_bounds(song_pos_numrange, song_pos, 0, mod->song_length);
   set_numrange_bounds(restart_pos_numrange, mod->restart_position, 0, mod->song_length);
   set_numrange(bpm_numrange, mod->bpm);
   set_numrange(spd_numrange, mod->tempo);
   set_numrange_bounds(pattern_numrange, get_current_pattern_number(), 0, mod->num_patterns-1);
   set_numrange_bounds(instrument_numrange, get_current_instrument_number()-1, 1, mod->num_instruments);
   
}

void dummy(void)
{
}

void modname_changed(char *s)
{
   strncpy(mod->mod_name, s, 22);
}


void create_control_window(void)
{
   char *window_name = "maube";
   char *icon_name = "maube";
   int button_width, button_height;
   int width, height;
   char ***patt_names;
   char ***instr_names;
   
   width = 600;
   height = 200;
   
   button_width = 50;
   button_height = 16;
   
   control_top = create_toplevel_window(window_name, icon_name,
					0, 0, width, height, 200, 100);


   patt_names = get_patt_names(mod);
   patt_list = create_textlist(control_top, 2, 2, 50, height-20,
			       (void *)patt_list_changed,
			       (void *)patt_list_changed,
			       (int)mod->song_length, 1,
			       patt_names);

   song_panel = create_panel(control_top, 54, 2, button_width + 10, button_height*3 +4);
   
   inc_patt_button = create_text_button(song_panel->window,
					8, 2, button_width/2, button_height,
					(void *)inc_patt_button_pressed,
					(char *)"+",
					(char *)"");
   
   dec_patt_button = create_text_button(song_panel->window,
					8 + button_width/2, 2, button_width/2, button_height,
					(void *)dec_patt_button_pressed,
					(char *)"-",
					(char *)"");

   ins_patt_button = create_text_button(song_panel->window,
					8, 2 + button_height, button_width, button_height,
					(void *)ins_patt_button_pressed,
					(char *)"ins",
					(char *)"stick one more in here!");

   del_patt_button = create_text_button(song_panel->window,
					8, 2 + 2*button_height, button_width, button_height,
					(void *)del_patt_button_pressed,
					(char *)"del",
					(char *)"screw this one off!");

   data_panel = create_panel(control_top, 54, 70, 126, 112);
   
   song_pos_label = create_label(data_panel->window, 8, 2, 66, 18, (char *)"song pos");
   restart_pos_label = create_label(data_panel->window, 8, 20, 66, 18, (char *)"restart");
   bpm_label = create_label(data_panel->window, 8, 38, 66, 18, (char *)"BPM");
   spd_label = create_label(data_panel->window, 8, 56, 66, 18, (char *)"spd");
   pattern_label = create_label(data_panel->window, 8, 74, 66, 18, (char *)"pattern");
   instrument_label = create_label(data_panel->window, 8, 92, 66, 18, (char *)"instrument");

   song_pos_numrange = create_numrange(data_panel->window, 74, 2, 50, 18,
				       song_pos, 0, mod->song_length,
				       (void *)song_pos_changed);

   restart_pos_numrange = create_numrange(data_panel->window, 74, 20, 50, 18,
					  mod->restart_position,
					  0, mod->song_length,
					  (void *)restart_pos_changed);
   
   bpm_numrange = create_numrange(data_panel->window, 74, 38, 50, 18,
				  mod->bpm, 0, 360, (void *)bpm_changed);
   
   spd_numrange = create_numrange(data_panel->window, 74, 56, 50, 18,
				  mod->tempo, 0, 16, (void *)spd_changed);
   
   pattern_numrange = create_numrange(data_panel->window, 74, 74, 50, 18,
				      get_current_pattern_number(),
				      0, (int)mod->num_patterns-1,
				      (void *)pattern_changed);
   
   instrument_numrange = create_numrange(data_panel->window, 74, 92, 50, 18,
					 get_current_instrument_number(),
					 1, (int)mod->num_instruments,
					 (void *)instrument_changed);

   instr_panel = create_panel(control_top,
			      184, 100, button_width +10, button_height +4);
   
   load_raw_button = create_text_button(instr_panel->window,
					8, 2,
					button_width, button_height,
					(void *)load_raw_button_pressed,
					(char *)"ld 16b/u",
					(char *)"Load 16 bit unsigned raw sample");
   
   file_panel = create_panel(control_top,
			     184, 132, 4*button_width +10, button_height+4);
   
   quit_button = create_text_button(file_panel->window,
				    8, 2,
				    button_width, button_height,
				    (void *)quit_button_pressed,
				    (char *)"quit",
				    (char *)"Bugger off then!");

   load_button = create_text_button(file_panel->window,
				    8 + button_width, 2,
				    button_width, button_height,
				    (void *)load_button_pressed,
				    (char *)"load",
				    (char *)"Load a new mod up ya!");

   save_button = create_text_button(file_panel->window,
			       8 + 2*button_width, 2,
			       button_width, button_height,
			       (void *)save_button_pressed,
			       (char *)"save",
			       (char *)"Save your spam for later!");

   zap_button = create_text_button(file_panel->window,
			      8 + 3*button_width, 2,
			      button_width, button_height,
			      (void *)zap_button_pressed,
			      (char *)"zap",
			      (char *)"Shove it up your arse and die!");

   play_panel = create_panel(control_top,
			     184, 158, 5*button_width/2 +10, button_height+4);
   
   play_song_toggle = create_play_toggle(play_panel->window,
					 8, 2,
					 button_width, button_height,
					 (void *)play_song_button_pressed,
					 (void *)stop_button_pressed,
					 (char *)"Play with everyone");

   song_rew_button = create_rew_button(play_panel->window,
				       button_width+8, 2,
				       button_width/2, button_height,
				       (void *)song_rew_button_pressed,
				       (char *)"rewind song");

   song_fwd_button = create_fwd_button(play_panel->window,
				       3*button_width/2 +8, 2,
				       button_width/2, button_height,
				       (void *)song_fwd_button_pressed,
				       (char *)"forward song");

   stop_button = create_stop_button(play_panel->window,
				    2*button_width +8, 2,
				    button_width/2, button_height,
				    (void *)stop_button_pressed,
				    (char *)"Stop this nonsense!");

   wins_panel = create_panel(control_top, 260, 100,
			     2 * button_width + 10, button_height+4);
   
   tracker_toggle = create_text_toggle(wins_panel->window,
				       8, 2,
				       button_width, button_height,
				       (void *)tracker_toggle_pressed,
				       (void *)tracker_toggle_released,
				       (char *)"tracker",
				       (char *)"Go on, fuck around with the tracks then!");
   
   saube_toggle = create_text_toggle(wins_panel->window,
				     8 + button_width, 2,
				     button_width, button_height,
				     (void *)saube_toggle_pressed,
				     (void *)saube_toggle_released,
				     (char *)"saube",
				     (char *)"saube - a sample editor for aube :)");
   
   
   modname_textfield = create_textfield(control_top, 438, 2, 160, 16,
					(void *)modname_changed,
					mod->mod_name, LEFT_JUSTIFY);
   
   instr_names = get_instr_names(mod);
   instr_list = create_textlist(control_top, 438, 18, 160, height - 36, 
				(void *)instrlist_changed,
				(void *)instrlist_changed,
				mod->num_instruments, 1,
				instr_names);
   
   
   status_label = create_label(control_top,
				   0, height - 16,
				   width, 16,
				   (char *)"welcome to maube...");

   XMapWindow(display, control_top);
}

char ***get_instr_names(struct xm *mod)
{
   char **namesv;
   char ***names;
   int i;
   
   namesv = (char **)malloc((size_t)mod->num_instruments * sizeof(char *));
   names = (char ***)malloc(sizeof(char ***));
   
   for(i=0; i<mod->num_instruments; i++) {
      namesv[i] = mod->instruments[i]->instrument_name;
   }
   names[0] = namesv;
   
   return names;
}

char ***get_patt_names(struct xm *mod)
{
   char **namesv;
   char ***names;
   int i;
   
   namesv = (char **)malloc((size_t)mod->song_length * sizeof(char *));
   names = (char ***)malloc(sizeof(char ***));
   
   for(i=0; i<(int)mod->song_length; i++) {      
      namesv[i] = (char *)malloc(4 * sizeof(char));
      sprintf(namesv[i], "%03u", mod->pattern_order_table[i]);
   }
   names[0] = namesv;
   
   return names;
}

   
void update_instrlist(void)
{
   char ***instr_names;
   
   instr_names = get_instr_names(mod);
   update_textlist(instr_list, mod->num_instruments, 1, current_instrument_number-1, instr_names);
}

void update_patt_list(void)
{
   char ***patt_names;
   
   patt_names = get_patt_names(mod);
/*   update_textlist(patt_list, (int)mod->song_length, 1, get_current_pattern_number()-1, patt_names);*/
   
   update_textlist(patt_list, (int)mod->song_length, 1, 0, patt_names);
}

void refresh_instrlist(void)
{
   change_selection(instr_list, get_current_instrument_number() - 1);
}


void refresh_control_window(void)
{
   refresh_instrlist();
}

void clear_status_text(void)
{
   change_label_text(status_label, (char *)"");
}

void set_status_text(char *new_text)
{
   change_label_text(status_label, new_text);
}



