#include <stdio.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

#include <aube.h>

#include "display.h"


extern char *progname;

extern Window control_top;

void getGC(Window toplevel, GC *gc, XFontStruct *font_info, XColor foreground);
void load_font(char *fontname, XFontStruct **font_info);


Display *display;
unsigned int display_depth;
int screen_num;

XFontStruct *small_font;
XFontStruct *large_font;

GC top_gc;
GC small_gc;
GC large_gc;
GC bvl_low_gc;
GC bvl_high_gc;
GC butt_hl_gc;
GC text_hl_gc;
GC hl_gc;
GC cursor_gc;
GC clear_gc;
GC label_gc;
GC label_hl_gc;
GC handle_gc;

Colormap cmap;

XColor top_bg;
XColor subwin_bg;
XColor bvl_low;
XColor bvl_high;
XColor butt_hl;
XColor text_hl;
XColor fg;
XColor hl;
XColor handle;
XColor cursor;


void setup_x(void)
{
   setup_display();
   setup_gfx();

   create_control_window();
/*
   create_tracker_window();
*/   
   XSync(display, True);
}

int x_error_handler(Display *d, XErrorEvent *xerr)
{
#ifdef K_DEBUG
   fprintf(stderr, "XError caught...\n");
#endif
   
   return 0;
}

void setup_display(void)
{
   unsigned int display_width, display_height;
   char *display_name = NULL;

   /* Connect to X Server */
   if ((display = XOpenDisplay(display_name)) == NULL) {
      (void)fprintf(stderr, "%s: cannot connect to X server %s\n",
		    progname, XDisplayName(display_name));
      exit(-1);
   }
   
   screen_num = DefaultScreen(display);
   display_width = DisplayWidth(display, screen_num);
   display_height = DisplayHeight(display, screen_num);
   display_depth = DefaultDepth(display, screen_num);
   
   XSetErrorHandler(x_error_handler);
}

void setup_gfx(void)
{
/*
   char *small_font_name = "-misc-fixed-medium-r-semicondensed--13-120-75-75-c-60-iso8859-1";
   char *large_font_name = "-misc-fixed-bold-r-normal--13-120-75-75-c-70-iso8859-1";
*/


   char *small_font_name = "-*-helvetica-medium-r-*-*-8-*-*-*-*-*-*-*";
   char *large_font_name = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";

/*
   char *small_font_name = "vga";
   char *large_font_name = "vga";
*/   
   cmap = DefaultColormap(display, screen_num);

   hl.red = 0x0;
   hl.green = 0x3000;
   hl.blue = 0x0;
   
   XAllocColor(display, cmap, &hl);

   text_hl.red = 0x0;
   text_hl.green = 0xffff;
   text_hl.blue = 0x0;
   
   XAllocColor(display, cmap, &text_hl);

   top_bg.red = 0x0000;
   top_bg.green = 0x2700;
   top_bg.blue = 0x2000;

   XAllocColor(display, cmap, &top_bg);

   subwin_bg.red = 0x6000;
   subwin_bg.green = 0x6700;
   subwin_bg.blue = 0x6000;
   
   XAllocColor(display, cmap, &subwin_bg);
      
   bvl_low.red = 0x4000;
   bvl_low.green = 0x4700;
   bvl_low.blue = 0x4000;
   
   XAllocColor(display, cmap, &bvl_low);
      
   bvl_high.red = 0xa000;
   bvl_high.green = 0xa700;
   bvl_high.blue = 0xa000;
   
   XAllocColor(display, cmap, &bvl_high);

   butt_hl.red = 0x8000;
   butt_hl.green = 0x8700;
   butt_hl.blue = 0x8000;
   
   XAllocColor(display, cmap, &butt_hl);

   fg.red = 0xf000;
   fg.green = 0xf000;
   fg.blue = 0xe000;
   
   XAllocColor(display, cmap, &fg);
   
   cursor.red = 0xffff;
   cursor.green = 0xffff;
   cursor.blue = 0x0;
   
   XAllocColor(display, cmap, &cursor);

   handle.red = 0x0000;
   handle.green = 0x6000;
   handle.blue = 0x4000;

   XAllocColor(display, cmap, &handle);

   load_font(small_font_name, &small_font);
   load_font(large_font_name, &large_font);
   
   getGC(RootWindow(display, screen_num), &small_gc, small_font, fg);
   getGC(RootWindow(display, screen_num), &large_gc, large_font, fg);
   getGC(RootWindow(display, screen_num), &cursor_gc, large_font, cursor);
   getGC(RootWindow(display, screen_num), &clear_gc, large_font, subwin_bg);
   getGC(RootWindow(display, screen_num), &bvl_low_gc, large_font, bvl_low);
   getGC(RootWindow(display, screen_num), &bvl_high_gc, large_font, bvl_high);
   getGC(RootWindow(display, screen_num), &butt_hl_gc, large_font, butt_hl);
   getGC(RootWindow(display, screen_num), &label_gc, large_font, fg);
   XSetBackground(display, label_gc, fg.pixel);
   getGC(RootWindow(display, screen_num), &label_hl_gc, large_font, text_hl);
   XSetBackground(display, label_hl_gc, text_hl.pixel);
   
   getGC(RootWindow(display, screen_num), &text_hl_gc, large_font, text_hl);
   getGC(RootWindow(display, screen_num), &hl_gc, large_font, hl);   
   XSetFunction(display, hl_gc, GXxor);
   
   getGC(RootWindow(display, screen_num), &top_gc, large_font, top_bg);
   getGC(RootWindow(display, screen_num), &handle_gc, large_font, handle);
}


void getGC(Window win, GC *gc, XFontStruct *font_info, XColor foreground)
{
   unsigned long valuemask = 0;
   XGCValues values;
   
   *gc = XCreateGC(display, win, valuemask, &values);
   
   XSetFont(display, *gc, font_info->fid);
   
   XSetForeground(display, *gc, foreground.pixel);
   XSetBackground(display, *gc, subwin_bg.pixel);
   
}

void load_font(char *fontname, XFontStruct **font_info)
{
   if((*font_info = XLoadQueryFont(display, fontname)) == NULL) {
      (void)fprintf(stderr, "%s: Cannot open font\n", progname);
      exit(-1);
   }
}

