#include <stdio.h>
#include <stdlib.h>

#include <aube.h>

#include "edit.h"
#include "toggle.h"

extern char *progname;

extern struct xm *mod;
extern struct xm_pattern *current_pattern;
extern struct xm_instrument *current_instrument;
extern int current_instrument_number;
extern int current_row;
extern int current_track;
extern int current_pos;

extern struct toggle_data *edit_toggle;
extern Window tracker_top;

int edit=0;
int jump=1;

void edit_on(void)
{
   edit=1;
}

void edit_off(void)
{
   edit=0;
}

void toggle_edit(void)
{
   if(tracker_top) {   
      edit = 1 - edit;
      toggle_toggle(edit_toggle);
   }
}


void delete_note_hex(void)
{
   struct xm_channel_note *curr_ch_note;
   
   curr_ch_note = get_current_xm_channel_note();

   if(!current_pos) {
      curr_ch_note->note = 0;
      curr_ch_note->instrument = 0;
      create_tracks_pixmap();
      scroll_units(1);
   }
   else {
      insert_hex((unsigned short)0x00);
   }
}


void insert_note(int note)
{
   struct xm_channel_note *curr_ch_note;
   
   curr_ch_note = get_current_xm_channel_note();
   
   curr_ch_note->note = note;
   curr_ch_note->instrument = get_current_instrument_number();

   create_tracks_pixmap();

   scroll_units(1);
}


void insert_hex(unsigned short hexval)
{
   struct xm_channel_note *curr_ch_note;
   
   curr_ch_note = get_current_xm_channel_note();

   switch(current_pos) {
    case 1:
      if(hexval & 0x08) {
	 return;
      }
      
      curr_ch_note->instrument &= 0x0f;
      hexval <<= 4;
      hexval &= 0xf0;
      curr_ch_note->instrument |= hexval;
      
      curr_ch_note->instrument &= 0x7f;
      break;
    case 2:
      curr_ch_note->instrument &= 0xf0;
      curr_ch_note->instrument |= hexval;
      
      curr_ch_note->instrument &= 0x7f;
      break;
    case 3:
      curr_ch_note->volume &= 0x0f;
      curr_ch_note->volume |= hexval<<4;
      break;
    case 4:
      curr_ch_note->volume &= 0xf0;
      curr_ch_note->volume |= hexval;
      break;
    case 5:
      curr_ch_note->effect_type &= 0x0f;
     curr_ch_note->effect_type |= hexval<<4;
      break;
    case 6:
      curr_ch_note->effect_type &= 0xf0;
      curr_ch_note->effect_type |= hexval;
      break;
    case 7:
      curr_ch_note->effect_parameter &= 0x0f;
      curr_ch_note->effect_parameter |= hexval<<4;
      break;
    case 8:
      curr_ch_note->effect_parameter &= 0xf0;
      curr_ch_note->effect_parameter |= hexval;
      break;
    default:
      break;
   }

   
   change_current_pos(1);
   create_tracks_pixmap();
   refresh_tracker();
}

   
struct xm_channel_note *get_current_xm_channel_note(void)
{
   int i;
   struct xm_row_list *row;
   struct xm_channel_note_list *channel_note_list;
   
   row = current_pattern->rows;
   for(i=0; i<current_row; i++) {
      row = row->next;
   }
   
   channel_note_list = row->row;
   for(i=0; i<current_track; i++) {
      channel_note_list = channel_note_list->next;
   }
   
   return channel_note_list->ch_note;
}

unsigned short get_current_instrument_number(void)
{
   return current_instrument_number;
}

unsigned short get_current_pattern_number(void)
{
   unsigned short i=0;
   struct xm_pattern_list *pattern_list;
   struct xm_pattern *pattern;

   if(current_pattern == NULL) {
      return 0;
   }
   
   for(pattern_list = mod->patterns;
       pattern_list != NULL;
       pattern_list = pattern_list->next) {
      if(pattern_list->pattern == current_pattern) break;
      i++;
   }
   
   return i;
}

void scroll_units(int scr_y)
{
   scroll_by(scr_y * jump);
}

void scroll_by(int scr_y)
{
   current_row += scr_y;
   
   if(current_row < 0) {
      current_row = 0;
   }
   else if (current_row >= current_pattern->num_rows) {
      current_row = current_pattern->num_rows - 1;
   }
   
   refresh_tracker();
}

void scroll_to(int scr_to)
{
   current_row = scr_to;
   
   if(current_row < 0) {
      /* allow scroll_to(-1) ==> scroll to last row */
      current_row += current_pattern->num_rows;
   }
   else if (current_row >= current_pattern->num_rows) {
      current_row = current_pattern->num_rows - 1;
   }
   
   refresh_tracker();
}

void next_instrument(void)
{
   change_current_instrument(1);
}

void prev_instrument(void)
{
   change_current_instrument(-1);
}

void change_current_instrument(int instr_inc)
{
   short new_instrument_number;
   
   new_instrument_number = get_current_instrument_number();
   
   new_instrument_number += instr_inc;
   if(new_instrument_number < 1) {
      new_instrument_number = 1;
   }
   if(new_instrument_number > 128) {
      new_instrument_number = 128;
   }
   
   change_current_instrument_number(new_instrument_number);
}

   
void change_current_instrument_number(int new_instr)
{
   struct xm_instrument_list *instrument_list;
   int i;
   char status_line[256];

   current_instrument = mod->instruments[new_instr-1];
   current_instrument_number = new_instr;
   
   sprintf(status_line, "Instrument set to: %s", current_instrument->instrument_name);

   update_modinfo();

   update_saube();
   
   set_status_text(status_line);

   refresh_instrlist();
}

void change_current_pattern(int pattern_inc)
{
   short new_pattern_number;

   new_pattern_number = get_current_pattern_number();

   new_pattern_number += pattern_inc;
   
   if(new_pattern_number < 0) {
      new_pattern_number = 0;
   }
   if(new_pattern_number >= mod->num_patterns) {
      new_pattern_number = mod->num_patterns - 1;
   }
   
   change_current_pattern_number(new_pattern_number);
}

void change_current_pattern_number(short new_pattern_number)
{
   int i;
   struct xm_pattern_list *pattern_list;
   struct xm_pattern *new_pattern;

   pattern_list = mod->patterns;
   for(i=0; i < new_pattern_number; i++) {
       pattern_list = pattern_list->next;
   }
   
   new_pattern = pattern_list->pattern;

   set_current_pattern(new_pattern);
}

void set_current_pattern(struct xm_pattern *pattern)
{
/*
   char status_line[256];
*/
   current_pattern = pattern;

/*
   sprintf(status_line, "Pattern set to: %d", current_pattern->pattern_name);
   set_status_text(status_line);
*/   
/*
   refresh_pattern_numrange();
  */
   update_modinfo();
   
   update_tracker();
}

void change_current_track(int track_inc)
{
   current_track += track_inc;
   
   current_track += mod->num_channels;
   current_track %= mod->num_channels;
}

void change_current_pos(int pos_inc)
{
   int abs_cp, sgn_cp;
   
   current_pos += pos_inc;

   abs_cp = abs(current_pos);
   sgn_cp = (current_pos >= 0)?1:-1;

   current_track += mod->num_channels;

   if(current_pos >= 0) {   
      current_track += ((abs_cp)/9);
   }
   
   else {
      current_track--;
   }
   
   
   current_pos += 9;
   current_pos %= 9;
   current_track %= mod->num_channels;
}


