#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

#include <aube.h>

#include "events.h"
#include "edit.h"

#define BITMAPDEPTH 1
#define TOO_SMALL 0
#define BIG_ENOUGH 1


extern char *progname;
extern struct xm *mod;
extern struct xm_instrument *current_instrument;
extern int current_instrument_number;
extern int current_pos;

extern Window control_top;
extern Window tracker_top;
extern Window saube_top;
extern Window track_window;
extern Window filerequest_top;
extern Window zap_top;
extern Window zap_all_button_window;
extern Window zap_patterns_button_window;
extern Window zap_instruments_button_window;
extern Window zap_cancel_button_window;

extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern XFontStruct *small_font;
extern XFontStruct *large_font;
extern GC small_gc;
extern GC large_gc;
extern Colormap cmap;

extern struct toggle_data *current_play_toggle;

extern int edit;

extern int player_pid;
extern int playing;

void note_key_pressed(int note)
{
   int pid_status;
         
   if(player_pid > 0) {
      kill((pid_t)player_pid, SIGKILL);
      waitpid((pid_t)player_pid, &pid_status, 0);
      player_pid = -1;
      if(current_play_toggle) {
	 set_toggle_state(current_play_toggle, 0);
      }
   }

   playing=1;
   current_play_toggle = NULL;

   player_pid = fork();
   if(!player_pid) {
      play_instrument_scale(mod->instruments[current_instrument_number -1], note);
      exit(0);
   }
   else if (player_pid <= 0) {
      fprintf(stderr, "Unable to fork...\n");
   }      
}


void handle_events(void)
{
   XEvent report;
   unsigned int width, height;
   Window press_window;
   int count;
   KeySym keysym;
   unsigned int state;
   XComposeStatus compose;
   char buffer[80];
   int bufsize=80;
   int note=-1;
   unsigned short hexval=0;
   
   while(1) {
      XNextEvent(display, &report);
      
      switch (report.type) {
       case Expose:
	 /* Unless this is the last contiguous expose, don't draw
	  * the window
	  */
	 if(report.xexpose.count != 0) {
	    break;
	 }

	 if(check_panels_expose(report)) {
	    break;
	 }
	 else if(check_labels_expose(report)) {
	    break;
	 }
	 else if(check_textfields_expose(report)) {
	    break;
	 }
	 else if(check_buttons_expose(report)) {
	    break;
	 }
	 else if(check_toggles_expose(report)) {
	    break;
	 }
	 else if(check_scrollbars_expose(report)) {
	    break;
	 }
	 else if(check_scroll_handles_expose(report)) {
	    break;
	 }
	 else if(check_textlists_expose(report)) {
	    break;
	 }
	 else if(check_numrange_handles_expose(report)) {
	    break;
	 }
	 else if(report.xexpose.window == control_top) {
	    refresh_control_window();
	 }
	 else if(report.xexpose.window == tracker_top) {
	    refresh_tracker_window();
	 }
	 else if(report.xexpose.window == saube_top) {
	    refresh_saube_graph();
	 }

	 break;
       case DestroyNotify:
	 if(check_buttons_destroyed(report)) {
	    break;
	 }
	 break;
       case ConfigureNotify:
	 /* Window has been resized; change width
	  * and height to send to place_text and place_graphics in next
	  * Expose
	  */
	 width = report.xconfigure.width;
	 height = report.xconfigure.height;
	 break;
       case MotionNotify:
 	 if(check_panels_motion(report)) {
	    break;
	 }
	 else if(check_scroll_handles_motion(report)) {
	    break;
	 }
	 else if(check_textlists_motion(report)) {
	    break;
	 }
	 else if(check_numrange_handles_motion(report)) {
	    break;
	 }
	 break;
       case EnterNotify:

	 if(report.xcrossing.state & Button1Mask) {
	    press_window = report.xcrossing.window;

	    if(check_buttons_pressdown(report)) {
	       break;
	    }
	 }
	 
	 else if(check_buttons_mouseover(report)) {
	    break;
	 }
	 else if(check_toggles_mouseover(report)) {
	    break;
	 }
	 else if(check_scroll_handles_mouseover(report)) {
	    break;
	 }
	 else if(check_textlists_mouseover(report)) {
	    break;
	 }

	 break;
       case LeaveNotify:
	 clear_status_text();
	 press_window = 0L;
	 
	 if((int)check_buttons_mouseexit(report)) {
	    break;
	 }
	 else if(check_toggles_mouseexit(report)) {
	    break;
	 }
	 else if(check_scroll_handles_mouseexit(report)) {
	    break;
	 }
	 else if(check_numrange_handles_mouseexit(report)) {
	    break;
	 }
	 else if(check_textlists_mouseexit(report)) {
	    break;
	 }
	 else if(check_textfields_mouseexit(report)) {
	    break;
	 }
	 
	 break;
       case ButtonPress:
	 press_window = report.xbutton.window;

	 
	 if(check_buttons_pressdown(report)) {
	    break;
	 }
	 else if(check_toggles_pressdown(report)) {
	    break;
	 }
	 else if(check_panels_pressdown(report)) {
	    break;
	 }
	 else if(check_scroll_handles_pressdown(report)) {
	    break;
	 }
	 else if(check_textlists_pressdown(report)) {
	    break;
	 }
	 else if(check_numrange_handles_pressdown(report)) {
	    break;
	 }
	 
	 break;
       case ButtonRelease:
	 if(report.xbutton.window == 0L) {
	    break;
	 }
	 else if(report.xbutton.window != press_window) {
	    break;
	 }
	 else if(check_panels_pressed(report)) {
	    break;
	 }
	 else if(check_buttons_pressed(report)) {
	    break;
	 }
	 else if(check_toggles_pressed(report)) {
	    break;
	 }
	 else if(check_scroll_handles_pressed(report)) {
	    break;
	 }
	 else if(check_textlists_pressed(report)) {
	    break;
	 }
	 else if(check_textfields_pressed(report)) {
	    break;
	 }
	 else if(check_numrange_handles_pressed(report)) {
	    break;
	 }
	 press_window = 0L;
	 break;
       case KeyPress:
	 state = report.xkey.state;
	 count = XLookupString((XKeyEvent *)(&report), buffer, bufsize, &keysym, &compose);

	 if(check_textfields_keypress(report)) {
	    break;
	 }
	 else if(state & ControlMask) {
	    switch(keysym) {
	     case XK_f:
	     case XK_F:
	       change_current_pattern(1);
	       break;
	     case XK_b:
	     case XK_B:
	       change_current_pattern(-1);
	       break;
	     default:
	       break;
	    }
	 }
	 else {
	    switch(keysym) {
	     case XK_Tab:
	       if(state & ShiftMask) {
		  change_current_track(-1);
	       }
	       else {  
		  change_current_track(1);
	       }
	       refresh_tracker();
	       note = -1;
	       hexval = 0xff;
	       break;
	     case XK_Left:
	       if(state & ShiftMask) {
		  change_current_instrument(-1);
	       }
	       else {
		  change_current_pos(-1);
		  refresh_tracker();
	       }
	       note = -1;
	       hexval = 0xff;
	       break;
	     case XK_Right:
	       if(state & ShiftMask) {
		  change_current_instrument(1);
	       }
	       else {
		  change_current_pos(1);
		  refresh_tracker();
	       }
	       note = -1;
	       hexval = 0xff;
	       break;
	     case XK_Up:
	       scroll_units(-1);
	       note = -1;
	       hexval = 0xff;
	       break;
	     case XK_Down:
	       scroll_units(1);
	       note = -1;
	       hexval = 0xff;
	       break;
	     case XK_Page_Up:
	       scroll_units(-16);
	       note = -1;
	       hexval = 0xff;
	       break;
	     case XK_Page_Down:
	       scroll_units(16);
	       note = -1;
	       hexval = 0xff;
	       break;
	     case XK_Home:
	       scroll_to(0);
	       note = -1;
	       hexval = 0xff;
	       break;
	     case XK_End:
	       scroll_to(-1);
	       note = -1;
	       hexval = 0xff;
	       break;
	     case XK_Escape:
	       note = -1;
	       hexval = 0xff;
	       toggle_edit();
	       break;
	     case XK_Delete:
	       delete_note_hex();
	       note = -1;
	       hexval = 0xff;
	       break;
	     case XK_z:
	     case XK_Z:
	       note = 0;
	       hexval = 0xff;
	       break;
	     case XK_s:
	     case XK_S:
	       note = 1;
	       hexval = 0xff;
	       break;
	     case XK_x:
	     case XK_X:
	       note = 2;
	       hexval = 0xff;
	       break;
	     case XK_d:
	     case XK_D:
	       note = 3;
	       hexval = 0xd;
	       break;
	     case XK_c:
	     case XK_C:
	       note = 4;
	       hexval = 0xc;
	       break;
	     case XK_v:
	     case XK_V:
	       note = 5;
	       hexval = 0xff;
	       break;
	     case XK_g:
	     case XK_G:
	       note = 6;
	       hexval = 0xff;
	       break;
	     case XK_b:
	     case XK_B:
	       note = 7;
	       hexval = 0xb;
	       break;
	     case XK_h:
	     case XK_H:
	       note = 8;
	       hexval = 0xff;
	       break;
	     case XK_n:
	     case XK_N:
	       note = 9;
	       hexval = 0xff;
	       break;
	     case XK_j:
	     case XK_J:
	       note = 10;
	       hexval = 0xff;
	       break;
	     case XK_m:
	     case XK_M:
	       note = 11;
	       hexval = 0xff;
	       break;
	     case XK_q:
	     case XK_Q:
	     case XK_comma:
	     case XK_less:
	       note = 12;
	       hexval = 0xff;
	       break;
	     case XK_l:
	     case XK_L:
	       note = 13;
	       hexval = 0xff;
	     case XK_2:
	     case XK_at:  
	       note = 13;
	       hexval = 0x2;
	       break;
	     case XK_w:
	     case XK_W:
	     case XK_period:
	     case XK_greater:
	       note = 14;
	       hexval = 0xff;
	       break;
	     case XK_3:
	     case XK_numbersign:
	     case XK_semicolon:
	     case XK_colon:
	       note = 15;
	       hexval = 0x3;
	       break;
	     case XK_e:
	     case XK_E:
	     case XK_slash:
	     case XK_question:
	       note = 16;
	       hexval = 0xe;
	       break;
	     case XK_r:
	     case XK_R:
	       note = 17;
	       hexval = 0xff;
	       break;
	     case XK_5:
	     case XK_percent:
	       note = 18;
	       hexval = 0x5;
	       break;
	     case XK_t:
	     case XK_T:
	       note = 19;
	       hexval = 0xff;
	       break;
	     case XK_6:
	     case XK_asciicircum:
	       note = 20;
	       hexval = 0x6;
	       break;
	     case XK_y:
	     case XK_Y:
	       note = 21;
	       hexval = 0xff;
	       break;
	     case XK_7:
	     case XK_ampersand:
	       note = 22;
	       hexval = 0x7;
	       break;
	     case XK_u:
	     case XK_U:
	       note = 23;
	       hexval = 0xff;
	       break;
	     case XK_i:
	     case XK_I:
	       note = 24;
	       hexval = 0xff;
	       break;
	     case XK_9:
	     case XK_parenleft:
	       note = 25;
	       hexval = 0x9;
	       break;
	     case XK_o:
	     case XK_O:
	       note = 26;
	       hexval = 0xff;
	       break;
	     case XK_0:
	     case XK_parenright:
	       note = 27;
	       hexval = 0x0;
	       break;
	     case XK_p:
	     case XK_P:
	       note = 28;
	       hexval = 0xff;
	       break;
	     case XK_bracketleft:
	     case XK_braceleft:
	       note = 29;
	       hexval = 0xff;
	       break;
	     case XK_equal:
	     case XK_plus:
	       note = 30;
	       hexval = 0xff;
	       break;
	     case XK_bracketright:
	     case XK_braceright:
	       note = 31;
	       hexval = 0xff;
	       break;              
	     case XK_backslash:
	     case XK_bar:
	       note = 32;
	       hexval = 0xff;
	       break;
	     case XK_1:
	     case XK_exclam:
	       note = -1;
	       hexval = 0x1;
	       break;
	     case XK_4:
	     case XK_dollar:
	       note = -1;
	       hexval = 0x4;
	       break;
	     case XK_8:
	     case XK_asterisk:
	       note = -1;
	       hexval = 0x8;
	       break;
	     case XK_minus:
	       note = -1;
	       hexval = 0x0;
	       break;
	     case XK_a:
	     case XK_A:
	       note = -1;
	       hexval = 0xa;
	       break;
	     case XK_f:
	     case XK_F:
	       note = -1;
	       hexval = 0xf;
	       break;
	     default:
	       note = -1;
	       hexval = 0xff;
	       break;
	    }
	    if(current_pos == 0 && note >= 0) {
	       note += 48;
	       if(edit) {	  
		  insert_note(note);
	       }
	       if(current_instrument) {	     
		  note_key_pressed(note);
	       }
	    }
	    else if(edit && !(hexval & 0xf0) && current_pos) {
	       insert_hex(hexval);
	    }
	 }
	 
	 break;
       default:
	 break;
      }
   }
}

void clean_exit(void)
{   
#ifdef __PARANOID_EX_AMIGA_HACKER__
   free_xm(mod);

   XUnloadFont(display, small_font->fid);
   XUnloadFont(display, large_font->fid);
   XFreeGC(display, small_gc);
   XFreeGC(display, large_gc);
   XFreeColormap(display, cmap);
   XCloseDisplay(display);
#endif
   
   exit(1);
}
