#include <stdio.h>

#include <aube.h>

struct xm *mod;
struct xm_pattern *current_pattern;
struct xm_instrument *current_instrument;
int current_instrument_number;
int current_track;
int current_row;
int current_pos;

struct xm *load_unknown(char *filename)
{
   if(test_xm(filename)) {
      return load_xm(filename);
   }
   else if(test_s3m(filename)) {
      return load_s3m(filename);
   }
   else {
      return load_mod(filename);
   }
}

void load_over(char *filename)
{
   struct xm *m;

/*   if(!(m=load_xm(filename))) return;*/
/*   if(!(m=load_mod(filename))) return;*/
/*   if(!(m=load_s3m(filename))) return;*/
   if(!(m=load_unknown(filename))) return;
   
   empty_xm(mod);
   mod = m;

   current_instrument = mod->instruments[0];
   current_instrument_number = 1;
   change_current_pattern_number(mod->pattern_order_table[0]);
   current_track = 0;
   current_row = 0;
   current_pos = 0;
   
   update_modinfo();

   update_instrlist();
   update_patt_list();
   
   refresh_control_window();
   refresh_tracker_window();
   update_saube();
}

void load_raw_sample(char *filename)
{
   struct xm_sample *s;
   
   if(!(s = load_raw(filename))) return;

   if(!current_instrument) {
      fprintf(stderr, "No current instrument!\n");
   }
   if(!current_instrument->samples) {
      current_instrument->samples = (struct xm_sample **)malloc(sizeof(struct xm_sample *));
   }
   
   current_instrument->samples[0] = s;
   
   /* update sample list/window */
}


void clear_current_xm_pattern(void)
{
   struct xm_pattern_list *pl;
   struct xm_pattern *temp;
   
   for(pl = mod->patterns; pl; pl = pl->next) {
      if(pl->pattern == current_pattern) break;
   }
   
   temp = current_pattern;
   pl->pattern = empty_xm_pattern((int)current_pattern->num_rows, (int)mod->num_channels);
   current_pattern = pl->pattern;
   free_xm_pattern(temp);
}

void shrink_current_xm_pattern(void)
{
   struct xm_row_list *rows, *trows;
   
   for(rows=current_pattern->rows;rows;rows=rows->next) {
      trows = rows->next;
      
      if(trows) {
	 rows->next = trows->next;
	 free_xm_channel_note_list(trows->row, mod->num_channels);
	 free(trows);
      }
      else {
	 break;
      }
   }
   
   current_pattern->num_rows /= 2;
}

void expand_current_xm_pattern(void)
{
   struct xm_row_list *rows, *trows;
   
   for(rows=current_pattern->rows; rows; ) {
      trows = rows->next;
      rows->next = (struct xm_row_list *)malloc(sizeof(struct xm_row_list));
      rows->next->row = empty_xm_channel_note_list(mod->num_channels);
      rows->next->next = trows;
      rows = trows;
   }
}

void del_current_xm_pattern(void)
{
   struct xm_pattern_list *pl, *ppl=NULL;
   
   for(pl = mod->patterns; pl; pl = pl->next) {
      if(pl->pattern == current_pattern) break;
      ppl = pl;
   }
   
   if(ppl) {   
      ppl->next = pl->next;
   }
   else {
      mod->patterns = pl->next;
   }
   
   free_xm_pattern(pl->pattern);
   free(pl);
   
   if(ppl) {   
      if(ppl->next) {   
	 current_pattern = ppl->next->pattern;
      }
      else {
	 current_pattern = ppl->pattern;
      }
   }
   else {
      current_pattern = mod->patterns->pattern;
   }
   
   mod->num_patterns--;
}

void ins_current_xm_pattern(void)
{
   struct xm_pattern_list *pl, *ppl=NULL, *npl;
   struct xm_pattern *temp;
   
   for(pl = mod->patterns; pl; pl = pl->next) {
      if(pl->pattern == current_pattern) break;
      ppl = pl;
   }
   
   npl = (struct xm_pattern_list *)malloc(sizeof(struct xm_pattern_list));
   npl->pattern = empty_xm_pattern((int)current_pattern->num_rows, (int)mod->num_channels);
   npl->next = pl;

   if(ppl) {   
      ppl->next = npl;
   }
   else {
      mod->patterns = npl;
   }
   
   current_pattern = npl->pattern;
   
   mod->num_patterns++;
}

void transpose_pattern(struct xm_pattern *pattern, int t)
{
   struct xm_row_list *rows;
   struct xm_channel_note_list *channel_note_list;
   struct xm_channel_note *ch_note;
   
   for(rows = pattern->rows; rows; rows = rows->next) {
      for(channel_note_list = rows->row; channel_note_list; channel_note_list = channel_note_list->next) {
	 ch_note = channel_note_list->ch_note;
	 if(ch_note->note) {   
	    ch_note->note += t;
	    if(ch_note->note > 71) {
	       ch_note->note = 71;
	    }

	    /* commented out for unsigned char note
	    else if(ch_note->note < 0) {
	       ch_note->note = 0;
	    }
	    */
	 }
      }
   }
}

void transpose_track(struct xm_pattern *pattern, int t, int track)
{
   struct xm_row_list *rows;
   struct xm_channel_note_list *channel_note_list;
   struct xm_channel_note *ch_note;
   int i;
   
   for(rows = pattern->rows; rows; rows = rows->next) {
      i=0;
      for(channel_note_list = rows->row; channel_note_list; channel_note_list = channel_note_list->next) {
	 if(i == track) {   
	    ch_note = channel_note_list->ch_note;
	    if(ch_note->note) {   
	       ch_note->note += t;
	       if(ch_note->note > 71) {
		  ch_note->note = 71;
	       }
	       /* commented out for unsigned char note
	       else if(ch_note->note < 0) {
		  ch_note->note = 0;
	       }
	       */
	    }
	 }
	 i++;
      }
   }
}

