#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include <aube.h>

#include <panel.h>
#include <numrange.h>
#include <label.h>
#include <textfield.h>
#include <textlist.h>

#include "saube.h"

extern char *progname;

extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC small_gc;
extern XFontStruct *small_font;
extern GC large_gc;
extern XFontStruct *large_font;
extern GC bvl_low_gc;
extern GC bvl_high_gc;
extern GC cursor_gc;
extern GC clear_gc;
extern GC hl_gc;

extern struct xm *mod;
extern struct xm_instrument *current_instrument;

extern Window saube_graph;

struct xm_sample *current_sample;

Window saube_top;


static struct panel *data_panel;
static struct panel *graph_panel;

static struct textfield *instrumentname_textfield;

static struct textlist *samplenames_textlist;

static struct numrange *sample_length_numrange;
static struct numrange *sample_loop_start_numrange;
static struct numrange *sample_loop_length_numrange;
static struct numrange *volume_numrange;
static struct numrange *finetune_numrange;
static struct numrange *panning_numrange;
static struct numrange *rel_note_number_numrange;

static struct label *sample_length_label;
static struct label *sample_loop_start_label;
static struct label *sample_loop_length_label;
static struct label *volume_label;
static struct label *finetune_label;
static struct label *panning_label;
static struct label *rel_note_number_label;

static int width, height;

void sample_length_changed(int new_len)
{
}

void sample_loop_start_changed(int new_start)
{
}

void sample_loop_length_changed(int new_ll)
{
}

void volume_changed(int new_vol)
{
   if(current_sample) {
      current_sample->volume = (bit8)new_vol;
   }
}

void finetune_changed(int new_fine)
{
   if(current_sample) {
      current_sample->finetune = (bit8)new_fine;
   }
}

void panning_changed(int new_pan)
{
   if(current_sample) {
      current_sample->panning = (bit8)new_pan;
   }
}

void rel_note_number_changed(int new_rel)
{
   if(current_sample) {
      current_sample->rel_note_number = (bit8)new_rel;
   }
}

void instrumentname_changed(char *s)
{
   strncpy(current_instrument->instrument_name, s, 28);
   update_instrlist();
}

void samplelist_changed(int sampl_index)
{
   current_sample = current_instrument->samples[sampl_index];
   sample_changed();
}


void update_saube(void)
{
   char saube_title[256];
   
   if(!saube_top) return;

   if(current_instrument) {
      
      sprintf(saube_title, "saube: %s", current_instrument->instrument_name);
      change_window_title(saube_top, saube_title);
      
      change_textfield_text(instrumentname_textfield, current_instrument->instrument_name);
      
      if(current_instrument->samples) {
	 current_sample = current_instrument->samples[0];
      }
      else {
	 current_sample = NULL;
      }

   }
   else {
      sprintf(saube_title, "saube:");
      change_window_title(saube_top, saube_title);
      
      change_textfield_text(instrumentname_textfield, NULL);
      
      set_numrange_bounds(sample_length_numrange, 0, 0, 0);
      set_numrange_bounds(sample_loop_start_numrange, 0, 0, 0);
      set_numrange_bounds(sample_loop_length_numrange, 0, 0, 0);
      set_numrange_bounds(volume_numrange, 0, 0, 0);
      set_numrange_bounds(finetune_numrange, 0, 0, 0);
      set_numrange_bounds(panning_numrange, 0, 0, 0);
      set_numrange_bounds(sample_loop_start_numrange, 0, 0, 0);
      set_numrange_bounds(rel_note_number_numrange, 0, 0, 0);
   }
   update_samplelist();
   sample_changed();
}

void update_samplelist(void)
{
   char ***sample_names;

   sample_names = get_sample_names();
   samplenames_textlist = create_textlist(saube_top, 2, 20, 126, height-154,
					  (void *)samplelist_changed,
					  (void *)samplelist_changed,
					  current_instrument->num_samples, 1,
					  sample_names);
}

void sample_changed(void)
{
   if(current_sample) {
	 
      set_numrange_bounds(sample_length_numrange, current_sample->sample_length,
			  current_sample->sample_length, current_sample->sample_length);
      set_numrange_bounds(sample_loop_start_numrange, current_sample->sample_loop_start,
			  current_sample->sample_loop_start, current_sample->sample_loop_start);
      set_numrange_bounds(sample_loop_length_numrange, current_sample->sample_loop_length,
			  current_sample->sample_loop_length, current_sample->sample_loop_length);
      set_numrange_bounds(volume_numrange, current_sample->volume,
			  current_sample->volume, current_sample->volume);
      set_numrange_bounds(finetune_numrange, current_sample->finetune,
			  current_sample->finetune, current_sample->finetune);
      set_numrange_bounds(panning_numrange, current_sample->panning,
			  current_sample->panning, current_sample->panning);
      set_numrange_bounds(sample_loop_start_numrange, current_sample->sample_loop_start,
			  current_sample->sample_loop_start, current_sample->sample_loop_start);
      set_numrange_bounds(rel_note_number_numrange, current_sample->rel_note_number,
			  current_sample->rel_note_number, current_sample->rel_note_number);
   }
   else {
      set_numrange_bounds(sample_length_numrange, 0, 0, 0);
      set_numrange_bounds(sample_loop_start_numrange, 0, 0, 0);
      set_numrange_bounds(sample_loop_length_numrange, 0, 0, 0);
      set_numrange_bounds(volume_numrange, 0, 0, 0);
      set_numrange_bounds(finetune_numrange, 0, 0, 0);
      set_numrange_bounds(panning_numrange, 0, 0, 0);
      set_numrange_bounds(sample_loop_start_numrange, 0, 0, 0);
      set_numrange_bounds(rel_note_number_numrange, 0, 0, 0);
   }
   refresh_saube_graph();
}


void create_saube_window(void)
{
   char window_name[256];
   char *icon_name = "saube";
   int button_width, button_height;
   char ***sample_names;

   width = 700;
   height = 220;
   
   button_width = 50;
   button_height = 16;
   
   sprintf(window_name, "saube: %s", current_instrument->instrument_name);

   
   current_sample = current_instrument->samples[0];
   
   
   saube_top = create_toplevel_window(window_name, window_name,
				      0, 0, width, height, 400, 200);

   instrumentname_textfield = create_textfield(saube_top, 2, 2, 126, 16,
					       (void *)instrumentname_changed,
					       current_instrument->instrument_name,
					       LEFT_JUSTIFY);

   sample_names = get_sample_names();
   samplenames_textlist = create_textlist(saube_top, 2, 20, 126, height-154,
					  (void *)samplelist_changed,
					  (void *)samplelist_changed,
					  current_instrument->num_samples, 1,
					  sample_names);
   
   data_panel = create_panel(saube_top, 2, height-132, 126, 130);

   sample_length_label = create_label(data_panel->window, 8, 2, 66, 18, (char *)"length");
   sample_loop_start_label = create_label(data_panel->window, 8, 20, 66, 18, (char *)"loop start");
   sample_loop_length_label = create_label(data_panel->window, 8, 38, 66, 18, (char *)"loop end");
   volume_label = create_label(data_panel->window, 8, 56, 66, 18, (char *)"volume");
   finetune_label = create_label(data_panel->window, 8, 74, 66, 18, (char *)"finetune");
   panning_label = create_label(data_panel->window, 8, 92, 66, 18, (char *)"panning");
   panning_label = create_label(data_panel->window, 8, 110, 66, 18, (char *)"relative note");
   
   sample_length_numrange = create_numrange(data_panel->window, 74, 2, 50, 18,
					    current_sample->sample_length, current_sample->sample_length, current_sample->sample_length,
					    (void *)sample_length_changed);

   sample_loop_start_numrange = create_numrange(data_panel->window, 74, 20, 50, 18,
					  current_sample->sample_loop_start,
					  current_sample->sample_loop_start, current_sample->sample_loop_start,
					  (void *)sample_loop_start_changed);
   
   sample_loop_length_numrange = create_numrange(data_panel->window, 74, 38, 50, 18,
						 current_sample->sample_loop_length,
						 current_sample->sample_loop_length, current_sample->sample_loop_length, (void *)sample_loop_length_changed);
   
   volume_numrange = create_numrange(data_panel->window, 74, 56, 50, 18,
				     current_sample->volume, 0, 64, (void *)volume_changed);
   
   finetune_numrange = create_numrange(data_panel->window, 74, 74, 50, 18,
				       current_sample->finetune,
				       0, 28,
				       (void *)finetune_changed);
   
   panning_numrange = create_numrange(data_panel->window, 74, 92, 50, 18,
				      current_sample->panning,
				      -127, (int)127,
				      (void *)panning_changed);

   rel_note_number_numrange = create_numrange(data_panel->window, 74, 110, 50, 18,
					      current_sample->rel_note_number,
					      -127, (int)127,
					      (void *)rel_note_number_changed);


   graph_panel = create_panel(saube_top, 130, 2, width-132, height-4);
   create_saube_graph(graph_panel->window, 8, 2, width-140, height-8);

   XMapWindow(display, saube_top);
}

char ***get_sample_names(void)
{
   char **namesv;
   char ***names;
   int i;
   
   namesv = (char **)malloc((size_t)current_instrument->num_samples * sizeof(char *));
   names = (char ***)malloc(sizeof(char ***));
   
   for(i=0; i<current_instrument->num_samples; i++) {
      namesv[i] = current_instrument->samples[i]->sample_name;
   }
   names[0] = namesv;
   
   return names;
}


void close_saube_window(void)
{
   if(saube_top) {
      XDestroyWindow(display, saube_top);
      saube_top = 0L;
   }
}

