#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include <aube.h>


extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC small_gc;
extern XFontStruct *small_font;
extern GC large_gc;
extern XFontStruct *large_font;
extern GC bvl_low_gc;
extern GC bvl_high_gc;
extern GC cursor_gc;
extern GC clear_gc;
extern GC hl_gc;

extern struct xm_sample *current_sample;

Window saube_graph;

static Pixmap saube_graph_pixmap = 0L;

static int width, height;   


void create_saube_graph(Window parent, int x, int y, int w, int h)
{		 
   width = w;
   height = h;
   
   saube_graph = create_subwindow(parent, x, y, width, height);
   saube_graph_pixmap = XCreatePixmap(display, parent, width, height, display_depth);
}

void refresh_saube_graph(void)
{
   int x, ly, ry, last_x, last_ly, last_ry;
   double xf; /* counter for len < width */
   double x_step;
   double y_mag;
   bit16 *dp;
   double l_total, r_total;
   double ii;
   int centreline, zoomline;
   int sample_start_x, sample_end_x;

   
   y_mag = ((double)height) / ((double)(65535.0));

   centreline = height>>1;
   
   XFillRectangle(display, saube_graph_pixmap, clear_gc, 0, 0,
		  width, height);
   
   if(!current_sample) return;
   
   if(((int)current_sample->sample_length) > width) {
      
      
      x_step = ((double)((int)current_sample->sample_length<<1)) /
	((double)width);
      
      /* similar calc for start_x ... */
      sample_start_x = 0;
      	 
      sample_end_x = width;
      
      
      
      for(x=sample_start_x; x<sample_end_x; x++) {
	 
	 dp = (bit16 *)((int)current_sample->data +
			(int)((double)x * x_step));
	 
	 l_total=0;
	 
	 for(ii=0; ii < x_step; ii += 1.0) {
	    l_total += *((short *)dp++);
	 }
	 l_total = l_total / ii;

	 ly = centreline - (l_total*y_mag);
	 if(x>0) {
	    XDrawLine(display, saube_graph_pixmap, small_gc,
		      last_x, last_ly, x, ly);
	 }
	 
	 
	 last_x = x;
	 last_ly = ly;
      }
      
      
   }
   
   else {
      
      x_step = (double)(width) /
	(double)((int)current_sample->sample_length);
      dp = (bit16 *)((int)current_sample->data);
      
      /* similar calc for start_x ... */
      sample_start_x = 0;
      
      if((int)width > (int)current_sample->sample_length) {
	 sample_end_x = (int)
	   ((double)(x_step * ((int)current_sample->sample_length)));
      }
      
      else {
	 sample_end_x = width;
      }
      
      
      for(xf=(double)sample_start_x; xf < (double)sample_end_x; xf += x_step) {
	 
	 
	 ly = centreline - ((*dp++ - 32768) * y_mag);
	 x = (int)xf;
	 
	 if(x>0) {
	    XDrawLine(display, saube_graph_pixmap, small_gc,
		      x, last_ly, x, ly);
	 }
	 
	 
	 XDrawLine(display, saube_graph_pixmap, small_gc,
		   x, ly, x+x_step, ly);
	 
	 last_ly = ly;
      }
      
   }

   XCopyArea(display, saube_graph_pixmap, saube_graph, small_gc,
	     0, 0, width, height,
	     0, 0);
}

