#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include <aube.h>

#include "tracker.h"

extern char *progname;

extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC small_gc;
extern XFontStruct *small_font;
extern GC large_gc;
extern XFontStruct *large_font;
extern GC bvl_low_gc;
extern GC bvl_high_gc;
extern GC cursor_gc;
extern GC clear_gc;
extern GC hl_gc;

extern struct xm *mod;
extern struct xm_pattern *current_pattern;
extern int current_track;
extern int current_row;
extern int current_pos;

Window tracker_tracks;

int channels_visible;
int rows_visible;

static Pixmap tracker_back_pixmap = 0;
static Pixmap tracks_pixmap = 0;
static Pixmap nums_pixmap = 0;

static int n_width; /* width of numbers pixmap */
static int t_width; /* width of tracks pixmap */
static int p_height; /* height of both pixmaps */
static int channel_width; /* width of channel */

static int cursor_start[9], cursor_len[9];
static int large_str_len, small_str_len, small_gap_len;

static int width, height;


void create_tracks_window(Window parent, int x, int y, int w, int h)
{		 
   width = w;
   height = h;
   
   tracker_tracks = create_subwindow(parent, x, y, width, height);
   
   tracker_back_pixmap = XCreatePixmap(display, parent,
				       width, height,
				       display_depth);
   
   XFillRectangle(display, tracker_back_pixmap, clear_gc, 0, 0,
		  width, height);

   setup_cursor_arrays();
   
   create_tracker();
   
/*   refresh_tracker();
 */
}

void create_tracker(void)
{
   create_nums_pixmap();

   if(current_pattern != NULL) {   
      create_tracks_pixmap();
   }
      
}

void setup_cursor_arrays(void)
{
   char *large_test_str = "D#4 ";
   char *small_test_str = "D";
   char *small_gap_str = " ";
   

   int x;
   
   large_str_len = XTextWidth(large_font, large_test_str,
			      strlen(large_test_str));
   small_str_len = XTextWidth(small_font, small_test_str,
			      strlen(small_test_str));
   small_gap_len = XTextWidth(small_font, small_gap_str,
			      strlen(small_gap_str));
   
   x = 0;
   cursor_start[0] = x-1;
   
   x += large_str_len;
   cursor_len[0] = large_str_len+2;
   
   x += small_gap_len;
   cursor_start[1] = x;
   
   x += small_str_len;
   cursor_len[1] = small_str_len;
   cursor_start[2] = x;
   
   x += small_str_len;
   cursor_len[2] = small_str_len;
   
   x += small_gap_len;
   cursor_start[3] = x;
   
   x += small_str_len;
   cursor_len[3] = small_str_len;
   
   cursor_start[4] = x;
   
   x += small_str_len;
   cursor_len[4] = small_str_len;
   
   x += small_gap_len;
   cursor_start[5] = x;
   
   x += small_str_len;
   cursor_len[5] = small_str_len;
   cursor_start[6] = x;
   
   x += small_str_len;
   cursor_len[6] = small_str_len;
   
   x += small_gap_len;
   cursor_start[7] = x;
   
   x += small_str_len;
   cursor_len[7] = small_str_len;
   
   cursor_start[8] = x;
   
   x += small_str_len;
   cursor_len[8] = small_str_len;
}


void create_nums_pixmap(void)
{
   int font_height;
   int i;
   int ypos = 0;
   char line_str[3];
   char *test_str = "DD";
   int numrows;

   if(current_pattern == NULL) {
      numrows = 64;
   }
   else {
      numrows = current_pattern->num_rows;
   }

   font_height = large_font->ascent + large_font->descent;

   ypos = font_height;
   
   n_width = XTextWidth(large_font, test_str, strlen(test_str)) + 10;
   p_height = (font_height + 2) * (numrows + 2);
   
   if(nums_pixmap != 0) {
      XFreePixmap(display, nums_pixmap);
   }

   nums_pixmap = XCreatePixmap(display, tracker_tracks, n_width, p_height,
			       display_depth);

   XFillRectangle(display, nums_pixmap, clear_gc, 0, 0, n_width, p_height);
   
   for(i=0; i < numrows; i++) {
      sprintf(line_str, "%02X", i);
      XDrawString(display, nums_pixmap, large_gc, 5, ypos + font_height,
		  line_str, strlen(line_str));
      ypos += font_height + 2;
   }
   
}

void create_tracks_pixmap(void)
{
   int font_height;
   int xpos = 0, ypos = 0;
   int i;
   struct xm_row_list *row_list;
   struct xm_channel_note_list *channel_note_list;
   struct xm_channel_note *ch_note;
   int numrows, numchannels;
   
   char *large_test_str = "DDD ";
   char *small_test_str = " DD DD DD DD";
   
   
   char note_str[16];
   char note[4], ins[3], vol[3], eff_type[3], eff_parm[3];
   
   font_height = large_font->ascent + large_font->descent;

   ypos = font_height;
   
   channel_width =
     XTextWidth(large_font, large_test_str, strlen(large_test_str)) +
     XTextWidth(small_font, small_test_str, strlen(small_test_str)) + 20;
   
   
   t_width = channel_width * mod->num_channels;
   
   p_height = (current_pattern->num_rows + 2) * (font_height + 2);

   if(tracks_pixmap != 0) {
      XFreePixmap(display, tracks_pixmap);
   }

   tracks_pixmap = XCreatePixmap(display, tracker_tracks, t_width, p_height,
				 display_depth);
   
   XFillRectangle(display, tracks_pixmap, clear_gc, 0, 0, t_width, p_height);
   
   row_list = current_pattern->rows;
   for(i=0; i < current_pattern->num_rows; i++) {
      xpos = 10;
      
      for(channel_note_list = row_list->row;
	  channel_note_list != NULL;
	  channel_note_list = channel_note_list->next) {

	 ch_note = channel_note_list->ch_note;
	 
	 if(ch_note->note && ch_note->note < 72) {
	    sprintf(note, "%3s", note_strings[(int)ch_note->note]);
	 }
	 else {
	    if(ch_note->instrument) {
	       sprintf(note, "%3s", note_strings[(int)ch_note->note]);
	    }
	    else {  
	       sprintf(note, "---");
	    }
	 }
	 
	 XDrawString(display, tracks_pixmap, large_gc,
		     xpos + cursor_start[0] +1, ypos + font_height,
		     note, strlen(note));
	 
	 if(ch_note->instrument) {
	    sprintf(ins, "%02X", ch_note->instrument);
	 }
	 else {
	    if(ch_note->note && ch_note->note < 72) {
	       sprintf(ins, "%02X", ch_note->instrument);
	    }
	    else {
	    sprintf(ins, "--");
	    }
	 }
	 
	 XDrawString(display, tracks_pixmap, small_gc,
		     xpos + cursor_start[1], ypos + font_height,
		     ins, 1);
	 XDrawString(display, tracks_pixmap, small_gc,
		     xpos + cursor_start[2], ypos + font_height,
		     ins+1, 1);
	 
	 if(ch_note->volume) {
	    sprintf(vol, "%02X", ch_note->volume);
	 }
	 else {
	    sprintf(vol, "--");
	 }
	 
	 XDrawString(display, tracks_pixmap, small_gc,
		     xpos + cursor_start[3], ypos + font_height,
		     vol, 1);
	 XDrawString(display, tracks_pixmap, small_gc,
		     xpos + cursor_start[4], ypos + font_height,
		     vol+1, 1);
	 
	 if(ch_note->effect_type) {
	    sprintf(eff_type, "%02X", ch_note->effect_type);
	 }
	 else {
	    sprintf(eff_type, "--");
	 }
	 
	 XDrawString(display, tracks_pixmap, small_gc,
		     xpos + cursor_start[5], ypos + font_height,
		     eff_type, 1);
	 XDrawString(display, tracks_pixmap, small_gc,
		     xpos + cursor_start[6], ypos + font_height,
		     eff_type+1, 1);
	 
	 if(ch_note->effect_parameter) {
	    sprintf(eff_parm, "%02X", ch_note->effect_parameter);
	 }
	 else {
	    sprintf(eff_parm, "--");
	 }
	 
	 XDrawString(display, tracks_pixmap, small_gc,
		     xpos + cursor_start[7], ypos + font_height,
		     eff_parm, 1);
	 XDrawString(display, tracks_pixmap, small_gc,
		     xpos + cursor_start[8], ypos + font_height,
		     eff_parm+1, 1);
	 
	 xpos += channel_width;
      }
      row_list = row_list->next;
      ypos += font_height + 2;
   }

}

void refresh_tracker(void)
{
   int font_height;
   int i;
   int first_channel; /* leftmost visible channel */
   char chan_num[3];
   int x,y;
   XPoint bvl_tri[6];
  
   if(tracker_tracks == 0L) {
      return;
   }
   

   font_height = large_font->ascent + large_font->descent;
   
   if(nums_pixmap == 0L) {
      create_nums_pixmap();
   }
   if(current_pattern != NULL && tracks_pixmap == 0L) {
      create_tracks_pixmap();
   }

   if(tracker_back_pixmap == 0L) {
      tracker_back_pixmap = XCreatePixmap(display, tracker_tracks,
					  width-20, height-30,
					  display_depth);
      XFillRectangle(display, tracker_back_pixmap, clear_gc, 0, 0,
		     width, height);
   }
   
   channels_visible = (int)((width - (2 * n_width))/(channel_width));
   if(channels_visible > mod->num_channels) {
      channels_visible = mod->num_channels;
   }
   
   rows_visible =
     (int)((height - (2 * (font_height + 2)))/(font_height + 2));
   
   if(current_track < channels_visible/2) {
      first_channel = 0;
   }
   else if(current_track >= mod->num_channels - (channels_visible/2)) {
      first_channel = mod->num_channels - (channels_visible);
   }
   else {
      first_channel = current_track - (channels_visible/2);
   }

   /* clear back_pixmap */

   XFillRectangle(display, tracker_back_pixmap, clear_gc,
		  0, 0, width, height);
   
   /* draw left number column */
   XCopyArea(display, nums_pixmap, tracker_back_pixmap, large_gc,
	     0, (current_row - rows_visible/2) * (font_height + 2),
	     n_width, height,
	     0, 0);
   
   /* draw channels */
   if(current_pattern != NULL) {   
      XCopyArea(display, tracks_pixmap, tracker_back_pixmap, small_gc,
		first_channel * channel_width,
		(current_row - rows_visible/2) * (font_height + 2),
		channels_visible * channel_width, height,
		n_width, 0);
   }
   
   /* draw right number column */
   XCopyArea(display, nums_pixmap, tracker_back_pixmap, large_gc,
	     0, (current_row - rows_visible/2) * (font_height + 2),
	     n_width, height,
	     width - n_width, 0);

   /* draw highlight */
   XFillRectangle(display, tracker_back_pixmap, hl_gc,
		  0, (rows_visible/2 + 1) * (font_height + 2) -1,
		  width, (font_height + 2));


   /* draw lines between channels and channel numbers*/
   y = height;
   x = n_width;
   for(i=0; i < channels_visible; i++) {
      
      XDrawLine(display, tracker_back_pixmap, bvl_high_gc, x-1, 0, x-1, y);
/*      XDrawLine(display, tracker_back_pixmap, cursor_gc, x, 0, x, y);*/
      XDrawLine(display, tracker_back_pixmap, bvl_low_gc, x+1, 0, x+1, y);
      
      sprintf(chan_num, "%X", first_channel + i);
      XDrawString(display, tracker_back_pixmap, cursor_gc,
		  x+4, large_font->ascent,
		  chan_num, strlen(chan_num));
      
      x += channel_width;
   }
   XDrawLine(display, tracker_back_pixmap, bvl_high_gc, x-1, 0, x-1, y);
/*   XDrawLine(display, tracker_back_pixmap, cursor_gc, x, 0, x, y);*/
   XDrawLine(display, tracker_back_pixmap, bvl_low_gc, x+1, 0, x+1, y);
   
   /* draw cursor */
   if(first_channel <= current_track &&
      current_track < first_channel + channels_visible) {
      
      XDrawRectangle(display, tracker_back_pixmap, cursor_gc,
		     n_width +
		     ((current_track - first_channel) * channel_width) +
		     cursor_start[current_pos] + 9,
		     (rows_visible/2 + 1) * (font_height + 2) - 2,
		     cursor_len[current_pos],
		     (font_height + 3));
   }
   

   /* finally, copy back pixmap to window */
   XCopyArea(display, tracker_back_pixmap, tracker_tracks, small_gc,
	     0, 0, width, height,
	     0, 0);
   
   /* And Flush the fucking display!! hehe */
   XSync(display, False);
}


void update_tracker(void)
{
   if(!tracker_tracks) {
      return;
   }
   
   create_tracker();
   refresh_tracker();
}


