#include <stdio.h>
#include <string.h>
#include <signal.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include <aube.h>

#include "panel.h"
#include "button.h"
#include "toggle.h"
#include "numrange.h"

extern char *progname;

extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC small_gc;
extern XFontStruct *small_font;
extern GC large_gc;
extern XFontStruct *large_font;
extern GC cursor_gc;
extern GC clear_gc;
extern GC hl_gc;

extern struct xm *mod;
extern struct xm_pattern *current_pattern;
extern int current_track;

extern int jump;

extern int player_pid;
extern playing;
extern struct toggle_data *current_play_toggle;

extern void edit_on(void);
extern void edit_off(void);

extern void stop_button_pressed(void);
extern void clear_current_xm_pattern(void);

extern Window tracker_tracks;

Window tracker_top;

struct toggle_data *play_pattern_toggle;
struct toggle_data *edit_toggle;

static struct panel *play_panel;
static struct panel *transpose_panel;
static struct panel *edit_panel;
static struct panel *patt_panel;
static struct panel *tracker_panel;

static struct button_data *rew_button;
static struct button_data *fwd_button;
static struct button_data *stop_button;
static struct button_data *expand_button;
static struct button_data *shrink_button;
static struct button_data *clear_pattern_button;
static struct button_data *del_pattern_button;
static struct button_data *ins_pattern_button;
static struct button_data *octup_pattern_button;
static struct button_data *up_pattern_button;
static struct button_data *down_pattern_button;
static struct button_data *octdown_pattern_button;
static struct button_data *octup_track_button;
static struct button_data *up_track_button;
static struct button_data *down_track_button;
static struct button_data *octdown_track_button;



static struct numrange *jump_numrange;

static struct label *jump_label;


void play_pattern_button_pressed(void)
{
   int pid_status;
   
   if(player_pid > 0) {
      kill((pid_t)player_pid, SIGKILL);
      waitpid((pid_t)player_pid, &pid_status, 0);
      player_pid = -1;
      if(current_play_toggle && current_play_toggle != play_pattern_toggle) {
	 set_toggle_state(current_play_toggle, 0);
      }
   }

   playing=1;
   current_play_toggle = play_pattern_toggle;
   
   player_pid = fork();
   if(!player_pid) {   
      play_single_pattern(current_pattern);
      exit(0);
   }
   else if (player_pid <= 0) {
      fprintf(stderr, "Unable to fork...\n");
   }
}

void rew_button_pressed(void)
{
   change_current_pattern(-1);
}

void fwd_button_pressed(void)
{
   change_current_pattern(1);
}

void jump_changed(int new_val)
{
   jump = new_val;
}


void expand_button_pressed(void)
{
   expand_current_xm_pattern();
   update_tracker();
}

void shrink_button_pressed(void)
{
   shrink_current_xm_pattern();
   update_tracker();
}

void clear_pattern_button_pressed(void)
{
   clear_current_xm_pattern();
   update_tracker();
}

void del_pattern_button_pressed(void)
{
   del_current_xm_pattern();
   update_modinfo();
   update_tracker();
}

void ins_pattern_button_pressed(void)
{
   ins_current_xm_pattern();
   update_tracker();
}

void octup_pattern_button_pressed(void)
{
   transpose_pattern(current_pattern, 12);
   update_tracker();
}

void up_pattern_button_pressed(void)
{
   transpose_pattern(current_pattern, 1);
   update_tracker();
}

void down_pattern_button_pressed(void)
{
   transpose_pattern(current_pattern, -1);
   update_tracker();
}

void octdown_pattern_button_pressed(void)
{
   transpose_pattern(current_pattern, -12);
   update_tracker();
}

void octup_track_button_pressed(void)
{
   transpose_track(current_pattern, 12, current_track);
   update_tracker();
}

void up_track_button_pressed(void)
{
   transpose_track(current_pattern, 1, current_track);
   update_tracker();
}

void down_track_button_pressed(void)
{
   transpose_track(current_pattern, -1, current_track);
   update_tracker();
}

void octdown_track_button_pressed(void)
{
   transpose_track(current_pattern, -12, current_track);
   update_tracker();
}

void create_tracker_window(void)
{
   char window_name[256];
   char *icon_name = "maube tracker";
   int width, height;
   int button_width, button_height;
   
   /* fixed width fonts
    width = 747;
    */
/*   
   width = 970;
   height = 474;
*/
   width = 700;
   height = 320;
   
   button_width = 50;
   button_height = 16;
   
   sprintf(window_name, "maube tracker: %s", mod->mod_name);
   
   tracker_top = create_toplevel_window(window_name, window_name,
					0, 0, width, height, 400, 200);

   tracker_panel = create_panel(tracker_top, 2, 2*button_height + 14, width-4, height-(2*button_height + 16));
   create_tracks_window(tracker_panel->window, 8, 2, width-14, height-(2*button_height + 20));

   play_panel = create_panel(tracker_top,
			     button_width*8 + 14, 2, 5*button_width/2 +10, button_height+4);
   
   play_pattern_toggle = create_play_toggle(play_panel->window,
					    8, 2,
					    button_width, button_height,
					    (void *)play_pattern_button_pressed,
					    (void *)stop_button_pressed,
					    (char *)"Play with yourself");

   rew_button = create_rew_button(play_panel->window,
				  8 + button_width, 2,
				  button_width/2, button_height,
				  (void *)rew_button_pressed,
				  (char *)"backtrack!");
   
   fwd_button = create_fwd_button(play_panel->window,
				  8 + (3*button_width)/2, 2,
				  button_width/2, button_height,
				  (void *)fwd_button_pressed,
				  (char *)"go forth for fuck's sake!");
   
   stop_button = create_stop_button(play_panel->window,
				    8 + 2*button_width, 2,
				    button_width/2, button_height,
				    (void *)stop_button_pressed,
				    (char *)"Stop this nonsense!");

   edit_panel = create_panel(tracker_top, 2, button_height+8, button_width + 98, button_height+4);
   
   edit_toggle = create_text_toggle(edit_panel->window, 8, 2,
				    button_width, button_height,
				    (void *)edit_on, (void *)edit_off,
				    (char *)"edit",
				    (char *)"Unleash the asparagus!");
   
   jump_label = create_label(edit_panel->window,
			     8 + button_width, 2, 38, button_height, (char *)"jump!");
   
   jump_numrange = create_numrange(edit_panel->window,
				   8 + button_width + 38, 2, 50, button_height,
				   jump,
				   - current_pattern->num_rows,
				   current_pattern->num_rows,
				   (void *)jump_changed);

   patt_panel = create_panel(tracker_top,
			     button_width+102, button_height+8, 5*button_width + 10, button_height+4);
   
   expand_button = create_text_button(patt_panel->window,
				      8, 2,
				      button_width, button_height,
				      (void *)expand_button_pressed,
				      (char *)"expand",
				      (char *)"Beefy pecs!");
   
   shrink_button = create_text_button(patt_panel->window,
				      8 + button_width, 2,
				      button_width, button_height,
				      (void *)shrink_button_pressed,
				      (char *)"shrink",
				      (char *)"Creeeeeeeeaaaarrrrgh!!");

   clear_pattern_button = create_text_button(patt_panel->window,
					     8 + button_width*2, 2,
					     button_width, button_height,
					     (void *)clear_pattern_button_pressed,
					     (char *)"clear patt",
					     (char *)"This pattern's fucked up! I wanna new one!");

   del_pattern_button = create_text_button(patt_panel->window,
					   8 + button_width*3, 2,
					   button_width, button_height,
					   (void *)del_pattern_button_pressed,
					   (char *)"del patt",
					   (char *)"Aaaaargh!! Get rid of this pattern!");
   
   ins_pattern_button = create_text_button(patt_panel->window,
					   8 + button_width*4, 2,
					   button_width, button_height,
					   (void *)ins_pattern_button_pressed,
					   (char *)"ins patt",
					   (char *)"Stick it in here honey!");

   transpose_panel = create_panel(tracker_top,
				  2, 2, button_width*8 + 10, button_height+4);
   
   octup_pattern_button = create_text_button(transpose_panel->window,
					     8, 2,
					     button_width, button_height,
					     (void *)octup_pattern_button_pressed,
					     (char *)"patt +12",
					     (char *)"transpose current pattern up one octave");
   
   up_pattern_button = create_text_button(transpose_panel->window,
					  8 + button_width, 2,
					  button_width, button_height,
					  (void *)up_pattern_button_pressed,
					  (char *)"patt +",
					  (char *)"transpose current pattern up one semitone");
   
   down_pattern_button = create_text_button(transpose_panel->window,
					    8 + button_width*2, 2,
					    button_width, button_height,
					    (void *)down_pattern_button_pressed,
					    (char *)"patt -",
					    (char *)"transpose current pattern down one semitone");
   
   octdown_pattern_button = create_text_button(transpose_panel->window,
					       8 + button_width*3, 2,
					       button_width, button_height,
					       (void *)octdown_pattern_button_pressed,
					       (char *)"patt -12",
					       (char *)"transpose current pattern down one octave");
   
   octup_track_button = create_text_button(transpose_panel->window,
					   8 + button_width*4, 2,
					   button_width, button_height,
					   (void *)octup_track_button_pressed,
					   (char *)"track +12",
					   (char *)"transpose current track up one octave");
   
   up_track_button = create_text_button(transpose_panel->window,
					8 + button_width*5, 2,
					button_width, button_height,
					(void *)up_track_button_pressed,
					(char *)"track +",
					(char *)"transpose current track up one semitone");
   
   down_track_button = create_text_button(transpose_panel->window,
					  8 + button_width*6, 2,
					  button_width, button_height,
					  (void *)down_track_button_pressed,
					  (char *)"track -",
					  (char *)"transpose current track down one semitone");
   
   octdown_track_button = create_text_button(transpose_panel->window,
					     8 + button_width*7, 2,
					     button_width, button_height,
					     (void *)octdown_track_button_pressed,
					     (char *)"track -12",
					     (char *)"transpose current track down one octave");
   
   XMapWindow(display, tracker_top);
}

void refresh_tracker_window(void)
{
   char tracker_title[256];
   
   if(!tracker_top) return;
   
   sprintf(tracker_title, "maube tracker: %s", mod->mod_name);
   change_window_title(tracker_top, tracker_title);
   
   refresh_tracker();
}

void close_tracker_window(void)
{
   if(tracker_top) {
      XDestroyWindow(display, tracker_top);
      tracker_top = 0L;
      tracker_tracks = 0L;
   }
   
}

   
