#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "button.h"


extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC small_gc;
extern GC large_gc;
extern XFontStruct *small_font;
extern XFontStruct *large_font;
extern GC bvl_low_gc;
extern GC bvl_high_gc;
extern GC butt_hl_gc;
extern GC text_hl_gc;
extern GC clear_gc;
extern GC hl_gc;
extern GC label_gc;
extern GC label_hl_gc;

struct button_data_list *buttons;


int check_buttons_destroyed(XEvent report)
{
   struct button_data_list *bdl, *pbdl=NULL;
   
   for(bdl = buttons; bdl; bdl = bdl->next) {
      if(bdl->data->window == report.xdestroywindow.window) {
	 if(pbdl) {
	    pbdl->next = bdl->next;
	 }
	 else {
	    buttons = bdl->next;
	 }
	 free(bdl->data);
	 free(bdl);
	 return 1;
      }
   }
   return 0;
}

void add_button(struct button_data *b)
{
   struct button_data_list *bdl;

   bdl = (struct button_data_list *)malloc(sizeof(struct button_data_list));
   bdl->data = b;
   
   if(buttons == NULL) {
      bdl->next = NULL;
      buttons = bdl;
   }
   else {
      bdl->next = buttons;
      buttons = bdl;
   }
}
   
struct button_data *create_button(Window parent, int x, int y, int width, int height,
				  void *pressed_func(), void *arg,
				  char *st,
				  Pixmap unset_pixmap, Pixmap set_pixmap,
				  Pixmap hl_pixmap)
{
   struct button_data *b;
   
   b = (struct button_data *)malloc(sizeof(struct button_data));
   
   b->window = create_subwindow(parent, x, y, width, height);
   b->width = width;
   b->height = height;
   b->pressed = (void *)pressed_func;
   b->arg = arg;
   
   b->status_text = st;
   
   b->unset_pixmap = unset_pixmap;
   b->set_pixmap = set_pixmap;
   b->hl_pixmap = hl_pixmap;
   b->current_pixmap = b->unset_pixmap;
   
   add_button(b);
   
   return(b);
}

struct button_data *create_text_button(Window parent, int x, int y,
				       int width, int height,
				       void *pressed_func(), char *text,
				       char *st)
{		
   Pixmap unset_pixmap, set_pixmap, hl_pixmap;
    
   unset_pixmap = create_button_pixmap(parent, width, height);
   label_button_pixmap(unset_pixmap, width, height, text);
   
   set_pixmap = create_button_set_pixmap(parent, width, height);
   label_button_hl_pixmap(set_pixmap, width, height, text);
   
   hl_pixmap = create_button_hl_pixmap(parent, width , height);
   label_button_hl_pixmap(hl_pixmap, width, height, text);
   
   return(create_button(parent, x, y, width, height, pressed_func, NULL, st,
			unset_pixmap, set_pixmap, hl_pixmap));
}

struct button_data *create_play_button(Window parent, int x, int y,
				       int width, int height,
				       void *pressed_func(), char *st)
{		
   Pixmap unset_pixmap, set_pixmap, hl_pixmap;
    
   unset_pixmap = create_button_pixmap(parent, width, height);
   play_button_pixmap(unset_pixmap, width, height);
   
   set_pixmap = create_button_set_pixmap(parent, width, height);
   play_button_hl_pixmap(set_pixmap, width, height);
   
   hl_pixmap = create_button_hl_pixmap(parent, width , height);
   play_button_hl_pixmap(hl_pixmap, width, height);
   
   return(create_button(parent, x, y, width, height, pressed_func, NULL, st,
			unset_pixmap, set_pixmap, hl_pixmap));
}

struct button_data *create_stop_button(Window parent, int x, int y,
				       int width, int height,
				       void *pressed_func(), char *st)
{		
   Pixmap unset_pixmap, set_pixmap, hl_pixmap;
    
   unset_pixmap = create_button_pixmap(parent, width, height);
   stop_button_pixmap(unset_pixmap, width, height);
   
   set_pixmap = create_button_set_pixmap(parent, width, height);
   stop_button_hl_pixmap(set_pixmap, width, height);
   
   hl_pixmap = create_button_hl_pixmap(parent, width , height);
   stop_button_hl_pixmap(hl_pixmap, width, height);
   
   return(create_button(parent, x, y, width, height, pressed_func, NULL, st,
			unset_pixmap, set_pixmap, hl_pixmap));
}

struct button_data *create_arrow_left_button(Window parent, int x, int y,
					     int width, int height,
					     void *pressed_func(), void *arg,
					     char *st)
{		
   Pixmap unset_pixmap, set_pixmap, hl_pixmap;
    
   unset_pixmap = create_button_pixmap(parent, width, height);
   arrow_left_button_pixmap(unset_pixmap, width, height);
   
   set_pixmap = create_button_set_pixmap(parent, width, height);
   arrow_left_button_hl_pixmap(set_pixmap, width, height);
   
   hl_pixmap = create_button_hl_pixmap(parent, width , height);
   arrow_left_button_hl_pixmap(hl_pixmap, width, height);
   
   return(create_button(parent, x, y, width, height, pressed_func, arg, st,
			unset_pixmap, set_pixmap, hl_pixmap));
}

struct button_data *create_arrow_right_button(Window parent, int x, int y,
					      int width, int height,
					      void *pressed_func(), void *arg,
					      char *st)
{		
   Pixmap unset_pixmap, set_pixmap, hl_pixmap;
    
   unset_pixmap = create_button_pixmap(parent, width, height);
   arrow_right_button_pixmap(unset_pixmap, width, height);
   
   set_pixmap = create_button_set_pixmap(parent, width, height);
   arrow_right_button_hl_pixmap(set_pixmap, width, height);
   
   hl_pixmap = create_button_hl_pixmap(parent, width , height);
   arrow_right_button_hl_pixmap(hl_pixmap, width, height);
   
   return(create_button(parent, x, y, width, height, pressed_func, arg, st,
			unset_pixmap, set_pixmap, hl_pixmap));
}

struct button_data *create_arrow_miniup_button(Window parent, int x, int y,
					       int width, int height,
					       void *pressed_func(), void *arg,
					       char *st)
{		
   Pixmap unset_pixmap, set_pixmap, hl_pixmap;
    
   unset_pixmap = create_button_pixmap(parent, width, height);
   arrow_miniup_button_pixmap(unset_pixmap, width, height);
   
   set_pixmap = create_button_set_pixmap(parent, width, height);
   arrow_miniup_button_hl_pixmap(set_pixmap, width, height);
   
   hl_pixmap = create_button_hl_pixmap(parent, width , height);
   arrow_miniup_button_hl_pixmap(hl_pixmap, width, height);
   
   return(create_button(parent, x, y, width, height, pressed_func, arg, st,
			unset_pixmap, set_pixmap, hl_pixmap));
}

struct button_data *create_arrow_minidown_button(Window parent, int x, int y,
					      int width, int height,
					      void *pressed_func(), void *arg,
					      char *st)
{		
   Pixmap unset_pixmap, set_pixmap, hl_pixmap;
    
   unset_pixmap = create_button_pixmap(parent, width, height);
   arrow_minidown_button_pixmap(unset_pixmap, width, height);
   
   set_pixmap = create_button_set_pixmap(parent, width, height);
   arrow_minidown_button_hl_pixmap(set_pixmap, width, height);
   
   hl_pixmap = create_button_hl_pixmap(parent, width , height);
   arrow_minidown_button_hl_pixmap(hl_pixmap, width, height);
   
   return(create_button(parent, x, y, width, height, pressed_func, arg, st,
			unset_pixmap, set_pixmap, hl_pixmap));
}

struct button_data *create_arrow_up_button(Window parent, int x, int y,
					   int width, int height,
					   void *pressed_func(), void *arg,
					   char *st)
{		
   Pixmap unset_pixmap, set_pixmap, hl_pixmap;
    
   unset_pixmap = create_button_pixmap(parent, width, height);
   arrow_up_button_pixmap(unset_pixmap, width, height);
   
   set_pixmap = create_button_set_pixmap(parent, width, height);
   arrow_up_button_hl_pixmap(set_pixmap, width, height);
   
   hl_pixmap = create_button_hl_pixmap(parent, width , height);
   arrow_up_button_hl_pixmap(hl_pixmap, width, height);
   
   return(create_button(parent, x, y, width, height, pressed_func, arg, st,
			unset_pixmap, set_pixmap, hl_pixmap));
}

struct button_data *create_arrow_down_button(Window parent, int x, int y,
					     int width, int height,
					     void *pressed_func(), void *arg,
					     char *st)
{		
   Pixmap unset_pixmap, set_pixmap, hl_pixmap;
    
   unset_pixmap = create_button_pixmap(parent, width, height);
   arrow_down_button_pixmap(unset_pixmap, width, height);
   
   set_pixmap = create_button_set_pixmap(parent, width, height);
   arrow_down_button_hl_pixmap(set_pixmap, width, height);
   
   hl_pixmap = create_button_hl_pixmap(parent, width , height);
   arrow_down_button_hl_pixmap(hl_pixmap, width, height);
   
   return(create_button(parent, x, y, width, height, pressed_func, arg, st,
			unset_pixmap, set_pixmap, hl_pixmap));
}

struct button_data *create_rew_button(Window parent, int x, int y,
				      int width, int height,
				      void *pressed_func(), char *st)
{		
   Pixmap unset_pixmap, set_pixmap, hl_pixmap;
    
   unset_pixmap = create_button_pixmap(parent, width, height);
   rew_button_pixmap(unset_pixmap, width, height);
   
   set_pixmap = create_button_set_pixmap(parent, width, height);
   rew_button_hl_pixmap(set_pixmap, width, height);
   
   hl_pixmap = create_button_hl_pixmap(parent, width , height);
   rew_button_hl_pixmap(hl_pixmap, width, height);
   
   return(create_button(parent, x, y, width, height, pressed_func, NULL, st,
			unset_pixmap, set_pixmap, hl_pixmap));
}

struct button_data *create_fwd_button(Window parent, int x, int y,
				      int width, int height,
				      void *pressed_func(), char *st)
{		
   Pixmap unset_pixmap, set_pixmap, hl_pixmap;
    
   unset_pixmap = create_button_pixmap(parent, width, height);
   fwd_button_pixmap(unset_pixmap, width, height);
   
   set_pixmap = create_button_set_pixmap(parent, width, height);
   fwd_button_hl_pixmap(set_pixmap, width, height);
   
   hl_pixmap = create_button_hl_pixmap(parent, width , height);
   fwd_button_hl_pixmap(hl_pixmap, width, height);
   
   return(create_button(parent, x, y, width, height, pressed_func, NULL, st,
			unset_pixmap, set_pixmap, hl_pixmap));
}

int check_buttons_pressdown(XEvent report)
{
   struct button_data_list *bdl;
   
   for(bdl = buttons; bdl != NULL; bdl = bdl->next) {
      if(bdl->data->window == report.xbutton.window) {
	 bdl->data->current_pixmap = bdl->data->set_pixmap;
	 
	 refresh_button(bdl->data);
	 return 1;
      }
   }
   return 0;
}

int check_buttons_pressed(XEvent report)
{
   struct button_data_list *bdl;
   
   for(bdl = buttons; bdl != NULL; bdl = bdl->next) {
      if(bdl->data->window == report.xbutton.window) {
	 bdl->data->current_pixmap = bdl->data->unset_pixmap;
	 refresh_button(bdl->data);
	 bdl->data->pressed(bdl->data->arg);
	 return 1;
      }
   }
   return 0;
}

int check_buttons_mouseover(XEvent report)
{
   struct button_data_list *bdl;
   
   for(bdl = buttons; bdl != NULL; bdl = bdl->next) {
      if(bdl->data->window == report.xcrossing.window) {
	 bdl->data->current_pixmap = bdl->data->hl_pixmap;
	 refresh_button(bdl->data);
	 set_status_text(bdl->data->status_text);
	 return 1;
      }
   }
   return 0;
}

int check_buttons_mouseexit(XEvent report)
{
   struct button_data_list *bdl;
   
   for(bdl = buttons; bdl != NULL; bdl = bdl->next) {
      if(bdl->data->window == report.xcrossing.window) {
	 bdl->data->current_pixmap = bdl->data->unset_pixmap;
	 refresh_button(bdl->data);
	 return 1;
      }
   }
   return 0;
}

int check_buttons_expose(XEvent report)
{
   struct button_data_list *bdl;
   
   for(bdl = buttons; bdl != NULL; bdl = bdl->next) {
      if(bdl->data->window == report.xexpose.window) {
	 refresh_button(bdl->data);
	 return 1;
      }
   }
   return 0;
}

void refresh_button(struct button_data *button)
{
   if(button->current_pixmap == 0L) {
      return;
   }

   XCopyArea(display, button->current_pixmap, button->window, large_gc,
	     0, 0,
	     button->width, button->height,
	     0, 0);
}

Pixmap create_button_pixmap(Window window,
			    unsigned int width, unsigned int height)
{
   Pixmap pixmap;
   XPoint bvl_tri[6];

   pixmap = XCreatePixmap(display, window, width, height,
			  display_depth);

   XFillRectangle(display, pixmap, bvl_high_gc, 0, 0, width, height);
   
   bvl_tri[0].x = width; bvl_tri[0].y = 0;
   bvl_tri[1].x = width; bvl_tri[1].y = height;
   bvl_tri[2].x = 0; bvl_tri[2].y = height;
   bvl_tri[3].x = 2; bvl_tri[3].y = height-2;
   bvl_tri[4].x = width-2; bvl_tri[4].y = height-2;
   bvl_tri[5].x = width-2; bvl_tri[5].y = 2;
   XFillPolygon(display, pixmap, bvl_low_gc, bvl_tri, 6,
		Nonconvex, CoordModeOrigin);

   XFillRectangle(display, pixmap, clear_gc, 1, 1, width-2, height-2);

   return(pixmap);
}

Pixmap create_button_set_pixmap(Window window,
				unsigned int width, unsigned int height)
{
   Pixmap pixmap;
   XPoint bvl_tri[6];

   pixmap = XCreatePixmap(display, window, width, height,
			  display_depth);

   XFillRectangle(display, pixmap, bvl_high_gc, 0, 0, width, height);
   
   bvl_tri[0].x = 0; bvl_tri[0].y = 0;
   bvl_tri[1].x = 0; bvl_tri[1].y = height;
   bvl_tri[2].x = 2; bvl_tri[2].y = height-2;
   bvl_tri[3].x = 2; bvl_tri[3].y = 2;
   bvl_tri[4].x = width-2; bvl_tri[4].y = 2;
   bvl_tri[5].x = width; bvl_tri[5].y = 0;
   XFillPolygon(display, pixmap, bvl_low_gc, bvl_tri, 6,
		Nonconvex, CoordModeOrigin);
   
   XFillRectangle(display, pixmap, clear_gc, 1, 1, width-2, height-2);   
   
   return(pixmap);
}

Pixmap create_button_hl_pixmap(Window window,
			    unsigned int width, unsigned int height)
{
   Pixmap pixmap;
   XPoint bvl_tri[6];

   pixmap = XCreatePixmap(display, window, width, height,
			  display_depth);


   XFillRectangle(display, pixmap, bvl_high_gc, 0, 0, width, height);

   bvl_tri[0].x = width; bvl_tri[0].y = 0;
   bvl_tri[1].x = width; bvl_tri[1].y = height;
   bvl_tri[2].x = 0; bvl_tri[2].y = height;
   bvl_tri[3].x = 2; bvl_tri[3].y = height-2;
   bvl_tri[4].x = width-2; bvl_tri[4].y = height-2;
   bvl_tri[5].x = width-2; bvl_tri[5].y = 2;
   XFillPolygon(display, pixmap, bvl_low_gc, bvl_tri, 6,
		Nonconvex, CoordModeOrigin);

   XFillRectangle(display, pixmap, butt_hl_gc, 1, 1, width-2, height-2);
   
   return(pixmap);
}

void label_button_pixmap(Pixmap pixmap,
			 unsigned int width, unsigned int height,
			 char *button_text)
{
   int font_height;
   int text_width, xpos, ypos;

   font_height = large_font->ascent + large_font->descent;

   text_width = XTextWidth(large_font, button_text, strlen(button_text));
   
   xpos = (width - text_width)/2;
   ypos = (height - font_height)/2 + large_font->ascent;
/*   
   XDrawString(display, pixmap, bvl_low_gc, xpos-1, ypos-1,
	       button_text, strlen(button_text));
   XDrawString(display, pixmap, bvl_low_gc, xpos-1, ypos+1,
	       button_text, strlen(button_text));
   XDrawString(display, pixmap, bvl_low_gc, xpos+1, ypos-1,
	       button_text, strlen(button_text));
   XDrawString(display, pixmap, bvl_high_gc, xpos+1, ypos+1,
	       button_text, strlen(button_text));
*/
   XDrawString(display, pixmap, large_gc, xpos, ypos,
	       button_text, strlen(button_text));
   
}

void label_button_hl_pixmap(Pixmap pixmap,
			   unsigned int width, unsigned int height,
			   char *button_text)
{
   int font_height;
   int text_width, xpos, ypos;

   font_height = large_font->ascent + large_font->descent;

   text_width = XTextWidth(large_font, button_text, strlen(button_text));
   
   xpos = (width - text_width)/2;
   ypos = (height - font_height)/2 + large_font->ascent;
/*   
   XDrawString(display, pixmap, bvl_low_gc, xpos-1, ypos-1,
	       button_text, strlen(button_text));
   XDrawString(display, pixmap, bvl_low_gc, xpos-1, ypos+1,
	       button_text, strlen(button_text));
   XDrawString(display, pixmap, bvl_low_gc, xpos+1, ypos-1,
	       button_text, strlen(button_text));
   XDrawString(display, pixmap, bvl_high_gc, xpos+1, ypos+1,
	       button_text, strlen(button_text));
*/
   XDrawString(display, pixmap, text_hl_gc, xpos, ypos,
	       button_text, strlen(button_text));
   
}

void play_button_pixmap(Pixmap pixmap,
			unsigned int width, unsigned int height)
{
   XPoint tri[3];
   
   tri[1].x = 0; tri[1].y = 8;
   tri[2].x = 24; tri[2].y = -4;

   tri[0].x = width/2 - 13; tri[0].y = height/2 - 5;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 - 13; tri[0].y = height/2 - 3;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 - 11; tri[0].y = height/2 - 5;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 - 11; tri[0].y = height/2 - 3;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 3, Convex, CoordModePrevious);
   
   tri[0].x = width/2 - 12; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, label_gc, tri, 3, Convex, CoordModePrevious);
}

void play_button_hl_pixmap(Pixmap pixmap,
			   unsigned int width, unsigned int height)
{
   XPoint tri[3];
   
   tri[1].x = 0; tri[1].y = 8;
   tri[2].x = 24; tri[2].y = -4;

   tri[0].x = width/2 - 13; tri[0].y = height/2 - 5;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 - 13; tri[0].y = height/2 - 3;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 - 11; tri[0].y = height/2 - 5;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 - 11; tri[0].y = height/2 - 3;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 3, Convex, CoordModePrevious);
   
   tri[0].x = width/2 - 12; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, label_hl_gc, tri, 3, Convex, CoordModePrevious);   
}

void stop_button_pixmap(Pixmap pixmap,
			unsigned int width, unsigned int height)
{
   XFillRectangle(display, pixmap, bvl_low_gc, width/2 - 5, height/2 - 5, 10, 10);
   XFillRectangle(display, pixmap, bvl_high_gc, width/2 - 3, height/2 - 3, 8, 8);
   XFillRectangle(display, pixmap, label_gc, width/2 - 4, height/2 - 4, 8, 8);
}

void stop_button_hl_pixmap(Pixmap pixmap,
			unsigned int width, unsigned int height)
{
   XFillRectangle(display, pixmap, bvl_low_gc, width/2 - 5, height/2 - 5, 10, 10);
   XFillRectangle(display, pixmap, bvl_high_gc, width/2 - 3, height/2 - 3, 8, 8);
   XFillRectangle(display, pixmap, label_hl_gc, width/2 - 4, height/2 - 4, 8, 8);
}

void arrow_right_button_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height)
{
   XPoint tri[3];
   
   tri[1].x = 0; tri[1].y = 8;
   tri[2].x = 8; tri[2].y = -4;

   tri[0].x = width/2 - 5; tri[0].y = height/2 - 5;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 - 5; tri[0].y = height/2 - 3;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 - 3; tri[0].y = height/2 - 5;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 - 3; tri[0].y = height/2 - 3;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 3, Convex, CoordModePrevious);
   
   tri[0].x = width/2 - 4; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, label_gc, tri, 3, Convex, CoordModePrevious);
}

void arrow_right_button_hl_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height)
{
   XPoint tri[3];
   
   tri[1].x = 0; tri[1].y = 8;
   tri[2].x = 8; tri[2].y = -4;

   tri[0].x = width/2 - 5; tri[0].y = height/2 - 5;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 - 5; tri[0].y = height/2 - 3;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 - 3; tri[0].y = height/2 - 5;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 - 3; tri[0].y = height/2 - 3;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 3, Convex, CoordModePrevious);
   
   tri[0].x = width/2 - 4; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, label_hl_gc, tri, 3, Convex, CoordModePrevious);
}

void arrow_left_button_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height)
{
   XPoint tri[3];
   
   tri[1].x = 0; tri[1].y = -8;
   tri[2].x = -8; tri[2].y = 4;
   
   tri[0].x = width/2 + 3; tri[0].y = height/2 + 3;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 + 3; tri[0].y = height/2 + 4;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 + 3; tri[0].y = height/2 + 5;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 + 5; tri[0].y = height/2 + 5;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 3, Convex, CoordModePrevious);
   
   tri[0].x = width/2 + 4; tri[0].y = height/2 + 4;
   XFillPolygon(display, pixmap, label_gc, tri, 3, Convex, CoordModePrevious);
}

void arrow_left_button_hl_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height)
{
   XPoint tri[3];
   
   tri[1].x = 0; tri[1].y = -8;
   tri[2].x = -8; tri[2].y = 4;
   
   tri[0].x = width/2 + 3; tri[0].y = height/2 + 3;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 + 3; tri[0].y = height/2 + 4;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 + 3; tri[0].y = height/2 + 5;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 + 5; tri[0].y = height/2 + 5;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 3, Convex, CoordModePrevious);

   tri[0].x = width/2 + 4; tri[0].y = height/2 + 4;   
   XFillPolygon(display, pixmap, label_hl_gc, tri, 3, Convex, CoordModePrevious);
}

void arrow_miniup_button_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height)
{
   XPoint tri[3];
   
   tri[0].x = width/2; tri[0].y = height/2 - 3;
   tri[1].x = - 2; tri[1].y = 5;
   tri[2].x = 4; tri[2].y = 0;

   XFillPolygon(display, pixmap, label_gc, tri, 3, Convex, CoordModePrevious);
}

void arrow_miniup_button_hl_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height)
{
   XPoint tri[3];
   
   tri[0].x = width/2; tri[0].y = height/2 - 2;
   tri[1].x = - 2; tri[1].y = 4;
   tri[2].x = 4; tri[2].y = 0;

   XFillPolygon(display, pixmap, label_hl_gc, tri, 3, Convex, CoordModePrevious);
}


void arrow_minidown_button_pixmap(Pixmap pixmap,
				  unsigned int width, unsigned int height)
{
   XPoint tri[3];
   
   tri[0].x = width/2; tri[0].y = height/2 + 2;
   tri[1].x = -2; tri[1].y = -4;
   tri[2].x = 4; tri[2].y = 0;

   XFillPolygon(display, pixmap, label_gc, tri, 3, Convex, CoordModePrevious);
}

void arrow_minidown_button_hl_pixmap(Pixmap pixmap,
				     unsigned int width, unsigned int height)
{
   XPoint tri[3];
   
   tri[0].x = width/2; tri[0].y = height/2 + 2;
   tri[1].x = -2; tri[1].y = -4;
   tri[2].x = 4; tri[2].y = 0;

   XFillPolygon(display, pixmap, label_hl_gc, tri, 3, Convex, CoordModePrevious);
}

void arrow_up_button_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height)
{
   XPoint tri[3];
   
   tri[1].x = -3; tri[1].y = 6;
   tri[2].x = 6; tri[2].y = 0;

   tri[0].x = width/2-1; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2-1; tri[0].y = height/2 - 2;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2+1; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2+1; tri[0].y = height/2 - 2;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 3, Convex, CoordModePrevious);
   
   tri[0].x = width/2; tri[0].y = height/2 - 3;
   XFillPolygon(display, pixmap, label_gc, tri, 3, Convex, CoordModePrevious);
}

void arrow_up_button_hl_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height)
{
   XPoint tri[3];
   
   tri[1].x = -3; tri[1].y = 6;
   tri[2].x = 6; tri[2].y = 0;

   tri[0].x = width/2-1; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2-1; tri[0].y = height/2 - 2;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2+1; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2+1; tri[0].y = height/2 - 2;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 3, Convex, CoordModePrevious);
   
   tri[0].x = width/2; tri[0].y = height/2 - 3;
   XFillPolygon(display, pixmap, label_hl_gc, tri, 3, Convex, CoordModePrevious);
}

void arrow_down_button_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height)
{
   XPoint tri[3];
   

   tri[1].x = -3; tri[1].y = -6;
   tri[2].x = 6; tri[2].y = 0;

   tri[0].x = width/2 -1; tri[0].y = height/2 + 2;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 -1; tri[0].y = height/2 + 4;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 +1; tri[0].y = height/2 + 2;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 +1; tri[0].y = height/2 + 4;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 3, Convex, CoordModePrevious);
   
   tri[0].x = width/2; tri[0].y = height/2 + 3;
   XFillPolygon(display, pixmap, label_gc, tri, 3, Convex, CoordModePrevious);
}

void arrow_down_button_hl_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height)
{
   XPoint tri[3];
   
   tri[1].x = -3; tri[1].y = -6;
   tri[2].x = 6; tri[2].y = 0;

   tri[0].x = width/2 -1; tri[0].y = height/2 + 2;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 -1; tri[0].y = height/2 + 4;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 +1; tri[0].y = height/2 + 2;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 3, Convex, CoordModePrevious);
   tri[0].x = width/2 +1; tri[0].y = height/2 + 4;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 3, Convex, CoordModePrevious);

   tri[0].x = width/2; tri[0].y = height/2 + 3;
   XFillPolygon(display, pixmap, label_hl_gc, tri, 3, Convex, CoordModePrevious);
}

void rew_button_pixmap(Pixmap pixmap,
		       unsigned int width, unsigned int height)
{
   XPoint tri[7];
   
   tri[1].x = 0; tri[1].y = 10;
   tri[2].x = -5; tri[2].y = -5;
   tri[3].x = 0; tri[3].y = 5;
   tri[4].x = -5; tri[4].y = -5;
   tri[5].x = 5; tri[5].y = -5;
   tri[6].x = 0; tri[6].y = 5; 

   tri[0].x = width/2 + 4; tri[0].y = height/2 - 6;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 7, Nonconvex, CoordModePrevious);
   tri[0].x = width/2 + 6; tri[0].y = height/2 - 6;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 7, Nonconvex, CoordModePrevious);
   tri[0].x = width/2 + 4; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 7, Nonconvex, CoordModePrevious);
   tri[0].x = width/2 + 6; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 7, Nonconvex, CoordModePrevious);
   
   tri[0].x = width/2 + 5; tri[0].y = height/2 - 5;
   XFillPolygon(display, pixmap, label_gc, tri, 7, Nonconvex, CoordModePrevious);
}

void rew_button_hl_pixmap(Pixmap pixmap,
		       unsigned int width, unsigned int height)
{
   XPoint tri[7];
   
   tri[1].x = 0; tri[1].y = 10;
   tri[2].x = -5; tri[2].y = -5;
   tri[3].x = 0; tri[3].y = 5;
   tri[4].x = -5; tri[4].y = -5;
   tri[5].x = 5; tri[5].y = -5;
   tri[6].x = 0; tri[6].y = 5; 

   tri[0].x = width/2 + 4; tri[0].y = height/2 - 6;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 7, Nonconvex, CoordModePrevious);
   tri[0].x = width/2 + 6; tri[0].y = height/2 - 6;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 7, Nonconvex, CoordModePrevious);
   tri[0].x = width/2 + 4; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 7, Nonconvex, CoordModePrevious);
   tri[0].x = width/2 + 6; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 7, Nonconvex, CoordModePrevious);

   tri[0].x = width/2 + 5; tri[0].y = height/2 - 5;
   XFillPolygon(display, pixmap, label_hl_gc, tri, 7, Nonconvex, CoordModePrevious);
}

void fwd_button_pixmap(Pixmap pixmap,
		       unsigned int width, unsigned int height)
{
   XPoint tri[7];
   
   tri[1].x = 0; tri[1].y = 10;
   tri[2].x = 5; tri[2].y = -5;
   tri[3].x = 0; tri[3].y = 5;
   tri[4].x = 5; tri[4].y = -5;
   tri[5].x = -5; tri[5].y = -5;
   tri[6].x = 0; tri[6].y = 5; 

   tri[0].x = width/2 - 6; tri[0].y = height/2 - 6;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 7, Nonconvex, CoordModePrevious);
   tri[0].x = width/2 - 6; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 7, Nonconvex, CoordModePrevious);
   tri[0].x = width/2 - 4; tri[0].y = height/2 - 6;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 7, Nonconvex, CoordModePrevious);
   tri[0].x = width/2 - 4; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 7, Nonconvex, CoordModePrevious);
   
   tri[0].x = width/2 - 5; tri[0].y = height/2 - 5;
   XFillPolygon(display, pixmap, label_gc, tri, 7, Nonconvex, CoordModePrevious);
   
}

void fwd_button_hl_pixmap(Pixmap pixmap,
			  unsigned int width, unsigned int height)
{
   XPoint tri[7];
   
   tri[1].x = 0; tri[1].y = 10;
   tri[2].x = 5; tri[2].y = -5;
   tri[3].x = 0; tri[3].y = 5;
   tri[4].x = 5; tri[4].y = -5;
   tri[5].x = -5; tri[5].y = -5;
   tri[6].x = 0; tri[6].y = 5; 

   tri[0].x = width/2 - 6; tri[0].y = height/2 - 6;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 7, Nonconvex, CoordModePrevious);
   tri[0].x = width/2 - 6; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 7, Nonconvex, CoordModePrevious);
   tri[0].x = width/2 - 4; tri[0].y = height/2 - 6;
   XFillPolygon(display, pixmap, bvl_low_gc, tri, 7, Nonconvex, CoordModePrevious);
   tri[0].x = width/2 - 4; tri[0].y = height/2 - 4;
   XFillPolygon(display, pixmap, bvl_high_gc, tri, 7, Nonconvex, CoordModePrevious);

   tri[0].x = width/2 - 5; tri[0].y = height/2 - 5;
   XFillPolygon(display, pixmap, label_hl_gc, tri, 7, Nonconvex, CoordModePrevious);
}

