#ifndef __BUTTON_H__
#define __BUTTON_H__

struct button_data {
   Window window;
   int width, height;
   void (*pressed)(void *a);
   void *arg;
   char *status_text;
   Pixmap unset_pixmap;
   Pixmap set_pixmap;
   Pixmap hl_pixmap;
   Pixmap current_pixmap;
};

struct button_data_list {
   struct button_data *data;
   struct button_data_list *next;
};

int check_buttons_destroyed(XEvent report);
void add_button(struct button_data *b);
struct button_data *create_button(Window parent, int x, int y, int width, int height,
				  void *pressed_func(), void *arg,
				  char *st,
				  Pixmap unset_pixmap, Pixmap set_pixmap,
				  Pixmap hl_pixmap);
struct button_data *create_text_button(Window parent, int x, int y,
				       int width, int height,
				       void *pressed_func(), char *text,
				       char *st);
struct button_data *create_play_button(Window parent, int x, int y,
				       int width, int height,
				       void *pressed_func(), char *st);
struct button_data *create_stop_button(Window parent, int x, int y,
				       int width, int height,
				       void *pressed_func(), char *st);
struct button_data *create_arrow_left_button(Window parent, int x, int y,
					     int width, int height,
					     void *pressed_func(), void *arg,
					     char *st);
struct button_data *create_arrow_right_button(Window parent, int x, int y,
					      int width, int height,
					      void *pressed_func(), void *arg,
					      char *st);
struct button_data *create_arrow_miniup_button(Window parent, int x, int y,
					       int width, int height,
					       void *pressed_func(), void *arg,
					       char *st);
struct button_data *create_arrow_minidown_button(Window parent, int x, int y,
					      int width, int height,
					      void *pressed_func(), void *arg,
					      char *st);
struct button_data *create_arrow_up_button(Window parent, int x, int y,
					   int width, int height,
					   void *pressed_func(), void *arg,
					   char *st);
struct button_data *create_arrow_down_button(Window parent, int x, int y,
					     int width, int height,
					     void *pressed_func(), void *arg,
					     char *st);
struct button_data *create_rew_button(Window parent, int x, int y,
				      int width, int height,
				      void *pressed_func(), char *st);
struct button_data *create_fwd_button(Window parent, int x, int y,
				      int width, int height,
				      void *pressed_func(), char *st);

/********** BUTTON EVENTS **************/
int check_buttons_pressdown(XEvent report);
void refresh_button(struct button_data *button);


Pixmap create_button_pixmap(Window window,
			    unsigned int width, unsigned int height);
Pixmap create_button_set_pixmap(Window window,
				unsigned int width, unsigned int height);
Pixmap create_button_hl_pixmap(Window window,
			    unsigned int width, unsigned int height);

void label_button_pixmap(Pixmap pixmap,
			 unsigned int width, unsigned int height,
			 char *button_text);
void label_button_hl_pixmap(Pixmap pixmap,
			   unsigned int width, unsigned int height,
			   char *button_text);
void play_button_pixmap(Pixmap pixmap,
			unsigned int width, unsigned int height);
void play_button_hl_pixmap(Pixmap pixmap,
			   unsigned int width, unsigned int height);
void stop_button_pixmap(Pixmap pixmap,
			unsigned int width, unsigned int height);
void stop_button_hl_pixmap(Pixmap pixmap,
			unsigned int width, unsigned int height);
void arrow_right_button_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height);
void arrow_right_button_hl_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height);
void arrow_left_button_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height);
void arrow_left_button_hl_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height);
void arrow_miniup_button_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height);
void arrow_miniup_button_hl_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height);
void arrow_minidown_button_pixmap(Pixmap pixmap,
				  unsigned int width, unsigned int height);
void arrow_minidown_button_hl_pixmap(Pixmap pixmap,
				     unsigned int width, unsigned int height);
void arrow_up_button_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height);
void arrow_up_button_hl_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height);
void arrow_down_button_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height);
void arrow_down_button_hl_pixmap(Pixmap pixmap,
			    unsigned int width, unsigned int height);
void rew_button_pixmap(Pixmap pixmap,
		       unsigned int width, unsigned int height);
void rew_button_hl_pixmap(Pixmap pixmap,
		       unsigned int width, unsigned int height);
void fwd_button_pixmap(Pixmap pixmap,
		       unsigned int width, unsigned int height);
void fwd_button_hl_pixmap(Pixmap pixmap,
			  unsigned int width, unsigned int height);


#endif /*  __BUTTON_H__  */
