#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "filerequest.h"


extern char *progname;

extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC top_gc;

static struct filerequest *only_f;

static struct filerequest_list *filerequests;


int check_filerequests_destroyed(XEvent report)
{
   struct filerequest_list *frdl, *pfrdl=NULL;
   
   for(frdl = filerequests; frdl; frdl = frdl->next) {
      if(frdl->data->window == report.xdestroywindow.window) {
	 if(pfrdl) {
	    pfrdl->next = frdl->next;
	 }
	 else {
	    filerequests = frdl->next;
	 }
	 free(frdl->data);
	 free(frdl);
	 return 1;
      }
   }
   return 0;
}

void add_filerequest(struct filerequest *f)
{
   struct filerequest_list *frdl;

   frdl = (struct filerequest_list *)malloc(sizeof(struct filerequest_list));
   frdl->data = f;
   
   if(filerequests == NULL) {
      frdl->next = NULL;
      filerequests = frdl;
   }
   else {
      frdl->next = filerequests;
      filerequests = frdl;
   }
}


void file_ok_button_pressed(void)
{
   close_filerequest_window(only_f);

   only_f->selected(only_f->selected_filename);
}

void file_cancel_button_pressed(void)
{
   close_filerequest_window(only_f);
}

void file_cwd_changed(char *s) {
   change_directory(only_f, s);
}

void file_fname_changed(char *s)
{
   only_f->selected_filename = s;
   
   close_filerequest_window(only_f);
   
   only_f->selected(only_f->selected_filename);
}

void file_selected(int list_index) {
   struct stat st;

   only_f->selected_filename = only_f->filenames[0][list_index];

   if(stat(only_f->selected_filename, &st) < 0) {
      perror(only_f->selected_filename);
      exit(1);
   }
   
   /* if directory ... */
   if((st.st_mode & S_IFMT) == S_IFDIR) {
      change_directory(only_f, only_f->selected_filename);
   }
   else {
      change_textfield_text(only_f->file_fname, only_f->selected_filename);
   }
}

void file_chosen(int list_index) {
   struct stat st;
   
    only_f->selected_filename = only_f->filenames[0][list_index];

   if(stat(only_f->selected_filename, &st) < 0) {
      perror(only_f->selected_filename);
      exit(1);
   }
   
   /* if directory ... */
   if((st.st_mode & S_IFMT) == S_IFDIR) {
      change_directory(only_f, only_f->selected_filename);
   }
   else {
      file_ok_button_pressed();
   }
}

void change_directory(struct filerequest *f, char *s)
{
   strcpy(f->dirname, s);
   
   f->selected_filename = NULL;
   
   f->filenames = get_file_names();
   
   update_textlist(f->file_list,
		   f->num_files, 1,
		   0,
		   f->filenames);
   
   change_textfield_text(f->file_fname, f->selected_filename);
}


void create_filerequest(void *selected_func())
{
   struct filerequest *f;
   char *window_name = "maube load file";
   char *icon_name = "maube load file";
   int width, height;
   
   char ***filens;
   
   width = 400;
   height = 300;
   
   f = (struct filerequest *)malloc(sizeof(struct filerequest));

   /***************************
    *  _                _     *
    * | |__   __ _  ___| | __ *
    * | '_ \ / _  |/ __| |/ / *
    * | | | | (_| | (__|   <  *
    * |_| |_|\__,_|\___|_|\_\ *
    *                         *
    ***************************/
   only_f = f;

   
   f->window = create_toplevel_window(window_name, icon_name,
				      0, 0, width, height, 200, 100);
   f->width = width;
   f->height = height;
   
   f->selected = (void *)selected_func;
   
   f->choice_panel = create_panel(f->window, 4, 18, width-8, 224);
   
   f->file_cwd =
     create_textfield(f->choice_panel->window, 8, 2, width-20, 20,
		      (void *)file_cwd_changed, (char *)"", LEFT_JUSTIFY);
   f->file_fname =
     create_textfield(f->choice_panel->window, 8, 202, width-20, 20,
		      (void *)file_fname_changed, (char *)"", LEFT_JUSTIFY);
   
   strcpy(f->dirname, (char *)".");
   f->dirname_size = 256;
   f->filenames = get_file_names();


   
   f->file_list =
     create_textlist(f->choice_panel->window, 8, 22, width-20, 180,
		     (void *)file_selected, (void *)file_chosen,
		     f->num_files, 1,
		     f->filenames);


   f->file_ok_button = create_text_button(f->window,
				       20, height - 30, 60, 20,
				       (void *)file_ok_button_pressed,
				       (char *)"Oathstix",
				       (char *)"Load it up ya! Now!");
   
   f->file_cancel_button = create_text_button(f->window,
					   300, height-30, 60, 20,
					   (void *)file_cancel_button_pressed,
					   (char *)"Bugger off",
					   (char *)"Nah, fuck it!");
   
   XMapWindow(display, f->window);
   
   add_filerequest(f);
}

char ***get_file_names(void)
{
   int i;
   
   char **filenamesv;
   char ***filenames;
   
   struct string_list *flist, *sl;

   flist = get_file_string_list();
   
   filenamesv = (char **)malloc((size_t)only_f->num_files * sizeof(char *));
   filenames = (char ***)malloc(sizeof(char **));
			
   i=0;
   for(sl = flist; sl; sl = sl->next) {   
      filenamesv[i] = sl->e->string;
      i++;
   }
   
   filenames[0] = filenamesv;
   
   return filenames;
}

struct string_list *get_file_string_list(void)
{
   int i;
   DIR *dp;
   struct dirent *d;
   struct stat st;
   struct string_list *sl, *psl = NULL;
   struct string_list *fname_list;

/* BUGGY!!
   if(fname_list != NULL) {
      free_string_list(fname_list);
   }
*/
   
   chdir(only_f->dirname);
   /* translate .. . /../ etc. properly */
   getcwd(only_f->dirname, only_f->dirname_size);
   change_textfield_text(only_f->file_cwd, only_f->dirname);
/*   change_textfield_text(file_fname, selected_filename);*/
   
   if((dp = opendir(only_f->dirname)) == NULL) {
      perror(only_f->dirname);
      fprintf(stderr, "fuck!! %s\n", only_f->dirname);
      exit(-1);
   }
   
   only_f->num_files = -1;
   
   while((d = readdir(dp)) != NULL) {
      only_f->num_files++;

      sl = (struct string_list *)malloc(sizeof(struct string_list));
      sl->e = (struct string_entry *)malloc(sizeof(struct string_entry));

      if(stat(d->d_name, &st) < 0) {
	 perror(d->d_name);
	 exit(1);
      }
   
      /* if directory ... */
      if((st.st_mode & S_IFMT) == S_IFDIR) {
	 sprintf(sl->e->string, "%s/", d->d_name);
      }
      else {
	 sprintf(sl->e->string, "%s", d->d_name);
      }
      
      if(psl == NULL) {
	 fname_list = sl;
      }
      else {
	 psl->next = sl;
      }
      
      psl = sl;
   }
      
   psl->next = NULL;

   fname_list = sort_file_string_list(fname_list);
   
   return fname_list;
}

struct string_list *sort_file_string_list(struct string_list *sl)
{
   /* use sexy insertion sort */
   struct string_list *ret=NULL, *tsl, *ntsl;
   
   for(tsl=sl; tsl; tsl=ntsl) {
      ntsl=tsl->next;
      ret = insert_fslist(ret, tsl);
   }
   return ret;
}

  
struct string_list *insert_fslist(struct string_list *sl, struct string_list *isl)
{
   struct string_list *tsl, *ptsl=NULL;
   
   for(tsl=sl; tsl; tsl=tsl->next) {
      if(strcmp(tsl->e->string, isl->e->string)>0) break;
      ptsl=tsl;
   }

   if(ptsl) {
      isl->next = ptsl->next;
      ptsl->next = isl;
      return sl;
   }
   else {
      isl->next = sl;
      return isl;
   }
   
}
   
  
void free_string_list(struct string_list *sl)
{
   struct string_list *tsl=sl;
   
   while(tsl != NULL) {
      tsl = sl->next;

      if(sl) {
	 free(sl->e);
	 free(sl);
      }
      
      sl = tsl;
   }
}

void clear_filerequest_window(struct filerequest *f)
{
   XFillRectangle(display, f->window, top_gc, 0, 0, f->width, f->height);
}

void close_filerequest_window(struct filerequest *f)
{
   if(f->window != 0L) {
      XDestroyWindow(display, f->window);
   }
   XFlush(display);
}

