#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "label.h"


extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC small_gc;
extern GC large_gc;
extern XFontStruct *small_font;
extern XFontStruct *large_font;
extern GC bvl_low_gc;
extern GC bvl_high_gc;
extern GC clear_gc;
extern GC hl_gc;

struct label_data_list *labels = NULL;


int check_labels_destroyed(XEvent report)
{
   struct label_data_list *ldl, *pldl=NULL;
   
   for(ldl = labels; ldl; ldl = ldl->next) {
      if(ldl->data->window == report.xdestroywindow.window) {
	 if(pldl) {
	    pldl->next = ldl->next;
	 }
	 else {
	    labels = ldl->next;
	 }
	 free(ldl->data);
	 free(ldl);
	 return 1;
      }
   }
   return 0;
}

void add_label(struct label *l)
{
   struct label_data_list *ldl;

   ldl = (struct label_data_list *)malloc(sizeof(struct label_data_list));
   ldl->data = l;
   
   if(labels == NULL) {
      ldl->next = NULL;
      labels = ldl;
   }
   else {
      ldl->next = labels;
      labels = ldl;
   }
}

struct label *create_label(Window parent, int x, int y, int width, int height,
			   char *init_str)
{
   struct label *l;
   
   l = (struct label *)malloc(sizeof(struct label));
   
   l->window = create_subwindow(parent, x, y, width, height);
   l->width = width;
   l->height = height;
   
   l->text = init_str;
   
   add_label(l);
   
   return(l);
}

int check_labels_expose(XEvent report)
{
   struct label_data_list *ldl;
   
   for(ldl = labels; ldl != NULL; ldl = ldl->next) {
      if(ldl->data->window == report.xexpose.window) {
	 refresh_label(ldl->data);
	 return 1;
      }
   }
   return 0;
}

void refresh_label(struct label *l)
{
   int font_height, xpos, ypos;
   
   font_height = large_font->ascent + large_font->descent;

   xpos = 6;
   ypos = (l->height - font_height)/2 + large_font->ascent;
   
   XClearWindow(display, l->window);

   XDrawLine(display, l->window, bvl_high_gc, 0, 0, l->width-1, 0);
   XDrawLine(display, l->window, bvl_high_gc, 0, 0, 0, l->height-1);
   XDrawLine(display, l->window, bvl_high_gc, 2, 0, 2, l->height-1);
   XDrawLine(display, l->window, bvl_low_gc, 1, 1, l->width-2, 1);
   XDrawLine(display, l->window, bvl_low_gc, 1, 1, 1, l->height-2);
   XDrawLine(display, l->window, bvl_low_gc, 3, 1, 3, l->height-2);

   XDrawLine(display, l->window, bvl_high_gc,
	     l->width-2, 1, l->width-2, l->height-1);
   XDrawLine(display, l->window, bvl_high_gc,
	     1, l->height-2, l->width-1, l->height-2);
   XDrawLine(display, l->window, bvl_low_gc,
	     l->width-1, 0, l->width-1, l->height-1);
   XDrawLine(display, l->window, bvl_low_gc,
	     0, l->height-1, l->width, l->height-1);

   if(l->text != NULL) {   
      XDrawString(display, l->window, large_gc, xpos, ypos,
		  l->text, strlen(l->text));
   }
   
}


void change_label_text(struct label *l, char *new_text)
{
   l->text = new_text;
   refresh_label(l);
}

