#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "numrange.h"

extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC small_gc;
extern GC large_gc;
extern XFontStruct *small_font;
extern XFontStruct *large_font;
extern GC clear_gc;
extern GC hl_gc;


void numrange_down(struct numrange *n) {
   n->val--;

   numrange_bounds(n);
   refresh_numrange(n);
   n->changed(n->val);
}

void numrange_up(struct numrange *n) {
   n->val++;

   numrange_bounds(n);
   refresh_numrange(n);
   n->changed(n->val);
}

void numrange_delta(struct numrange *n, int delta)
{
   n->val += delta;
   
   numrange_bounds(n);
   refresh_numrange(n);
   n->changed(n->val);
}

void set_numrange(struct numrange *n, int val)
{
   n->val = val;
   
   numrange_bounds(n);
   refresh_numrange(n);
}

void set_numrange_bounds(struct numrange *n, int val, int min, int max)
{
   n->min = min;
   n->max = max;
   
   set_numrange(n, val);
}

void numrange_bounds(struct numrange *n) {
   if(n->val < n->min) {
      n->val = n->min;
   }
   else if(n->val > n->max) {
      n->val = n->max;
   }
}

void refresh_numrange(struct numrange *n) {
   char *num_str;
   
   num_str = (char *)malloc(8 * sizeof(char));
   sprintf(num_str, "%d", n->val);

   change_label_text(n->numdisplay, num_str);
}


struct numrange *create_numrange(Window parent, int x, int y,
				 int width, int height,
				 int init_val, int min, int max,
				 void *changed())
{
   struct numrange *nr;
   
   nr = (struct numrange *)malloc(sizeof(struct numrange));

   nr->width = width;
   nr->height = height;
   
   nr->val = init_val;
   nr->min = min; nr->max = max;

   nr->numdisplay = create_label(parent, x, y, (width - 12), height, (char *)"");

   nr->upbutton =
     create_arrow_miniup_button(parent,
				x + width-12, y, 12, (height-2)/2,
				(void *)numrange_up, nr, (char *)"");
   
   nr->handle =
     create_numrange_handle(parent,
			    x + width-12, y+(height/2)-1, 12, 4,
			    (void *)numrange_delta, nr);
   
   nr->downbutton =
     create_arrow_minidown_button(parent,
			       x + width-12, y+(height/2)+1, 12, (height-2)/2,
			       (void *)numrange_down, nr, (char *)"");

   nr->changed = (void *)changed;
   
   refresh_numrange(nr);
   
   return nr;
}

