#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "numrange_handle.h"


extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC small_gc;
extern GC large_gc;
extern XFontStruct *small_font;
extern XFontStruct *large_font;
extern GC bvl_low_gc;
extern GC bvl_high_gc;
extern GC text_hl_gc;
extern GC clear_gc;
extern GC hl_gc;
extern GC label_gc;
extern GC label_hl_gc;

struct numrange_handle_list *numrange_handles;


int check_numrange_handles_destroyed(XEvent report)
{
   struct numrange_handle_list *shdl, *pshdl=NULL;
   
   for(shdl = numrange_handles; shdl; shdl = shdl->next) {
      if(shdl->data->window == report.xdestroywindow.window) {
	 if(pshdl) {
	    pshdl->next = shdl->next;
	 }
	 else {
	    numrange_handles = shdl->next;
	 }
	 free(shdl->data);
	 free(shdl);
	 return 1;
      }
   }
   return 0;
}

void add_numrange_handle(struct numrange_handle *t)
{
   struct numrange_handle_list *shdl;

   shdl = (struct numrange_handle_list *)malloc(sizeof(struct numrange_handle_list));
   shdl->data = t;
   
   if(numrange_handles == NULL) {
      shdl->next = NULL;
      numrange_handles = shdl;
   }
   else {
      shdl->next = numrange_handles;
      numrange_handles = shdl;
   }
}

struct numrange_handle *create_numrange_handle(Window parent, int x, int y,
					   int width, int height,
					   void *moved_func(), void *arg)
{
   struct numrange_handle *t;
   
   t = (struct numrange_handle *)malloc(sizeof(struct numrange_handle));
   
   t->window = create_subwindow(parent, x, y, width, height);
   t->x = 0;
   t->y = 0;
   t->width = width;
   t->height = height;
   t->moved = (void *)moved_func;
   t->arg = arg;

   t->unset_pixmap = create_button_pixmap(parent, width, height);
   t->set_pixmap = create_button_set_pixmap(parent, width, height);
   t->hl_pixmap = t->unset_pixmap;
   t->current_pixmap = t->unset_pixmap;
   
   add_numrange_handle(t);
   
   return(t);
}

int check_numrange_handles_pressdown(XEvent report)
{
   struct numrange_handle_list *shdl;
   struct numrange_handle *s;
   
   for(shdl = numrange_handles; shdl != NULL; shdl = shdl->next) {
      if(shdl->data->window == report.xbutton.window) {
	 s = shdl->data;
	 s->my = report.xbutton.y;
	 
	 s->current_pixmap = s->set_pixmap;
	 refresh_numrange_handle(s);
	 
	 return 1;
      }
   }
   return 0;
}

int check_numrange_handles_pressed(XEvent report)
{
   struct numrange_handle_list *shdl;
   
   for(shdl = numrange_handles; shdl != NULL; shdl = shdl->next) {
      if(shdl->data->window == report.xbutton.window) {
	 shdl->data->current_pixmap = shdl->data->unset_pixmap;
	 refresh_numrange_handle(shdl->data);
	 return 1;
      }
   }
   return 0;
}

int check_numrange_handles_motion(XEvent report)
{
   struct numrange_handle_list *shdl;
   struct numrange_handle *s;
   
   for(shdl = numrange_handles; shdl != NULL; shdl = shdl->next) {
      if(shdl->data->window == report.xbutton.window) {
	 s = shdl->data;
	 
	 s->moved(s->arg, s->my - report.xbutton.y);
	 s->my = report.xbutton.y;
	 
	 return 1;
      }
   }
   return 0;
}

int check_numrange_handles_mouseover(XEvent report)
{
   struct numrange_handle_list *shdl;
   
   for(shdl = numrange_handles; shdl != NULL; shdl = shdl->next) {
      if(shdl->data->window == report.xcrossing.window) {
	 shdl->data->current_pixmap = shdl->data->hl_pixmap;
	 refresh_numrange_handle(shdl->data);
	 return 1;
      }
   }
   return 0;
}

int check_numrange_handles_mouseexit(XEvent report)
{
   struct numrange_handle_list *shdl;
   
   for(shdl = numrange_handles; shdl != NULL; shdl = shdl->next) {
      if(shdl->data->window == report.xcrossing.window) {
	 shdl->data->current_pixmap = shdl->data->unset_pixmap;
	 refresh_numrange_handle(shdl->data);
	 return 1;
      }
   }
   return 0;
}

int check_numrange_handles_expose(XEvent report)
{
   struct numrange_handle_list *shdl;
   
   for(shdl = numrange_handles; shdl != NULL; shdl = shdl->next) {
      if(shdl->data->window == report.xexpose.window) {
	 refresh_numrange_handle(shdl->data);
	 return 1;
      }
   }
   return 0;
}

void refresh_numrange_handle(struct numrange_handle *numrange_handle)
{
   if(numrange_handle->current_pixmap == 0L) {
      return;
   }

   XCopyArea(display, numrange_handle->current_pixmap, numrange_handle->window, large_gc,
	     0, 0,
	     numrange_handle->width, numrange_handle->height,
	     0, 0);
}

