#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "panel.h"


extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC small_gc;
extern GC large_gc;
extern XFontStruct *small_font;
extern XFontStruct *large_font;
extern GC bvl_low_gc;
extern GC bvl_high_gc;
extern GC text_hl_gc;
extern GC clear_gc;
extern GC hl_gc;
extern GC label_gc;
extern GC label_hl_gc;
extern GC handle_gc;

struct panel_list *panels;


int check_panels_destroyed(XEvent report)
{
   struct panel_list *bdl, *pbdl=NULL;
   
   for(bdl = panels; bdl; bdl = bdl->next) {
      if(bdl->data->window == report.xdestroywindow.window) {
	 if(pbdl) {
	    pbdl->next = bdl->next;
	 }
	 else {
	    panels = bdl->next;
	 }
	 free(bdl->data);
	 free(bdl);
	 return 1;
      }
   }
   return 0;
}

void add_panel(struct panel *b)
{
   struct panel_list *bdl;

   bdl = (struct panel_list *)malloc(sizeof(struct panel_list));
   bdl->data = b;
   
   if(panels == NULL) {
      bdl->next = NULL;
      panels = bdl;
   }
   else {
      bdl->next = panels;
      panels = bdl;
   }
}
   
struct panel *create_panel(Window parent, int x, int y, int width, int height)
{
   struct panel *b;
   
   b = (struct panel *)malloc(sizeof(struct panel));
   
   b->window = create_subwindow(parent, x, y, width, height);
   b->width = width;
   b->height = height;

   b->pixmap = create_panel_pixmap(parent, width, height);
   
   add_panel(b);
   
   return(b);
}

int check_panels_pressdown(XEvent report)
{
   struct panel_list *bdl;
   struct panel *p;
   
   for(bdl = panels; bdl != NULL; bdl = bdl->next) {
      if(bdl->data->window == report.xbutton.window) {
	 p = bdl->data;
	 p->mx = report.xbutton.x;
	 p->my = report.xbutton.y;
	 
	 XRaiseWindow(display, p->window);
/*	 refresh_panel(bdl->data);*/
	 return 1;
      }
   }
   return 0;
}

int check_panels_pressed(XEvent report)
{
   struct panel_list *bdl;
   
   for(bdl = panels; bdl != NULL; bdl = bdl->next) {
      if(bdl->data->window == report.xbutton.window) {
/*	 refresh_panel(bdl->data);*/
	 return 1;
      }
   }
   return 0;
}

int check_panels_motion(XEvent report)
{
   struct panel_list *bdl;
   struct panel *p;
   
   for(bdl = panels; bdl != NULL; bdl = bdl->next) {
      if(bdl->data->window == report.xbutton.window) {
	 p = bdl->data;
	 p->x += report.xbutton.x - p->mx;
	 p->y += report.xbutton.y - p->my;
	 XMoveWindow(display, p->window, p->x, p->y);
	 /*	 refresh_panel(bdl->data);*/
	 return 1;
      }
   }
   return 0;
}  

int check_panels_mouseover(XEvent report)
{
   struct panel_list *bdl;
   
   for(bdl = panels; bdl != NULL; bdl = bdl->next) {
      if(bdl->data->window == report.xcrossing.window) {
/*	 refresh_panel(bdl->data);*/
	 return 1;
      }
   }
   return 0;
}

int check_panels_mouseexit(XEvent report)
{
   struct panel_list *bdl;
   
   for(bdl = panels; bdl != NULL; bdl = bdl->next) {
      if(bdl->data->window == report.xcrossing.window) {
/*	 refresh_panel(bdl->data);*/
	 return 1;
      }
   }
   return 0;
}

int check_panels_expose(XEvent report)
{
   struct panel_list *bdl;
   
   for(bdl = panels; bdl != NULL; bdl = bdl->next) {
      if(bdl->data->window == report.xexpose.window) {
	 refresh_panel(bdl->data);
	 return 1;
      }
   }
   return 0;
}

void refresh_panel(struct panel *panel)
{
   if(panel->pixmap == 0L) {
      return;
   }

   XCopyArea(display, panel->pixmap, panel->window, large_gc,
	     0, 0,
	     panel->width, panel->height,
	     0, 0);
}

Pixmap create_panel_pixmap(Window window,
			   unsigned int width, unsigned int height)
{
   Pixmap pixmap;

   pixmap = XCreatePixmap(display, window, width, height,
			  display_depth);
   
   XFillRectangle(display, pixmap, handle_gc, 1, 1, width-2, height-2);

   XDrawLine(display, pixmap, bvl_high_gc, 0, 0, width, 0);
   XDrawLine(display, pixmap, bvl_high_gc, 0, 0, 0, height);
   XDrawLine(display, pixmap, bvl_low_gc, 1, 1, width-2, 1);
   XDrawLine(display, pixmap, bvl_low_gc, 1, 1, 1, height-2);

   XDrawLine(display, pixmap, bvl_high_gc, width-1, 1, width-1, height-1);
   XDrawLine(display, pixmap, bvl_high_gc, 1, height-1, width-1, height-1);
   XDrawLine(display, pixmap, bvl_low_gc, width, 0, width, height);
   XDrawLine(display, pixmap, bvl_low_gc, 0, height, width, height);
   
   return(pixmap);
}

