#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "scroll_handle.h"


extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC small_gc;
extern GC large_gc;
extern XFontStruct *small_font;
extern XFontStruct *large_font;
extern GC bvl_low_gc;
extern GC bvl_high_gc;
extern GC text_hl_gc;
extern GC clear_gc;
extern GC hl_gc;
extern GC label_gc;
extern GC label_hl_gc;

struct scroll_handle_list *scroll_handles;


int check_scroll_handles_destroyed(XEvent report)
{
   struct scroll_handle_list *shdl, *pshdl=NULL;
   
   for(shdl = scroll_handles; shdl; shdl = shdl->next) {
      if(shdl->data->window == report.xdestroywindow.window) {
	 if(pshdl) {
	    pshdl->next = shdl->next;
	 }
	 else {
	    scroll_handles = shdl->next;
	 }
	 free(shdl->data);
	 free(shdl);
	 return 1;
      }
   }
   return 0;
}

void add_scroll_handle(struct scroll_handle *t)
{
   struct scroll_handle_list *shdl;

   shdl = (struct scroll_handle_list *)malloc(sizeof(struct scroll_handle_list));
   shdl->data = t;
   
   if(scroll_handles == NULL) {
      shdl->next = NULL;
      scroll_handles = shdl;
   }
   else {
      shdl->next = scroll_handles;
      scroll_handles = shdl;
   }
}

struct scroll_handle *create_scroll_handle(Window parent, int x, int y,
					   int width, int height,
					   int min_y, int max_y,
					   void *moved_func(), void *arg)
{
   struct scroll_handle *t;
   
   t = (struct scroll_handle *)malloc(sizeof(struct scroll_handle));
   
   t->window = create_subwindow(parent, x, y, width, height);
   t->x = x;
   t->y = y;
   t->width = width;
   t->height = height;
   t->min_y = min_y;
   t->max_y = max_y;
   t->moved = (void *)moved_func;
   t->arg = arg;

   t->unset_pixmap = create_button_pixmap(parent, width, height);
   t->set_pixmap = create_button_set_pixmap(parent, width, height);
   t->hl_pixmap = create_button_hl_pixmap(parent, width, height);
   t->current_pixmap = t->unset_pixmap;
   
   add_scroll_handle(t);
   
   return(t);
}

void move_scroll_handle(struct scroll_handle *s, int y)
{
   s->y = s->min_y + y;

   XMoveWindow(display, s->window, s->x, s->y);
}  

void move_resize_scroll_handle(struct scroll_handle *s, int y, int height)
{
   s->y = s->min_y + y;
   s->height = height;

   XFreePixmap(display, s->unset_pixmap);
   XFreePixmap(display, s->set_pixmap);
   
   s->unset_pixmap = create_button_pixmap(s->window, s->width, s->height);
   s->set_pixmap = create_button_set_pixmap(s->window, s->width, s->height);
   s->hl_pixmap = create_button_hl_pixmap(s->window, s->width, s->height);
   s->current_pixmap = s->unset_pixmap;
   
   XMoveResizeWindow(display, s->window, s->x, s->y, s->width, s->height);
}  

/*
void set_scroll_handle_bounds(struct scroll_handle *s, int min_y, int max_y)
   s->min_y = min_y;
   s->max_y = max_y - s->height;
}
*/

int check_scroll_handles_pressdown(XEvent report)
{
   struct scroll_handle_list *shdl;
   struct scroll_handle *s;
   
   for(shdl = scroll_handles; shdl != NULL; shdl = shdl->next) {
      if(shdl->data->window == report.xbutton.window) {
	 s = shdl->data;
/*	 s->mx = report.xbutton.x;*/
	 s->my = report.xbutton.y;
/*	 s->x = s->mx;*/
/*	 s->y = s->my;*/
	 
	 s->current_pixmap = s->set_pixmap;
	 refresh_scroll_handle(s);
	 
	 return 1;
      }
   }
   return 0;
}

int check_scroll_handles_pressed(XEvent report)
{
   struct scroll_handle_list *shdl;
   
   for(shdl = scroll_handles; shdl != NULL; shdl = shdl->next) {
      if(shdl->data->window == report.xbutton.window) {
	 shdl->data->current_pixmap = shdl->data->unset_pixmap;
	 refresh_scroll_handle(shdl->data);
	 return 1;
      }
   }
   return 0;
}

int check_scroll_handles_motion(XEvent report)
{
   struct scroll_handle_list *shdl;
   struct scroll_handle *s;
   
   for(shdl = scroll_handles; shdl != NULL; shdl = shdl->next) {
      if(shdl->data->window == report.xbutton.window) {
	 s = shdl->data;
/*	 s->x += report.xbutton.x - s->mx;*/
	 s->y += report.xbutton.y - s->my;

	 if(s->y < s->min_y) {
	    s->y = s->min_y;
	 }
	 else if(s->y > s->max_y - s->height) {
	    s->y = s->max_y - s->height;
	 }

	 XMoveWindow(display, s->window, s->x, s->y);
	 
	 s->moved(s->arg, s->y - s->min_y);
	 
	 return 1;
      }
   }
   return 0;
}

int check_scroll_handles_mouseover(XEvent report)
{
   struct scroll_handle_list *shdl;
   
   for(shdl = scroll_handles; shdl != NULL; shdl = shdl->next) {
      if(shdl->data->window == report.xcrossing.window) {
	 shdl->data->current_pixmap = shdl->data->hl_pixmap;
	 refresh_scroll_handle(shdl->data);
	 return 1;
      }
   }
   return 0;
}

int check_scroll_handles_mouseexit(XEvent report)
{
   struct scroll_handle_list *shdl;
   
   for(shdl = scroll_handles; shdl != NULL; shdl = shdl->next) {
      if(shdl->data->window == report.xcrossing.window) {
	 shdl->data->current_pixmap = shdl->data->unset_pixmap;
	 refresh_scroll_handle(shdl->data);
	 return 1;
      }
   }
   return 0;
}

int check_scroll_handles_expose(XEvent report)
{
   struct scroll_handle_list *shdl;
   
   for(shdl = scroll_handles; shdl != NULL; shdl = shdl->next) {
      if(shdl->data->window == report.xexpose.window) {
	 refresh_scroll_handle(shdl->data);
	 return 1;
      }
   }
   return 0;
}

void refresh_scroll_handle(struct scroll_handle *scroll_handle)
{
   if(scroll_handle->current_pixmap == 0L) {
      return;
   }

   XCopyArea(display, scroll_handle->current_pixmap, scroll_handle->window, large_gc,
	     0, 0,
	     scroll_handle->width, scroll_handle->height,
	     0, 0);
}

