#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

#include "textfield.h"


extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC small_gc;
extern GC large_gc;
extern XFontStruct *small_font;
extern XFontStruct *large_font;
extern GC cursor_gc;
extern GC bvl_low_gc;
extern GC bvl_high_gc;
extern GC clear_gc;
extern GC hl_gc;

struct textfield_data_list *textfields;


int check_textfields_destroyed(XEvent report)
{
   struct textfield_data_list *tdl, *ptdl=NULL;
   
   for(tdl = textfields; tdl; tdl = tdl->next) {
      if(tdl->data->window == report.xdestroywindow.window) {
	 if(ptdl) {
	    ptdl->next = tdl->next;
	 }
	 else {
	    textfields = tdl->next;
	 }
	 free(tdl->data);
	 free(tdl);
	 return 1;
      }
   }
   return 0;
}


struct textfield *create_textfield(Window parent, int x, int y, int width, int height,
		   void *changed_func(), char *init_str, int justify)
{
   struct textfield_data_list *tdl;
   
   tdl = (struct textfield_data_list *)malloc(sizeof(struct textfield_data_list));
   tdl->data = (struct textfield *)malloc(sizeof(struct textfield));
   
   tdl->data->window = create_subwindow(parent, x, y, width, height);
   tdl->data->width = width;
   tdl->data->height = height;
   tdl->data->changed = (void *)changed_func;

   strcpy(tdl->data->text, init_str);
   
   tdl->data->justify = justify;
   tdl->data->cursor = CURSOR_OFF;
   
   if(textfields == NULL) {
      tdl->next = NULL;
      textfields = tdl;
   }
   else {
      tdl->next = textfields;
      textfields = tdl;
   }
   
   return(tdl->data);
}

int place_cursor(struct textfield *t, int x)
{
   int i;
   
   for(i=0; i < strlen(t->text); i++) {
      if((XTextWidth(large_font, t->text, i+1) + 4) > x) {
	 return i;
      }
   }
   
   return strlen(t->text);
}

int check_textfields_pressed(XEvent report)
{
   struct textfield_data_list *tdl;
   
   for(tdl = textfields; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->window == report.xbutton.window) {
	 tdl->data->cursor = place_cursor(tdl->data, report.xbutton.x); /* place_cursor() */
	 refresh_textfield(tdl->data);
	 return 1;
      }
   }
   return 0;
}


int check_textfields_mouseover(XEvent report)
{
   struct textfield_data_list *tdl;
   
   for(tdl = textfields; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->window == report.xcrossing.window) {
	 /*
	 set_status_text(tdl->data->status_text);
	  */
	 return 1;
      }
   }
   return 0;
}

int check_textfields_mouseexit(XEvent report)
{
   struct textfield_data_list *tdl;
   
   for(tdl = textfields; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->window == report.xcrossing.window) {
	 if(tdl->data->cursor != CURSOR_OFF) {   
	    tdl->data->cursor = CURSOR_OFF;
	    refresh_textfield(tdl->data);
	    tdl->data->changed(tdl->data->text);
	 }
	 
	 return 1;
      }
   }
   return 0;
}

int check_textfields_expose(XEvent report)
{
   struct textfield_data_list *tdl;
   
   for(tdl = textfields; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->window == report.xexpose.window) {
	 refresh_textfield(tdl->data);
	 return 1;
      }
   }
   return 0;
}

int check_textfields_keypress(XEvent report)
{
   struct textfield_data_list *tdl;
   KeySym keysym;
   unsigned int state;
   int count;
   XComposeStatus compose;
   char buffer[80];
   int bufsize=80;

   
   for(tdl = textfields; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->window == report.xkey.window) {
	 if(tdl->data->cursor == CURSOR_OFF) {
	    return 1;
	 }
	 
	 state = report.xkey.state;
	 count = XLookupString((XKeyEvent *)(&report), buffer, bufsize, &keysym, &compose);

	 if(state & ControlMask) {
	    switch(keysym) {
	     case XK_a:
	     case XK_A:
	       tdl->data->cursor = 0;
	       break;
	     case XK_e:
	     case XK_E:
	       tdl->data->cursor = strlen(tdl->data->text);
	       break;
	     default:
	       break;
	    }
	    refresh_textfield(tdl->data);
	 }
	 else {
	    switch(keysym) {

	     case XK_Left:
	       cursor_left(tdl->data);
	       break;
	     case XK_Right:
	       cursor_right(tdl->data);
	       break;
	     case XK_Home:
	       tdl->data->cursor = 0;
	       break;
	     case XK_End:
	       tdl->data->cursor = strlen(tdl->data->text);
	       break;
	     case XK_BackSpace:
	       delete_prev_char(tdl->data);
	       break;
	     case XK_Delete:
	       delete_next_char(tdl->data);
	       break;
	     case XK_Return:
	       tdl->data->cursor = CURSOR_OFF;
	       tdl->data->changed(tdl->data->text);
	       break;
	     case XK_Control_L:
	     case XK_Control_R:
	     case XK_Shift_L:
	     case XK_Shift_R:
	       break;
	     case XK_space:
	     case XK_Tab:
	       insert_char(tdl->data, (char *)" ");
	       break;
	     case XK_exclam:
	       insert_char(tdl->data, (char *)"!");
	       break;
	     case XK_quotedbl:
	       insert_char(tdl->data, (char *)"\"");
	       break;
	     case XK_numbersign:
	       insert_char(tdl->data, (char *)"#");
	       break;
	     case XK_dollar:
	       insert_char(tdl->data, (char *)"$");
	       break;
	     case XK_percent:
	       insert_char(tdl->data, (char *)"%");
	       break;
	     case XK_ampersand:
	       insert_char(tdl->data, (char *)"&");
	       break;
	     case XK_apostrophe:
	       insert_char(tdl->data, (char *)"\'");
	       break;
	     case XK_parenleft:
	       insert_char(tdl->data, (char *)"(");
	       break;
	     case XK_parenright:
	       insert_char(tdl->data, (char *)")");
	       break;
	     case XK_asterisk:
	       insert_char(tdl->data, (char *)"*");
	       break;
	     case XK_plus:
	       insert_char(tdl->data, (char *)"+");
	       break;
	     case XK_comma:
	       insert_char(tdl->data, (char *)",");
	       break;
	     case XK_minus:
	       insert_char(tdl->data, (char *)"-");
	       break;
	     case XK_period:
	       insert_char(tdl->data, (char *)".");
	       break;
	     case XK_slash:
	       insert_char(tdl->data, (char *)"/");
	       break;
	     case XK_colon:
	       insert_char(tdl->data, (char *)":");
	       break;
	     case XK_semicolon:
	       insert_char(tdl->data, (char *)";");
	       break;
	     case XK_less:
	       insert_char(tdl->data, (char *)"<");
	       break;
	     case XK_equal:
	       insert_char(tdl->data, (char *)"=");
	       break;
	     case XK_greater:
	       insert_char(tdl->data, (char *)">");
	       break;
	     case XK_question:
	       insert_char(tdl->data, (char *)"?");
	       break;
	     case XK_at:
	       insert_char(tdl->data, (char *)"@");
	       break;
	     case XK_bracketleft:
	       insert_char(tdl->data, (char *)"[");
	       break;
	     case XK_bracketright:
	       insert_char(tdl->data, (char *)"]");
	       break;
	     case XK_asciicircum:
	       insert_char(tdl->data, (char *)"^");
	       break;
	     case XK_underscore:
	       insert_char(tdl->data, (char *)"_");
	       break;
	     case XK_grave:
	       insert_char(tdl->data, (char *)"`");
	       break;
	     case XK_braceleft:
	       insert_char(tdl->data, (char *)"{");
	       break;
	     case XK_bar:
	       insert_char(tdl->data, (char *)"|");
	       break;
	     case XK_braceright:
	       insert_char(tdl->data, (char *)"}");
	       break;
	     case XK_asciitilde:
	       insert_char(tdl->data, (char *)"~");
	       break;
	     default:
	       insert_char(tdl->data, XKeysymToString(keysym));
	       break;
	       
	    }
	    
	    refresh_textfield(tdl->data);
	    return 1;
	 }
      }
   }

      
   return 0;
}


void refresh_textfield(struct textfield *t)
{
   int font_height;
   int text_width;
   int xpos, ypos;
   int cursor_x;
   
   font_height = large_font->ascent + large_font->descent;

   XClearWindow(display, t->window);

   XDrawLine(display, t->window, bvl_high_gc, 0, 0, t->width-1, 0);
   XDrawLine(display, t->window, bvl_high_gc, 0, 0, 0, t->height-1);
   XDrawLine(display, t->window, bvl_low_gc, 1, 1, t->width-2, 1);
   XDrawLine(display, t->window, bvl_low_gc, 1, 1, 1, t->height-2);

   XDrawLine(display, t->window, bvl_high_gc, t->width-2, 1, t->width-2, t->height-1);
   XDrawLine(display, t->window, bvl_high_gc, 1, t->height-2, t->width-1, t->height-2);
   XDrawLine(display, t->window, bvl_low_gc, t->width-1, 0, t->width-1, t->height-1);
   XDrawLine(display, t->window, bvl_low_gc, 0, t->height-1, t->width, t->height-1);

   
   if(t->text != NULL) {   
      text_width = XTextWidth(large_font, t->text, strlen(t->text));
      
      ypos = (t->height - font_height)/2 + large_font->ascent;
      
      switch(t->justify) {
       case LEFT_JUSTIFY:
	 xpos = 4;
	 break;
       case RIGHT_JUSTIFY:
	 xpos = t->width - 4 - text_width;
	 break;
       case CENTRE_JUSTIFY:
	 xpos = (t->width - text_width)/2;
	 break;
       default:
	 break;
      }
      XDrawString(display, t->window, large_gc,
		  xpos, ypos,
		  t->text, strlen(t->text));
   }
   

   if(t->cursor != CURSOR_OFF) {
      text_width = XTextWidth(large_font, t->text, t->cursor);
      switch(t->justify) {
       case LEFT_JUSTIFY:
	 cursor_x = text_width + 4;
	 break;
       case RIGHT_JUSTIFY:
	 /* wrong */
	 cursor_x = t->width - 4 - text_width;
	 break;
       case CENTRE_JUSTIFY:
	 /* not sure */
	 cursor_x = text_width + (t->width - text_width)/2;
      }	 
      XDrawLine(display, t->window, cursor_gc, cursor_x, 2, cursor_x, t->height-2);
   }
   
}

void cursor_left(struct textfield *t)
{
   if(t->cursor <= 0) {
      t->cursor = 0;
      return;
   }
   
   t->cursor--;
}

void cursor_right(struct textfield *t)
{
   if(t->cursor >= strlen(t->text)) {
      t->cursor = strlen(t->text);
      return;
   }
   
   t->cursor++;
}

void insert_char(struct textfield *t, char *c)
{
   int i;
   
   for(i=strlen(t->text); i > t->cursor; i--) {
      t->text[i] = t->text[i-1];
   }
   
   t->text[t->cursor] = *c;
   cursor_right(t);
}

void delete_next_char(struct textfield *t)
{
   delete_char(t, t->cursor);
}

void delete_prev_char(struct textfield *t)
{
   delete_char(t, --t->cursor);
   cursor_left(t);
}

void delete_char(struct textfield *t, int pos)
{
   int i;
   
   if(i<0) return;
   
   for(i=pos; i < strlen(t->text); i++) {
      t->text[i] = t->text[i+1];
   }
   for(;i < 256; i++) {   
      t->text[i] = '\0';
   }
   
}

void change_textfield_text(struct textfield *t, char *new_text)
{
   int i;
   
   for(i=0;i < 256; i++) {   
      t->text[i] = '\0';
   }
   if(new_text)
     strcpy(t->text, new_text);
   else
     strcpy(t->text, (char *)"");

   refresh_textfield(t);
}

