#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "textlist.h"
#include "scrollbar.h"

extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC large_gc;
extern XFontStruct *large_font;
extern GC bvl_low_gc;
extern GC bvl_high_gc;
extern GC clear_gc;
extern GC hl_gc;
extern GC cursor_gc;

struct textlist_list *textlists;

void scroll_textlist(struct textlist *t, int new_top_row)
{
   t->top_row = new_top_row;
   
   refresh_textlist(t);
}

int check_textlists_destroyed(XEvent report)
{
   struct textlist_list *tdl, *ptdl=NULL;
   
   for(tdl = textlists; tdl; tdl = tdl->next) {
      if(tdl->data->window == report.xdestroywindow.window) {
	 if(ptdl) {
	    ptdl->next = tdl->next;
	 }
	 else {
	    textlists = tdl->next;
	 }
	 free(tdl->data);
	 free(tdl);
	 return 1;
      }
   }
   return 0;
}

void add_textlist(struct textlist *t)
{
   struct textlist_list *tdl;

   tdl = (struct textlist_list *)malloc(sizeof(struct textlist_list));
   tdl->data = t;
   
   if(textlists == NULL) {
      tdl->next = NULL;
      textlists = tdl;
   }
   else {
      tdl->next = textlists;
      textlists = tdl;
   }
}


struct textlist *create_textlist(Window parent, int x, int y, int width, int height,
				 void *changed(), void *chosen(),
				 int rows, int cols, char ***cells)
{
   struct textlist *t;
   int i, j;
   int font_height;

   font_height = large_font->ascent + large_font->descent;

   t = (struct textlist *)malloc(sizeof(struct textlist));
   
   t->window = create_subwindow(parent, x, y, width, height);
   t->listwin = create_subwindow(t->window, 0, 0, width-12, height);

   t->x = x;
   t->y = y;
   t->width = width-12;
   t->height = height;

   t->rows_visible =
     (int)((t->height)/(font_height + 2));

   t->scrollbar = create_scrollbar(t->window, width-12, 0, 12, height,
				   0, 0, rows, t->rows_visible,
				   (void *)scroll_textlist, t);
				   
   t->changed = (void *)changed;
   t->chosen = (void *)chosen;
   
   t->rows = rows;
   t->cols = cols;
   
   t->curr_row = 0;
   t->top_row = 0;

   t->time = 0;
   
   t->cells = cells;
   
   t->columns = (struct textcolumn **)malloc((size_t)cols * sizeof(struct textcolumn *));

   if(t->rows) {   
      render_column_pixmaps(t);
   }
   
   
   t->back_pixmap = XCreatePixmap(display, t->listwin, t->width, t->height,
				  display_depth);

   refresh_textlist(t);

   add_textlist(t);
   
   return t;
}

/*
 * update_textlist
 * 
 * completely replace cells, and reset rows, cols appropriately.
 */
void update_textlist(struct textlist *t, int rows, int cols, int curr_row, char ***cells)
{
   int i, j;
   /*
   for(i=0; i<t->cols; i++) {   
      free(t->cells[i]);
   }
   free(t->cells);
   */

   if(t->window == 0L) {
      return;
   }
   
   
   t->rows = rows;
   t->cols = cols;
   
   t->cells = cells;
   t->curr_row = curr_row;
   t->top_row = 0;
   
   t->columns = (struct textcolumn **)malloc((size_t)cols * sizeof(struct textcolumn *));

   if(t->rows) {
      render_column_pixmaps(t);
   }
   
  /*   
   t->back_pixmap = XCreatePixmap(display, t->listwin, t->width, t->height,
				  display_depth);
   */   


   
   set_scrollbar_bounds(t->scrollbar, t->curr_row, 0, t->rows, t->rows_visible);
   
   refresh_textlist(t);
}


void render_column_pixmaps(struct textlist *t)
{
   int i;
   
   for(i=0; i < t->cols; i++) {
/*      if(t->columns[i] == NULL) {*/
      t->columns[i] = (struct textcolumn *)malloc(sizeof(struct textcolumn));
/*      }*/
/*      if(t->columns[i]->pixmap) {
	 XFreePixmap(display, t->columns[i]->pixmap);
      }
*/      
      t->columns[i]->pixmap = render_column_pixmap(t, i);
      t->columns[i]->x = i * 60;
   }
}

Pixmap render_column_pixmap(struct textlist *t, int i)
{
   int font_height;
   int xpos = 0, ypos = 0;
   int p_width, p_height;
   int j;
   Pixmap pixmap;
   
   font_height = large_font->ascent + large_font->descent;

   p_height = (t->rows) * (font_height + 2);
   
   p_width = 200;

   pixmap = XCreatePixmap(display, t->listwin, p_width, p_height,
			  display_depth);

   XFillRectangle(display, pixmap, clear_gc, 0, 0, p_width, p_height);
   
   xpos = 4;
   for(j=0; j < t->rows; j++) {
      XDrawString(display, pixmap, large_gc, xpos, ypos + font_height,
		  t->cells[i][j], strlen(t->cells[i][j]));

      ypos += font_height + 2;
   }
   
   
   t->columns[i]->width = p_width;
   
   return(pixmap);
}
   
void refresh_textlist(struct textlist *t)
{
   render_textlist_back_pixmap(t);
   render_textlist(t);
}

void render_textlist_back_pixmap(struct textlist *t)
{
   int font_height;
   int i, j;
   int xpos, ypos;

   font_height = large_font->ascent + large_font->descent;

   t->rows_visible =
          (int)((t->height)/(font_height + 2));

   /*
   if(t->curr_row < (t->rows_visible/2)) {
      t->top_row = 1;
   }
   else if(t->curr_row > (t->rows - (t->rows_visible/2))) {
      t->top_row = t->rows - (t->rows_visible - 1);
   }
   else {
      t->top_row = t->curr_row -
	(t->rows_visible/2 - 1);
   }
   */

   XFillRectangle(display, t->back_pixmap, clear_gc, 0, 0, t->width, t->height);

   /* draw bevelled left edge */
   
   XDrawLine(display, t->back_pixmap, bvl_high_gc, 0, 0, 0, t->height);
   XDrawLine(display, t->back_pixmap, bvl_low_gc, 2, 0, 2, t->height);
   
   /* draw column pixmaps */
   if(t->rows) {
      for(i=0; i<t->cols; i++) {   
	 XCopyArea(display, t->columns[i]->pixmap, t->back_pixmap, large_gc,
		   0, (t->top_row) * (font_height + 2),
		   t->columns[i]->width, t->height,
		   3 + t->columns[i]->x, 0);
      }
   }

}

void render_textlist(struct textlist *t)
{
   int font_height;

   font_height = large_font->ascent + large_font->descent;
   
   XCopyArea(display, t->back_pixmap, t->listwin, large_gc, 0, 0,
	     t->width, t->height, 0, 0);
   
   /* draw highlight */
   if(t->rows) {   
      XDrawRectangle(display, t->listwin, cursor_gc,
		     3,
		     (t->curr_row - t->top_row) *
		     (font_height + 2),
		     t->width-4, (font_height + 2));
   }
   
   /* and flush the display */
   XSync(display, False);

}

void select_item(struct textlist *t, int x, int y)
{
   int font_height;
   int old_row=t->curr_row;
   
   font_height = large_font->ascent + large_font->descent;
   
   t->curr_row = (int)(y/(font_height + 2)) + t->top_row;

   if(t->curr_row != old_row) {
      render_textlist(t);   
      t->changed(t->curr_row);
      t->time=0;
   }
   
}

void change_selection(struct textlist *t, int sel)
{   
   t->curr_row = sel;
   
   render_textlist(t);  
}

int check_textlists_pressdown(XEvent report)
{
   struct textlist_list *tdl;
   
   for(tdl = textlists; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->listwin == report.xbutton.window) {
/*	 select_item(tdl->data, report.xbutton.x, report.xbutton.y);*/
/*	 refresh_textlist(tdl->data);*/
	 
	 return 1;
      }
   }
   return 0;
}

int check_textlists_pressed(XEvent report)
{
   struct textlist_list *tdl;
   
   for(tdl = textlists; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->listwin == report.xbutton.window) {
	 select_item(tdl->data, report.xbutton.x, report.xbutton.y);
	 if((report.xbutton.time - tdl->data->time)<DOUBLECLICK_TIME) {
	    tdl->data->chosen(tdl->data->curr_row);
	 }
	 tdl->data->time = report.xbutton.time;
	 return 1;
      }
   }
   return 0;
}

int check_textlists_motion(XEvent report)
{
   struct textlist_list *tdl;
   
   for(tdl = textlists; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->listwin == report.xbutton.window) {
	 select_item(tdl->data, report.xbutton.x, report.xbutton.y);
	 return 1;
      }
   }

   return 0;
}
   

int check_textlists_mouseover(XEvent report)
{
   struct textlist_list *tdl;
   
   for(tdl = textlists; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->listwin == report.xcrossing.window) {

/*	 refresh_textlist(tdl->data);*/
	 return 1;
      }
   }
   return 0;
}

int check_textlists_mouseexit(XEvent report)
{
   struct textlist_list *tdl;
   
   for(tdl = textlists; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->listwin == report.xcrossing.window) {

/*	 refresh_textlist(tdl->data);*/
	 return 1;
      }
   }
   return 0;
}

int check_textlists_expose(XEvent report)
{
   struct textlist_list *tdl;
   
   for(tdl = textlists; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->listwin == report.xexpose.window) {
	 refresh_textlist(tdl->data);
	 return 1;
      }
   }
   return 0;
}


