#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include "toggle.h"


extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern GC small_gc;
extern GC large_gc;
extern XFontStruct *small_font;
extern XFontStruct *large_font;
extern GC bvl_low_gc;
extern GC bvl_high_gc;
extern GC text_hl_gc;
extern GC clear_gc;
extern GC hl_gc;
extern GC label_gc;
extern GC label_hl_gc;

struct toggle_data_list *toggles;


int check_toggles_destroyed(XEvent report)
{
   struct toggle_data_list *tdl, *ptdl=NULL;
   
   for(tdl = toggles; tdl; tdl = tdl->next) {
      if(tdl->data->window == report.xdestroywindow.window) {
	 if(ptdl) {
	    ptdl->next = tdl->next;
	 }
	 else {
	    toggles = tdl->next;
	 }
	 free(tdl->data);
	 free(tdl);
	 return 1;
      }
   }
   return 0;
}

void add_toggle(struct toggle_data *t)
{
   struct toggle_data_list *tdl;

   tdl = (struct toggle_data_list *)malloc(sizeof(struct toggle_data_list));
   tdl->data = t;
   
   if(toggles == NULL) {
      tdl->next = NULL;
      toggles = tdl;
   }
   else {
      tdl->next = toggles;
      toggles = tdl;
   }
}
   
struct toggle_data *create_toggle(Window parent, int x, int y, int width, int height,
				  void *pressed_func(), void *released_func(),
				  char *st,
				  Pixmap unset_pixmap, Pixmap set_pixmap,
				  Pixmap hl_pixmap)
{
   struct toggle_data *t;
   
   t = (struct toggle_data *)malloc(sizeof(struct toggle_data));
   
   t->window = create_subwindow(parent, x, y, width, height);
   t->width = width;
   t->height = height;
   t->pressed = (void *)pressed_func;
   t->released = (void *)released_func;
   t->state = 0;
   
   t->status_text = st;
   
   t->unset_pixmap = unset_pixmap;
   t->set_pixmap = set_pixmap;
   t->hl_pixmap = hl_pixmap;
   t->current_pixmap = t->unset_pixmap;
   
   add_toggle(t);
   
   return(t);
}

struct toggle_data *create_text_toggle(Window parent, int x, int y,
				       int width, int height,
				       void *pressed_func(),
				       void *released_func(),
				       char *text,
				       char *st)
{		
   Pixmap unset_pixmap, set_pixmap, hl_pixmap;
    
   unset_pixmap = create_button_pixmap(parent, width, height);
   label_button_pixmap(unset_pixmap, width, height, text);
   
   set_pixmap = create_button_set_pixmap(parent, width, height);
   label_button_hl_pixmap(set_pixmap, width, height, text);
   
   hl_pixmap = create_button_hl_pixmap(parent, width , height);
   label_button_hl_pixmap(hl_pixmap, width, height, text);
   
   return(create_toggle(parent, x, y, width, height,
			pressed_func, released_func, st,
			unset_pixmap, set_pixmap, hl_pixmap));
}

struct toggle_data *create_play_toggle(Window parent, int x, int y,
				       int width, int height,
				       void *pressed_func(), 
				       void *released_func(), char *st)
{		
   Pixmap unset_pixmap, set_pixmap, hl_pixmap;
    
   unset_pixmap = create_button_pixmap(parent, width, height);
   play_button_pixmap(unset_pixmap, width, height);
   
   set_pixmap = create_button_set_pixmap(parent, width, height);
   play_button_hl_pixmap(set_pixmap, width, height);
   
   hl_pixmap = create_button_hl_pixmap(parent, width , height);
   play_button_hl_pixmap(hl_pixmap, width, height);
   
   return(create_toggle(parent, x, y, width, height,
			pressed_func, released_func, st,
			unset_pixmap, set_pixmap, hl_pixmap));
}

int check_toggles_pressdown(XEvent report)
{
   struct toggle_data_list *tdl;
   
   for(tdl = toggles; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->window == report.xbutton.window) {
	 if(!tdl->data->state) {
	    tdl->data->current_pixmap = tdl->data->set_pixmap;
	    refresh_toggle(tdl->data);
	 }
	 else {
	    tdl->data->current_pixmap = tdl->data->unset_pixmap;
	    refresh_toggle(tdl->data);
	 }
	 return 1;
      }
   }
   return 0;
}

int check_toggles_pressed(XEvent report)
{
   struct toggle_data_list *tdl;
   
   for(tdl = toggles; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->window == report.xbutton.window) {
	 toggle_toggle(tdl->data);

	 return 1;
      }
   }
   return 0;
}

int check_toggles_mouseover(XEvent report)
{
   struct toggle_data_list *tdl;
   
   for(tdl = toggles; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->window == report.xcrossing.window) {
	 if(!tdl->data->state) {
	    tdl->data->current_pixmap = tdl->data->hl_pixmap;
	    refresh_toggle(tdl->data);
	 }
	 /*	 else {
	    tdl->data->current_pixmap = tdl->data->unset_pixmap;
	    refresh_toggle(tdl->data);
	 }
	  */
	 set_status_text(tdl->data->status_text);
	 return 1;
      }
   }
   return 0;
}

int check_toggles_mouseexit(XEvent report)
{
   struct toggle_data_list *tdl;
   
   for(tdl = toggles; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->window == report.xcrossing.window) {
	 if(tdl->data->state) {
	    tdl->data->current_pixmap = tdl->data->set_pixmap;
	    refresh_toggle(tdl->data);
	 }
	 else {
	    tdl->data->current_pixmap = tdl->data->unset_pixmap;
	    refresh_toggle(tdl->data);
	 }
	 return 1;
      }
   }
   return 0;
}

int check_toggles_expose(XEvent report)
{
   struct toggle_data_list *tdl;
   
   for(tdl = toggles; tdl != NULL; tdl = tdl->next) {
      if(tdl->data->window == report.xexpose.window) {
	 refresh_toggle(tdl->data);
	 return 1;
      }
   }
   return 0;
}

void toggle_toggle(struct toggle_data *toggle)
{
   toggle->state = 1 - toggle->state;
   refresh_toggle_state(toggle);
}

void set_toggle_state(struct toggle_data *toggle, int newstate)
{
   toggle->state = newstate;
   refresh_toggle_state(toggle);
}

void refresh_toggle_state(struct toggle_data *toggle)
{
   if(toggle->state) {
      toggle->current_pixmap = toggle->set_pixmap;
      refresh_toggle(toggle);
      toggle->pressed();
   }
   else {
      toggle->current_pixmap = toggle->unset_pixmap;
      refresh_toggle(toggle);
      toggle->released();
   }   
}

void refresh_toggle(struct toggle_data *toggle)
{
   if(toggle->current_pixmap == 0L) {
      return;
   }

   XCopyArea(display, toggle->current_pixmap, toggle->window, large_gc,
	     0, 0,
	     toggle->width, toggle->height,
	     0, 0);
}

