#include <stdio.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

extern char *progname;

extern Display *display;
extern unsigned int display_depth;
extern int screen_num;

extern XColor top_bg;
extern XColor subwin_bg;
extern XColor fg;
extern XColor hl;

Window create_toplevel_window(char *window_name, char *icon_name,
			      int x, int y, unsigned int width,
			      unsigned int height,
			      unsigned int min_width,
			      unsigned int min_height)
{
   unsigned int border_width = 2;
   Pixmap icon_pixmap;
   XIconSize *size_list;
   XWMHints *wm_hints;
   XClassHint *class_hints;
   XSizeHints *size_hints;
   XTextProperty windowName, iconName;
   Window ret_window;

   
   if (!(size_hints = XAllocSizeHints())) {
      fprintf(stderr, "%s: failure allocating memory\n", progname);
      exit(0);
   }

   if (!(wm_hints = XAllocWMHints())) {
      fprintf(stderr, "%s: failure allocating memory\n", progname);
      exit(0);
   }

   if (!(class_hints = XAllocClassHint())) {
      fprintf(stderr, "%s: failure allocating memory\n", progname);
      exit(0);
   }
   
   
   ret_window = XCreateSimpleWindow(display, RootWindow(display, screen_num),
				    x, y, width, height, border_width,
				    fg.pixel, top_bg.pixel);

   size_hints->flags = PPosition | PSize | PMinSize;
   size_hints->min_width = min_width;
   size_hints->min_height = min_height;
   
   if(XStringListToTextProperty(&window_name, 1, &windowName) == 0) {
      (void)fprintf(stderr, "%s: structure allocation for windowName failed.\n",
		    progname);
      exit(-1);
   }

   if(XStringListToTextProperty(&icon_name, 1, &iconName) == 0) {
      (void)fprintf(stderr, "%s: structure allocation for iconName failed.\n",
		    progname);
      exit(-1);
   }

   wm_hints->initial_state = NormalState;
   wm_hints->input = True;
   wm_hints->flags = StateHint | IconPixmapHint | InputHint;
   
   class_hints->res_name = progname;
   class_hints->res_class = "Maube";

   /*
   XSetWMProperties(display, ret_window, &windowName, &iconName,
		    argv, argc, size_hints, wm_hints, class_hints);
   */
   XSetWMName(display, ret_window, &windowName);
   
   XSelectInput(display, ret_window,
		ExposureMask | KeyPressMask |

/*		EnterWindowMask | LeaveWindowMask | 
*/
		StructureNotifyMask);

   
   return(ret_window);
}

Window create_subwindow(Window parent, int x, int y, int width, int height)
{
   Window ret_window;
   
   ret_window = XCreateSimpleWindow(display, parent,
				    x, y, width, height,
				    0,
				    fg.pixel, subwin_bg.pixel);
   
   XSelectInput(display, ret_window,
/*		ButtonPressMask | ButtonMotionMask |
*/
		ButtonPressMask | ButtonReleaseMask | ButtonMotionMask |
		ExposureMask | KeyPressMask |
		EnterWindowMask | LeaveWindowMask);
   
   XMapWindow(display, ret_window);
   
   return(ret_window);
}

void change_window_title(Window w, char *window_name)
{
   XTextProperty windowName;
   
   if(XStringListToTextProperty(&window_name, 1, &windowName) == 0) {
      (void)fprintf(stderr, "%s: structure allocation for windowName failed.\n",
		    progname);
      exit(-1);
   }
   
   XSetWMName(display, w, &windowName);
}

